#ifndef HistoBook_H
#define HistoBook_H
/**
 * @file HistoBook.h
 * Description of cpp file
 *
 * @brief A brief description
 *
 * @author Jakub Cuth <Jakub.Cuth@cern.ch>
 * @date 2016-08-29
 */

#include "HistoHandler.h"
#include "KinematicDefinitions.h"
#include "settings.h"
#include "user_book.h"

using namespace Kinematics;
namespace HistoHandler{
    void Book() {
        DeleteHists();
        Add( new Histo1D        <BosPT           > ("qt"          ) );
	Add( new Histo1D        <BosY            > ("y"          ) );
	Add( new Histo1D        <BosM            > ("m"          ) );
        Add( new Histo1D        <LepPT           > ("ptl"          ) );
        Add( new Histo1D        <LepEta          > ("etal"          ) );
        Add( new Histo2D        <BosPT,BosY      > ("qt","y"      ) );
        Add( new Histo3D        <BosPT,BosY,BosM > ("qt","y","m"  ) );
        Add( new HistoProfile   <BosPT,A1        > ("qt","a1"     ) );
        Add( new HistoProfile2D <BosPT,BosY,A1   > ("qt","y","a1" ) );

	// book a set of optional default histograms
	if (opts.hetal)
	  if (opts.nproc == 1)
	    Add( new Histo1D        <ALpAbsEta          > ("etalep"          ) )->SetName("eta_lep");
	  else if (opts.nproc == 2)
	    Add( new Histo1D        <LepAbsEta          > ("etalep"          ) )->SetName("eta_lep");

	if (opts.hptl)
	  if (opts.nproc == 1)
	    Add( new Histo1D        <ALpPT           > ("ptlep"          ) )->SetName("pt_lep");
	  else if (opts.nproc == 2)
	    Add( new Histo1D        <LepPT           > ("ptlep"          ) )->SetName("pt_lep");

	if (opts.hptnu)
	  if (opts.nproc == 1)
	    Add( new Histo1D        <LepPT           > ("ptnu"          ) )->SetName("pt_nu");
	  else if (opts.nproc == 2)
	    Add( new Histo1D        <ALpPT           > ("ptnu"          ) )->SetName("pt_nu");
	
	if (opts.hmt)
	  Add( new Histo1D        <BosMT           > ("mt"          ) )->SetName("mt");

	//if (opts.hphistareta) //to be implemented
	//Add( new Histo1D        <BosPhist           > ("phistareta"          ) )->SetName("phistareta");

	//if (opts.haiqt) //to be implemented
	//{
	//  Add( new HistoWeighted <BosPT,A0>("qt","a0") )->SetName("a0_qt");
	//  Add( new HistoWeighted <BosPT,A1>("qt","a1") )->SetName("a1_qt");
	//  Add( new HistoWeighted <BosPT,A2>("qt","a2") )->SetName("a2_qt");
	//  Add( new HistoWeighted <BosPT,A3>("qt","a3") )->SetName("a3_qt");
	//  Add( new HistoWeighted <BosPT,A4>("qt","a4") )->SetName("a4_qt");
	//  Add( new HistoWeighted <BosPT,A5>("qt","a5") )->SetName("a5_qt");
	//  Add( new HistoWeighted <BosPT,A6>("qt","a6") )->SetName("a6_qt");
	//  Add( new HistoWeighted <BosPT,A7>("qt","a7") )->SetName("a7_qt");
	//  Add( new Histo1D       <BosPT   >("qt"     ) )->SetName("xs_qt");
	//}

	//if (opts.haiy) //to be implemented
	//{
	//  Add( new HistoWeighted <BosY,A0>("y","a0") )->SetName("a0_y");
	//  Add( new HistoWeighted <BosY,A1>("y","a1") )->SetName("a1_y");
	//  Add( new HistoWeighted <BosY,A2>("y","a2") )->SetName("a2_y");
	//  Add( new HistoWeighted <BosY,A3>("y","a3") )->SetName("a3_y");
	//  Add( new HistoWeighted <BosY,A4>("y","a4") )->SetName("a4_y");
	//  Add( new HistoWeighted <BosY,A5>("y","a5") )->SetName("a5_y");
	//  Add( new HistoWeighted <BosY,A6>("y","a6") )->SetName("a6_y");
	//  Add( new HistoWeighted <BosY,A7>("y","a7") )->SetName("a7_y");
	//  Add( new Histo1D       <BosY   >("y"     ) )->SetName("xs_y");
	//}

	//if (opts.haim) //to be implemented
	//{
	//  Add( new HistoWeighted <BosM,A0>("m","a0") )->SetName("a0_m");
	//  Add( new HistoWeighted <BosM,A1>("m","a1") )->SetName("a1_m");
	//  Add( new HistoWeighted <BosM,A2>("m","a2") )->SetName("a2_m");
	//  Add( new HistoWeighted <BosM,A3>("m","a3") )->SetName("a3_m");
	//  Add( new HistoWeighted <BosM,A4>("m","a4") )->SetName("a4_m");
	//  Add( new HistoWeighted <BosM,A5>("m","a5") )->SetName("a5_m");
	//  Add( new HistoWeighted <BosM,A6>("m","a6") )->SetName("a6_m");
	//  Add( new HistoWeighted <BosM,A7>("m","a7") )->SetName("a7_m");
	//  Add( new Histo1D       <BosM   >("m"     ) )->SetName("xs_m");
	//}
	
        // book user histograms
        UserBook();
    }
}

#endif /* ifndef HistoBook_H */
