#include "btilde.h"
#include "settings.h"
#include "resconst.h"
#include "phasespace.h"
#include "gaussrules.h"
#include "besselint.h"
//#include "TMath.h"

#include <iostream>

using namespace std;

const int btilde::p = 4;

double x1 = 0.3;
double x2 = 0.6;
double x3 = 0.9;

int fact(int n)
{
  int p = 1;
  for (int i = 1; i <= n; i++)
    p *= i;
  return p;
}

double int1mj0ox(double bq)
{
  //Compute int_0^bq {dx/x * (1-J0(x))}
  int qrule = 100;
  double res = 0.;

  //quadrule loop
  double cc = 0.5*(bq);
  double mm = 0.5*(bq);
  for (int k = 0; k < qrule; k++)
    {
      double x = cc+mm*gr::xxx[qrule-1][k];
      double fac = gr::www[qrule-1][k]*mm;
      res += (1-fort_besj0_(x))/x * fac;
    }
  
  return res;
}

complex <double> btilde::mubtl(complex <double> b, double Q)
{
  complex <double> mub = resconst::b0/b;
  complex <double> mubtilde;
  if (opts.modlog == 0)
    mubtilde = mub;
  else if (opts.modlog == 1)
    mubtilde = mub * Q / sqrt((pow(mub,2) + pow(Q,2)));
  else if (opts.modlog == 2)
    mubtilde = mub * Q / pow((pow(mub,2*btilde::p) + pow(Q,2*btilde::p)),0.5/btilde::p);
  else if (opts.modlog == 3) //Eq. (2.35) of https://arxiv.org/pdf/1912.07550.pdf
    if (Q/real(mub) < 1e-4)
      mubtilde = Q;
    else
      mubtilde = mub * pow(1.-exp(-pow(Q/mub,p)),1./p);
  //profile scales Eq. (3.11)-(3.13) of https://arxiv.org/pdf/2006.11382.pdf
  else if (opts.modlog == 4)
    {
      double x = phasespace::qt/Q;
      complex <double> y = mub/Q;
      double g;
      if (x < x1)
	g = 1.;
      else if (x < x2)
	g = 1. - pow(x-x1,2)/((x2-x1)*(x3-x1));
      else if (x < x3)
	g = pow(x-x3,2)/((x3-x1)*(x3-x2));
      else
	g = 0.;
      
      //cout << x << "  " << x3 << "  " << g << endl;
      mubtilde = Q*(1.+g*(y-1.));
    }
  //Eq. (22) of https://arxiv.org/pdf/hep-ph/9704239.pdf
  else if (opts.modlog == 5)
    {
      double bq = real(b)*Q;
      double jint = 0.;
      /*
	int m = 1;
	double err = 1.;
	cout << endl;
	cout << b << "  " << Q << "  " << bq << endl;
	while (fabs(err) > 1e-20)
	{
	//err = -pow(-1,m)/(2.*double(m)*pow(TMath::Gamma(double(m+1.)),2))*pow(bq/2.,2*m);
	//m++;
	err = 1./(2.*double(m)*pow(TMath::Gamma(double(m+1.)),2))*pow(bq/2.,2*m+1)/(double(m+1));
	m += 2;
	jint += err;
	cout << "jint " << jint << "  " << err << endl;
	}
      */
      jint = int1mj0ox(bq);
      //cout << "jint int1mj0ox " << jint << endl;
      mubtilde = Q*exp(-jint);
    }
  //cout << b << "  " << mub << "  " << mubtilde << endl;
  return mubtilde;
}

complex <double> btilde::calc(complex <double> b, double Q)
{
  complex <double> mubtilde = mubtl(b,Q);
  return resconst::b0/mubtilde;
}

complex <double> btilde::blog(complex <double> b, double Q)
{
  if (opts.modlog == 0)
    return 2.*log(Q*b/resconst::b0);
  
  complex <double> blog = 2.*log(Q/mubtl(b,Q));
  //if (opts.modlog && real(blog) < 0.)
  //blog = max(0.,real(blog));

  //cout << " Q " << Q << "  mubtl " << mubtl(b,Q) << " blog " << blog << "  " << " qt " << phasespace::qt << endl;
  
  return blog;
}
