#ifndef hell_h
#define hell_h

#include "mellinint.h"
#include "evoltrunc.h"
#include "hell-x.hh"

#include <complex>

using namespace std;

namespace hell
{
  extern int nf;
  extern HELLx::Order order;
  extern HELLx::LogOrder logorder;
  extern double *alphas;
  extern int nas;
  extern map <int,HELLx::xTableP*> xT;
  extern HELLx::HELLxnf *sxtable;
  extern int dim;

  double beta0(int nf);
  double beta1(int nf);
  double beta2(int nf);
  double beta3(int nf);
  
  inline int index(int i, int beam, int sign)  {return i + mellinint::mdim*((beam-1) + 2*sign);};
  inline int index(int i, int sign)            {return i + dim*sign;};
  
  extern void init();
  extern void allocate();
  extern void update();
  extern void evolve();
  extern void free();
  extern void release();

  extern void allocateusg();
  extern void calcusg(double asi, complex <double> asf, int order);
  extern void freeusg();
  extern cmatrix *usg;
#pragma omp threadprivate(usg)
  
  //Decomposed PDFs at the starting scale
  extern complex <double> *gli;
  extern complex <double> *vai;
  extern complex <double> *m3i;
  extern complex <double> *m8i;
  extern complex <double> *m15i;
  extern complex <double> *m24i;
  extern complex <double> *sgi;
  extern complex <double> *p3i;
  extern complex <double> *p8i;
  extern complex <double> *p15i;
  extern complex <double> *p24i;
  extern complex <double> *cxn;
#pragma omp threadprivate(gli,vai,m3i,m8i,m15i,m24i,sgi,p3i,p8i,p15i,p24i,cxn)

  //alphas tables at x knots
  extern double *xdpgg;
  extern double *xdpqg;
  extern double *xdpgg_1;
  extern double *xdpqg_1;
  extern double *xdpgg_2;
  extern double *xdpqg_2;
#pragma omp threadprivate(xdpgg,xdpqg,xdpgg_1,xdpqg_1,xdpgg_2,xdpqg_2)
  
  void smallx(double x, double as, double fsx[4]);
  void smallx_update(double x, double* xdPgg, double *xdPqg);
  void smallx_eval(double x, int i, double as, double fsx[4]);
  void smallx_eval_1(double x, int i, double as, double fsx[4]);
  void smallx_eval_2(double x, int i, double as, double fsx[4]);
}

#endif
