#include "muf.h"
#include "resint.h"
#include "pmom.h"
#include "mesq.h"
#include "settings.h"
#include "ccoeff.h"
#include "resconst.h"
#include "hcoeff.h"
#include "anomalous.h"

using namespace pmom;
using namespace resconst;
using namespace ccoeff;

void muf::ccalc2d()
{
  LR  = resint::LR;
  LR2 = pow(LR,2);
  LR3 = pow(LR,3);

  LF  = resint::LF;
  LF2 = pow(LF,2);
  LF3 = pow(LF,3);
  LF4 = pow(LF,4);

  LQ  = resint::LQ;
  LQ2 = pow(LQ,2);
  LQ3 = pow(LQ,3);
  LQ4 = pow(LQ,4);
  LQ5 = pow(LQ,5);
  LQ6 = pow(LQ,6);
  LQ7 = pow(LQ,7);

  LQF  = LF - LQ;
  LQF2 = pow(LQF,2);
  LQF3 = pow(LQF,3);
  LQF4 = pow(LQF,4);

  double as = resint::aass;
  double as2 = pow(as,2);
  double as3 = pow(as,3);
  double as4 = pow(as,4);

  A1 = A1q;
  A2 = A2q;
  A3 = A3q;
  A4 = A4q;

  B1 = B1q;
  B2 = B2q;
  B3 = B3q;
  B4 = B4q;
  
  complex <double> *H1st_qqb_qqb  = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H1st_qg_1_qqb = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H1st_qg_2_qqb = new complex <double> [mellinint::mdim*mellinint::mdim*2];

  //NLO
  if (max(opts.order_hcoef,opts.order_evol) >= 1) //use order_evol to match the order of the PDF evolution from Q to b0/b, could use order_sudak instead
    {
      if (opts.order_hcoef < 1)
	{
	  LR  = 0;
	  LR2 = 0;
	  LR3 = 0;
	  LF  = 0;
	  LF2 = 0;
	  LF3 = 0;
	  LF4 = 0;
	  LQF  = -LQ;
	  LQF2 = pow(LQF,2);
	  LQF3 = pow(LQF,3);
	  LQF4 = pow(LQF,4);
	}
	  
      for (int sign = mesq::positive; sign <= mesq::negative; sign++)
	for (int i1 = 0; i1 < mellinint::mdim; i1++)
	  for (int i2 = 0; i2 < mellinint::mdim; i2++)
	    {
	      int ii1 = anomalous::index(i1,mesq::positive);
	      int ii2 = anomalous::index(i2,sign);
	      int idx = hcoeff::index(i1,i2,sign);

	      H1st_qqb_qqb[idx]  = (gamma1qq_1[ii1] + gamma1qq_2[ii2])*LQF ;
	      H1st_qg_1_qqb[idx] = gamma1qg_1[ii1]*LQF;
	      H1st_qg_2_qqb[idx] = gamma1qg_2[ii2]*LQF;
	      qqb_eqqb[idx]   += as*H1st_qqb_qqb[idx];
	      qg_1_eqqb[idx]  += as*H1st_qg_1_qqb[idx];
	      qg_2_eqqb[idx]  += as*H1st_qg_2_qqb[idx];
	    }
    }


  //NNLO
  complex <double> *H2st_qqb_qqb     = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qqb_qg_1    = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qqb_qg_2    = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qg_1_qqb    = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qg_1_qg_1   = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qg_2_qqb    = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qg_2_qg_2   = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qq_1_qqb    = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qq_1_qg_1   = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qq_2_qqb    = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qq_2_qg_2   = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qqp_1_qqb   = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qqp_1_qg_1  = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qqp_2_qqb   = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qqp_2_qg_2  = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qqbp_1_qqb  = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qqbp_1_qg_1 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qqbp_2_qqb  = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_qqbp_2_qg_2 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_gg_qqb      = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_gg_qg_1     = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H2st_gg_qg_2     = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  if (max(opts.order_hcoef,opts.order_evol) >= 2)
    {
      if (opts.order_hcoef < 2)
	{
	  LR  = 0;
	  LR2 = 0;
	  LR3 = 0;
	  LF  = 0;
	  LF2 = 0;
	  LF3 = 0;
	  LF4 = 0;
	  LQF  = -LQ;
	  LQF2 = pow(LQF,2);
	  LQF3 = pow(LQF,3);
	  LQF4 = pow(LQF,4);
	}
	  
      for (int sign = mesq::positive; sign <= mesq::negative; sign++)
	for (int i1 = 0; i1 < mellinint::mdim; i1++)
	  for (int i2 = 0; i2 < mellinint::mdim; i2++)
	    {
	      int ii1 = anomalous::index(i1,mesq::positive);
	      int ii2 = anomalous::index(i2,sign);
	      int idx = hcoeff::index(i1,i2,sign);

	      //Include exponentiation in the C coefficients
	      complex <double> eC1qq_1   = ccoeff::C1qq_1[ii1];
	      complex <double> eC1qg_1   = ccoeff::C1qg_1[ii1];
	      complex <double> eC1qq_2   = ccoeff::C1qq_2[ii2];
	      complex <double> eC1qg_2   = ccoeff::C1qg_2[ii2];
	  
	      H2st_qqb_qqb     [idx] = LQF*(eC1qq_2*gamma1qq_1[ii1] + eC1qq_2*gamma1qq_2[ii2] + eC1qq_1*(gamma1qq_1[ii1] + gamma1qq_2[ii2]) + gamma2qq_1[ii1] + gamma2qq_2[ii2] + ((gamma1gq_1[ii1]*gamma1qg_1[ii1] + gamma1gq_2[ii2]*gamma1qg_2[ii2] + (gamma1qq_1[ii1] + gamma1qq_2[ii2])*(beta0 + gamma1qq_1[ii1] + gamma1qq_2[ii2]))*LQF)/ 2. - ((gamma1qq_1[ii1] + gamma1qq_2[ii2])*LQ*(2.*B1q - 2.*beta0 + A1q*LQ))/2.);
	      H2st_qqb_qg_1    [idx] = LQF*(eC1qg_1*gamma1gq_1[ii1]);
	      H2st_qqb_qg_2    [idx] = LQF*(eC1qg_2*gamma1gq_2[ii2]);

	      H2st_qg_1_qqb    [idx] = (LQF*(2.*gamma2qg_1[ii1] + gamma1qg_1[ii1]*(2.*eC1qq_2 + 2.*eC1qq_1 + (beta0 + 2.*gamma1qq_2[ii2] + gamma1gg_1[ii1] + gamma1qq_1[ii1])*LF - (2.*B1q - beta0 + 2.*gamma1qq_2[ii2] + gamma1gg_1[ii1] + gamma1qq_1[ii1])*LQ - A1q*LQ2)))/2.;
	      H2st_qg_1_qg_1   [idx] = (LQF*(2.*eC1qg_1*(gamma1qq_2[ii2] + gamma1gg_1[ii1])))/2.;

	      H2st_qg_2_qqb    [idx] = (LQF*(2.*gamma2qg_2[ii2] + gamma1qg_2[ii2]*(2.*eC1qq_1 + 2.*eC1qq_2 + (beta0 + 2.*gamma1qq_1[ii1] + gamma1gg_2[ii2] + gamma1qq_2[ii2])*LF - (2.*B1q - beta0 + 2.*gamma1qq_1[ii1] + gamma1gg_2[ii2] + gamma1qq_2[ii2])*LQ - A1q*LQ2)))/2.;
	      H2st_qg_2_qg_2   [idx] = (LQF*(2.*eC1qg_2*(gamma1qq_1[ii1] + gamma1gg_2[ii2])))/2.;

	      H2st_qq_1_qqb    [idx] = ((2.*gamma2qqb_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	      H2st_qq_1_qg_1   [idx] = ((2.*eC1qg_1*gamma1gq_1[ii1])* LQF)/2.;

	      H2st_qq_2_qqb    [idx] = ((2.*gamma2qqb_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	      H2st_qq_2_qg_2   [idx] = ((2.*eC1qg_2*gamma1gq_2[ii2])* LQF)/2.;

	      H2st_qqp_1_qqb   [idx] = ((2.*gamma2qqbp_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	      H2st_qqp_1_qg_1  [idx] = ((2.*eC1qg_1*gamma1gq_1[ii1])* LQF)/2.;

	      H2st_qqp_2_qqb   [idx] = ((2.*gamma2qqbp_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	      H2st_qqp_2_qg_2  [idx] = ((2.*eC1qg_2*gamma1gq_2[ii2])* LQF)/2.;

	      H2st_qqbp_1_qqb  [idx] = ((2.*gamma2qqp_1[ii1] + gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	      H2st_qqbp_1_qg_1 [idx] = ((2.*eC1qg_1*gamma1gq_1[ii1])* LQF)/2.;

	      H2st_qqbp_2_qqb  [idx] = ((2.*gamma2qqp_2[ii2] + gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	      H2st_qqbp_2_qg_2 [idx] = ((2.*eC1qg_2*gamma1gq_2[ii2])* LQF)/2.;

	      H2st_gg_qqb      [idx] = (gamma1qg_2[ii2]*gamma1qg_1[ii1]*LQF)*LQF;
	      H2st_gg_qg_1     [idx] = (eC1qg_1*gamma1qg_2[ii2])*LQF;
	      H2st_gg_qg_2     [idx] = (eC1qg_2*gamma1qg_1[ii1])*LQF;

	      //Mixed LR*LQF variations
	      H2st_qqb_qqb  [idx] += - beta0*H1st_qqb_qqb[idx]*LR;
	      H2st_qg_1_qqb [idx] += - beta0*H1st_qg_1_qqb[idx]*LR;
	      H2st_qg_2_qqb [idx] += - beta0*H1st_qg_2_qqb[idx]*LR;
	    
	      qqb_eqqb[idx]    += as2*H2st_qqb_qqb[idx];
	      qg_1_eqqb[idx]   += as2*H2st_qg_1_qqb[idx];
	      qg_2_eqqb[idx]   += as2*H2st_qg_2_qqb[idx];
	      qq_1_eqqb[idx]   += as2*H2st_qq_1_qqb[idx];
	      qq_2_eqqb[idx]   += as2*H2st_qq_2_qqb[idx];
	      qqp_1_eqqb[idx]  += as2*H2st_qqp_1_qqb[idx];
	      qqp_2_eqqb[idx]  += as2*H2st_qqp_2_qqb[idx];
	      qqbp_1_eqqb[idx] += as2*H2st_qqbp_1_qqb[idx];
	      qqbp_2_eqqb[idx] += as2*H2st_qqbp_2_qqb[idx];
	      gg_eqqb[idx]     += as2*H2st_gg_qqb[idx];

	      qqb_eqg_1[idx]    += as2*H2st_qqb_qg_1[idx];
	      qg_1_eqg_1[idx]   += as2*H2st_qg_1_qg_1[idx];
	      qq_1_eqg_1[idx]   += as2*H2st_qq_1_qg_1[idx];
	      qqp_1_eqg_1[idx]  += as2*H2st_qqp_1_qg_1[idx];
	      qqbp_1_eqg_1[idx] += as2*H2st_qqbp_1_qg_1[idx];
	      gg_eqg_1[idx]     += as2*H2st_gg_qg_1[idx];

	      qqb_eqg_2[idx]    += as2*H2st_qqb_qg_2[idx];
	      qg_2_eqg_2[idx]   += as2*H2st_qg_2_qg_2[idx];
	      qq_2_eqg_2[idx]   += as2*H2st_qq_2_qg_2[idx];
	      qqp_2_eqg_2[idx]  += as2*H2st_qqp_2_qg_2[idx];
	      qqbp_2_eqg_2[idx] += as2*H2st_qqbp_2_qg_2[idx];
	      gg_eqg_2[idx]     += as2*H2st_gg_qg_2[idx];
	    }
    }

	  
  //NNNLO
  complex <double> *H3st_qqb_qqb         = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqb_qg_1 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqb_qg_2 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qq_1_qqb   	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qq_1_qg_1  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qq_1_qq_1  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqp_1_qqb  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqp_1_qg_1 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqp_1_qqp_1	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqbp_1_qqb 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqbp_1_qg_1     = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqbp_1_qqbp_1   = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_1_qqb  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_1_gg   	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_1_qg_1  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_1_qg_2  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_1_qq_1  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_1_qqbp_1	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_1_qqp_1 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qq_2_qqb   	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qq_2_qg_2  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qq_2_qq_2  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqp_2_qqb  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqp_2_qg_2      = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqp_2_qqp_2     = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqbp_2_qqb 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqbp_2_qg_2     = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qqbp_2_qqbp_2   = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_2_qqb  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_2_gg   	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_2_qg_2  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_2_qg_1  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_2_qq_2  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_2_qqbp_2	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qg_2_qqp_2 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_gg_qqb    	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_gg_qg_1   	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_gg_qg_2   	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_gg_gg     	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbg_1_qqb 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbg_1_qg_1	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbg_1_qg_2	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbg_1_qq_2	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbg_1_gg  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbg_2_qqb 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbg_2_qg_2	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbg_2_qg_1	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbg_2_qq_1	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbg_2_gg  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qpg_1_qqb  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qpg_1_qg_1      = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qpg_1_qg_2      = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qpg_1_qqbp_2    = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qpg_1_gg  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qpg_2_qqb  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qpg_2_qg_2      = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qpg_2_qg_1      = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qpg_2_qqbp_1    = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qpg_2_gg  	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbpg_1_qqb 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbpg_1_qg_1     = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbpg_1_qg_2     = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbpg_1_qqp_2    = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbpg_1_gg 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbpg_2_qqb 	 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbpg_2_qg_2     = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbpg_2_qg_1     = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbpg_2_qqp_1    = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  complex <double> *H3st_qbpg_2_gg       = new complex <double>  [mellinint::mdim*mellinint::mdim*2];
  if (max(opts.order_hcoef,opts.order_evol) >= 3)
    {
      if (opts.order_hcoef < 3)
	{
	  LR  = 0;
	  LR2 = 0;
	  LR3 = 0;
	  LF  = 0;
	  LF2 = 0;
	  LF3 = 0;
	  LF4 = 0;
	  LQF  = -LQ;
	  LQF2 = pow(LQF,2);
	  LQF3 = pow(LQF,3);
	  LQF4 = pow(LQF,4);
	}
	  
      for (int sign = mesq::positive; sign <= mesq::negative; sign++)
	for (int i1 = 0; i1 < mellinint::mdim; i1++)
	  for (int i2 = 0; i2 < mellinint::mdim; i2++)
	    {
	      int ii1 = anomalous::index(i1,mesq::positive);
	      int ii2 = anomalous::index(i2,sign);
	      int idx = hcoeff::index(i1,i2,sign);

	      //Include exponentiation in the C coefficients
	      complex <double> eC1qq_1   = ccoeff::C1qq_1[ii1];
	      complex <double> eC2qq_1   = ccoeff::C2qq_1[ii1];
	      complex <double> eC1qg_1   = ccoeff::C1qg_1[ii1];
	      complex <double> eC2qg_1   = ccoeff::C2qg_1[ii1];
	      complex <double> eC2qqp_1  = ccoeff::C2qqp_1[ii1];
	      complex <double> eC2qqb_1  = ccoeff::C2qqb_1[ii1];
	      complex <double> eC2qqbp_1 = ccoeff::C2qqbp_1[ii1];
	      complex <double> eC1qq_2   = ccoeff::C1qq_2[ii2];
	      complex <double> eC2qq_2   = ccoeff::C2qq_2[ii2];
	      complex <double> eC1qg_2   = ccoeff::C1qg_2[ii2];
	      complex <double> eC2qg_2   = ccoeff::C2qg_2[ii2];
	      complex <double> eC2qqp_2  = ccoeff::C2qqp_2[ii2];
	      complex <double> eC2qqb_2  = ccoeff::C2qqb_2[ii2];
	      complex <double> eC2qqbp_2 = ccoeff::C2qqbp_2[ii2];


	      double nf = 5.;
	      H3st_qqb_qqb    [idx] = (LQF*(+ 24.*eC1qq_2*eC1qq_1*gamma1qq_2[ii2]+ 24.*eC2qq_2*gamma1qq_2[ii2]+ 24.*eC2qq_1*gamma1qq_2[ii2]+ 24.*eC1qq_2*eC1qq_1*gamma1qq_1[ii1]+ 24.*eC2qq_2*gamma1qq_1[ii1]+ 24.*eC2qq_1*gamma1qq_1[ii1]+ 24.*eC1qq_2*gamma2qq_2[ii2]+ 24.*eC1qq_1*gamma2qq_2[ii2]+ 24.*eC1qq_2*gamma2qq_1[ii1]+ 24.*eC1qq_1*gamma2qq_1[ii1]+ 24.*gamma3qq_2[ii2]+ 24.*gamma3qq_1[ii1]+ 12.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF+ 12.*eC1qq_1*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF+ 12.*beta1*gamma1qq_2[ii2]*LF+ 12.*beta0*eC1qq_2*gamma1qq_2[ii2]*LF+ 12.*beta0*eC1qq_1*gamma1qq_2[ii2]*LF + 12.*eC1qq_2*pow(gamma1qq_2[ii2],2)*LF + 12.*eC1qq_1*pow(gamma1qq_2[ii2],2)*LF + 12.*eC1qq_2*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 12.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 12.*beta1*gamma1qq_1[ii1]*LF + 12.*beta0*eC1qq_2*gamma1qq_1[ii1]*LF + 12.*beta0*eC1qq_1*gamma1qq_1[ii1]*LF + 24.*eC1qq_2*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LF + 24.*eC1qq_1*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LF + 12.*eC1qq_2*pow(gamma1qq_1[ii1],2)*LF + 12.*eC1qq_1*pow(gamma1qq_1[ii1],2)*LF + 12.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LF + 12.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LF + 24.*beta0*gamma2qq_2[ii2]*LF + 24.*gamma1qq_2[ii2]*gamma2qq_2[ii2]*LF + 24.*gamma1qq_1[ii1]*gamma2qq_2[ii2]*LF + 12.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LF + 12.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LF + 24.*beta0*gamma2qq_1[ii1]*LF + 24.*gamma1qq_2[ii2]*gamma2qq_1[ii1]*LF + 24.*gamma1qq_1[ii1]*gamma2qq_1[ii1]*LF + 12.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 8.*pow(beta0,2)*gamma1qq_2[ii2]*LF2 + 12.*beta0*pow(gamma1qq_2[ii2],2)*LF2 + 12.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 12.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LF2 + 8.*pow(beta0,2)*gamma1qq_1[ii1]*LF2 + 12.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_1[ii1]* LF2 + 24.*beta0*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LF2 + 12.*pow(gamma1qq_2[ii2],2)*gamma1qq_1[ii1]* LF2 + 12.*beta0*pow(gamma1qq_1[ii1],2)*LF2 + 12.*gamma1qq_2[ii2]*pow(gamma1qq_1[ii1],2)*LF2 + 4.*(gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2] + 2.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2] + pow(gamma1qq_2[ii2],3))*LQF2 + 4.*(gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1] + 2.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1] + pow(gamma1qq_1[ii1],3))*LQF2- 12.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 12.*eC1qq_1*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 24.*B2q*gamma1qq_2[ii2]*LQ + 12.*beta1*gamma1qq_2[ii2]*LQ - 24.*B1q*eC1qq_2*gamma1qq_2[ii2]*LQ + 36.*beta0*eC1qq_2*gamma1qq_2[ii2]*LQ - 24.*B1q*eC1qq_1*gamma1qq_2[ii2]*LQ + 36.*beta0*eC1qq_1*gamma1qq_2[ii2]*LQ - 12.*eC1qq_2*pow(gamma1qq_2[ii2],2)*LQ - 12.*eC1qq_1*pow(gamma1qq_2[ii2],2)*LQ - 12.*eC1qq_2*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 12.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 24.*B2q*gamma1qq_1[ii1]*LQ + 12.*beta1*gamma1qq_1[ii1]*LQ - 24.*B1q*eC1qq_2*gamma1qq_1[ii1]*LQ + 36.*beta0*eC1qq_2*gamma1qq_1[ii1]*LQ - 24.*B1q*eC1qq_1*gamma1qq_1[ii1]*LQ + 36.*beta0*eC1qq_1*gamma1qq_1[ii1]*LQ - 24.*eC1qq_2*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LQ - 24.*eC1qq_1*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LQ - 12.*eC1qq_2*pow(gamma1qq_1[ii1],2)*LQ - 12.*eC1qq_1*pow(gamma1qq_1[ii1],2)*LQ - 12.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LQ - 12.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LQ - 24.*B1q*gamma2qq_2[ii2]*LQ + 24.*beta0*gamma2qq_2[ii2]*LQ - 24.*gamma1qq_2[ii2]*gamma2qq_2[ii2]*LQ - 24.*gamma1qq_1[ii1]*gamma2qq_2[ii2]*LQ - 12.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LQ - 12.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LQ - 24.*B1q*gamma2qq_1[ii1]*LQ + 24.*beta0*gamma2qq_1[ii1]*LQ - 24.*gamma1qq_2[ii2]*gamma2qq_1[ii1]*LQ - 24.*gamma1qq_1[ii1]*gamma2qq_1[ii1]*LQ - 12.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 12.*B1q*beta0*gamma1qq_2[ii2]*LF*LQ + 8.*pow(beta0,2)*gamma1qq_2[ii2]*LF*LQ - 12.*B1q*pow(gamma1qq_2[ii2],2)*LF*LQ - 12.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 24.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF* LQ - 12.*B1q*beta0*gamma1qq_1[ii1]*LF*LQ + 8.*pow(beta0,2)*gamma1qq_1[ii1]*LF*LQ - 24.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_1[ii1]*LF*LQ - 24.*B1q*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LF* LQ - 24.*pow(gamma1qq_2[ii2],2)*gamma1qq_1[ii1]*LF*LQ - 12.*B1q*pow(gamma1qq_1[ii1],2)*LF*LQ - 24.*gamma1qq_2[ii2]*pow(gamma1qq_1[ii1],2)*LF*LQ + 12.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 - 12.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 - 12.*A2q*gamma1qq_2[ii2]*LQ2 + 12.*pow(B1q,2)*gamma1qq_2[ii2]*LQ2 - 24.*B1q*beta0*gamma1qq_2[ii2]*LQ2 + 8.*pow(beta0,2)*gamma1qq_2[ii2]*LQ2 - 12.*A1q*eC1qq_2*gamma1qq_2[ii2]*LQ2 - 12.*A1q*eC1qq_1*gamma1qq_2[ii2]*LQ2 + 12.*B1q*pow(gamma1qq_2[ii2],2)*LQ2 - 12.*beta0*pow(gamma1qq_2[ii2],2)*LQ2 + 12.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 - 12.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 12.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LQ2 - 12.*A2q*gamma1qq_1[ii1]*LQ2 + 12.*pow(B1q,2)*gamma1qq_1[ii1]*LQ2 - 24.*B1q*beta0*gamma1qq_1[ii1]*LQ2 + 8.*pow(beta0,2)*gamma1qq_1[ii1]*LQ2 - 12.*A1q*eC1qq_2*gamma1qq_1[ii1]*LQ2 - 12.*A1q*eC1qq_1*gamma1qq_1[ii1]*LQ2 + 12.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_1[ii1]*LQ2 + 24.*B1q*gamma1qq_2[ii2]*gamma1qq_1[ii1]* LQ2 - 24.*beta0*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LQ2 + 12.*pow(gamma1qq_2[ii2],2)*gamma1qq_1[ii1]* LQ2 + 12.*B1q*pow(gamma1qq_1[ii1],2)*LQ2 - 12.*beta0*pow(gamma1qq_1[ii1],2)*LQ2 + 12.*gamma1qq_2[ii2]*pow(gamma1qq_1[ii1],2)*LQ2 - 12.*A1q*gamma2qq_2[ii2]*LQ2 - 12.*A1q*gamma2qq_1[ii1]*LQ2 - 6.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ2 - 6.*A1q*beta0*gamma1qq_2[ii2]*LF*LQ2 - 6.*A1q*pow(gamma1qq_2[ii2],2)*LF*LQ2 - 6.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ2 - 6.*A1q*beta0*gamma1qq_1[ii1]*LF*LQ2 - 12.*A1q*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LF*LQ2 - 6.*A1q*pow(gamma1qq_1[ii1],2)*LF*LQ2 + 6.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ3 + 12.*A1q*B1q*gamma1qq_2[ii2]*LQ3 - 14.*A1q*beta0*gamma1qq_2[ii2]*LQ3 + 6.*A1q*pow(gamma1qq_2[ii2],2)*LQ3 + 6.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ3 + 12.*A1q*B1q*gamma1qq_1[ii1]*LQ3 - 14.*A1q*beta0*gamma1qq_1[ii1]*LQ3 + 12.*A1q*gamma1qq_2[ii2]*gamma1qq_1[ii1]*LQ3 + 6.*A1q*pow(gamma1qq_1[ii1],2)*LQ3 + 3.*pow(A1q,2)*gamma1qq_2[ii2]*LQ4 + 3.*pow(A1q,2)*gamma1qq_1[ii1]*LQ4))/24.;
	    
	      H3st_qqb_qg_1 [idx] = (LQF*(+ 24.*eC1qq_2*eC1qg_1*gamma1gq_1[ii1]+ 24.*eC1qg_1*gamma2gq_1[ii1]+ 12.*beta0*eC1qg_1*gamma1gq_1[ii1]*LF + 24.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LF + 12.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LF + 12.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LF - 24.*B1q*eC1qg_1*gamma1gq_1[ii1]*LQ + 36.*beta0*eC1qg_1*gamma1gq_1[ii1]*LQ - 24.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LQ - 12.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LQ - 12.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LQ - 12.*A1q*eC1qg_1*gamma1gq_1[ii1]*LQ2 + 24.*eC2qg_1*gamma1gq_1[ii1]))/24.;

	      H3st_qqb_qg_2 [idx] = (LQF*(+ 24.*eC1qg_2*eC1qq_1*gamma1gq_2[ii2]+ 24.*eC1qg_2*gamma2gq_2[ii2]+ 12.*beta0*eC1qg_2*gamma1gq_2[ii2]*LF+ 24.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_1[ii1]*LF + 12.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LF+ 12.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LF - 24.*B1q*eC1qg_2*gamma1gq_2[ii2]*LQ + 36.*beta0*eC1qg_2*gamma1gq_2[ii2]*LQ - 12.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LQ - 12.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LQ - 24.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_1[ii1]*LQ - 12.*A1q*eC1qg_2*gamma1gq_2[ii2]*LQ2 + 24.*eC2qg_2*gamma1gq_2[ii2]))/24.;
	    
	      H3st_qq_1_qqb   [idx] = (LQF*(+ 12.*eC1qq_1*gamma2qqb_1[ii1] + 12.*gamma3qqb_1[ii1] + 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LF + 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LF + 12.*beta0*gamma2qqb_1[ii1]*LF + 12.*gamma1qq_2[ii2]*gamma2qqb_1[ii1]*LF + 12.*gamma1qq_1[ii1]*gamma2qqb_1[ii1]*LF + 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LF2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF2 + 6.*eC1qq_2*2.*gamma2qqb_1[ii1] + 6.*eC1qq_2*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LQF- 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LQ - 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LQ - 12.*B1q*gamma2qqb_1[ii1]*LQ + 12.*beta0*gamma2qqb_1[ii1]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqb_1[ii1]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqb_1[ii1]*LQ - 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 12.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF* LQ - 4.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 8.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ + 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 - 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]* gamma1qg_1[ii1]*LQ2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ2 - 6.*A1q*gamma2qqb_1[ii1]*LQ2 - 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ2 + 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ3))/12.;
	      H3st_qq_1_qg_1   [idx] = (LQF*(+ 12.*eC1qg_1*gamma2gq_1[ii1] + 6.*beta0*eC1qg_1*gamma1gq_1[ii1]*LF + 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LF + 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LF + 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LF + 6.*eC1qq_2*2.*eC1qg_1*gamma1gq_1[ii1] - 12.*B1q*eC1qg_1*gamma1gq_1[ii1]*LQ + 18.*beta0*eC1qg_1*gamma1gq_1[ii1]* LQ - 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LQ - 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]* LQ - 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]* LQ - 6.*A1q*eC1qg_1*gamma1gq_1[ii1]*LQ2 + 12.*eC2qg_1*gamma1gq_1[ii1] ))/12.;
	      H3st_qq_1_qq_1   [idx] = (LQF*(+ 12.*eC2qqb_1*(gamma1qq_2[ii2] + gamma1qq_1[ii1]) ))/12.;
	    
	      H3st_qqp_1_qqb  [idx] = (LQF*(+ 12.*eC1qq_1*gamma2qqbp_1[ii1] + 12.*gamma3qqbp_1[ii1] + 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LF + 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LF + 12.*beta0*gamma2qqbp_1[ii1]*LF + 12.*gamma1qq_2[ii2]*gamma2qqbp_1[ii1]*LF + 12.*gamma1qq_1[ii1]*gamma2qqbp_1[ii1]*LF + 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LF2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF2 + 6.*eC1qq_2*2.*gamma2qqbp_1[ii1] + 6.*eC1qq_2*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF- 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LQ - 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LQ - 12.*B1q*gamma2qqbp_1[ii1]*LQ + 12.*beta0*gamma2qqbp_1[ii1]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqbp_1[ii1]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqbp_1[ii1]*LQ - 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 12.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 4.*gamma1gg_1[ii1]*gamma1gq_1[ii1]* gamma1qg_1[ii1]*LF*LQ - 8.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ + 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 - 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LQ2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ2 - 6.*A1q*gamma2qqbp_1[ii1]*LQ2 - 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ2 + 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ3))/12.;
	      H3st_qqp_1_qg_1  [idx] = (LQF*(+ 12.*eC1qg_1*gamma2gq_1[ii1] + 6.*beta0*eC1qg_1*gamma1gq_1[ii1]*LF + 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LF + 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LF + 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LF + 6.*eC1qq_2*2.*eC1qg_1*gamma1gq_1[ii1] - 12.*B1q*eC1qg_1*gamma1gq_1[ii1]*LQ + 18.*beta0*eC1qg_1*gamma1gq_1[ii1]*LQ - 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LQ - 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LQ - 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LQ - 6.*A1q*eC1qg_1*gamma1gq_1[ii1]*LQ2 + 12.*eC2qg_1*gamma1gq_1[ii1] ))/12.;
	      H3st_qqp_1_qqp_1  [idx] = (LQF*(+ 12.*eC2qqbp_1*(gamma1qq_2[ii2] + gamma1qq_1[ii1]) ))/12.;
	    
	      H3st_qqbp_1_qqb [idx] = (LQF*(+ 12.*eC1qq_1*gamma2qqp_1[ii1] + 12.*gamma3qqp_1[ii1] + 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LF + 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LF + 12.*beta0*gamma2qqp_1[ii1]*LF + 12.*gamma1qq_2[ii2]*gamma2qqp_1[ii1]*LF + 12.*gamma1qq_1[ii1]*gamma2qqp_1[ii1]*LF + 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LF2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF2 + 6.*eC1qq_2*2.*gamma2qqp_1[ii1] + 6.*eC1qq_2*gamma1gq_1[ii1]*gamma1qg_1[ii1]* LQF- 6.*eC1qq_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 6.*gamma1qg_1[ii1]*gamma2gq_1[ii1]*LQ - 6.*gamma1gq_1[ii1]*gamma2qg_1[ii1]*LQ - 12.*B1q*gamma2qqp_1[ii1]*LQ + 12.*beta0*gamma2qqp_1[ii1]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqp_1[ii1]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqp_1[ii1]*LQ - 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 12.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF* LQ - 4.*gamma1gg_1[ii1]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 8.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ + 6.*B1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 - 6.*beta0*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 6.*gamma1qq_2[ii2]*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ2 + 2.*gamma1gg_1[ii1]*gamma1gq_1[ii1]* gamma1qg_1[ii1]*LQ2 + 4.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ2 - 6.*A1q*gamma2qqp_1[ii1]*LQ2 - 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*LQ2 + 3.*A1q*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ3))/12.;
	      H3st_qqbp_1_qg_1 [idx] = (LQF*(+ 12.*eC1qg_1*gamma2gq_1[ii1] + 6.*beta0*eC1qg_1*gamma1gq_1[ii1]*LF + 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LF + 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]*LF + 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]*LF + 6.*eC1qq_2*2.*eC1qg_1*gamma1gq_1[ii1] - 12.*B1q*eC1qg_1*gamma1gq_1[ii1]*LQ + 18.*beta0*eC1qg_1*gamma1gq_1[ii1]* LQ - 12.*eC1qg_1*gamma1qq_2[ii2]*gamma1gq_1[ii1]*LQ - 6.*eC1qg_1*gamma1gg_1[ii1]*gamma1gq_1[ii1]* LQ - 6.*eC1qg_1*gamma1gq_1[ii1]*gamma1qq_1[ii1]* LQ - 6.*A1q*eC1qg_1*gamma1gq_1[ii1]*LQ2 + 12.*eC2qg_1*gamma1gq_1[ii1] ))/12.;
	      H3st_qqbp_1_qqbp_1 [idx] = (LQF*(+ 12.*eC2qqp_1*(gamma1qq_2[ii2] + gamma1qq_1[ii1]) ))/12.;	    
	    
	      H3st_qg_1_qqb   [idx] = (LQF*(+ 24.*eC2qq_2*gamma1qg_1[ii1] + 24.*eC2qq_1*gamma1qg_1[ii1] + 24.*eC1qq_1*gamma2qg_1[ii1] + 24.*gamma3qg_1[ii1] + 12.*beta1*gamma1qg_1[ii1]*LF + 12.*beta0*eC1qq_1*gamma1qg_1[ii1]*LF + 24.*eC1qq_1*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LF + 12.*eC1qq_1*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF + 12.*eC1qq_1*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF + 24.*gamma1qg_1[ii1]*gamma2qq_2[ii2]*LF + 12.*gamma1qg_1[ii1]*gamma2gg_1[ii1]*LF + 24.*beta0*gamma2qg_1[ii1]*LF + 24.*gamma1qq_2[ii2]*gamma2qg_1[ii1]*LF + 12.*gamma1gg_1[ii1]*gamma2qg_1[ii1]*LF + 12.*gamma1qq_1[ii1]*gamma2qg_1[ii1]*LF + 12.*gamma1qg_1[ii1]*gamma2qq_1[ii1]*LF + 12.*gamma1qg_1[ii1]*gamma2qqb_1[ii1]*LF - 12.*gamma1qg_1[ii1]*gamma2qqbp_1[ii1]*LF - 12.*gamma1qg_1[ii1]*gamma2qqp_1[ii1]*LF + 8.*pow(beta0,2)*gamma1qg_1[ii1]*LF2 + 12.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qg_1[ii1]*LF2 + 24.*beta0*gamma1qq_2[ii2]*gamma1qg_1[ii1]* LF2 + 12.*pow(gamma1qq_2[ii2],2)*gamma1qg_1[ii1]*LF2 + 12.*beta0*gamma1gg_1[ii1]*gamma1qg_1[ii1]* LF2 + 12.*gamma1qq_2[ii2]*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF2 + 4.*pow(gamma1gg_1[ii1],2)*gamma1qg_1[ii1]*LF2 + 12.*beta0*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF2 + 12.*gamma1qq_2[ii2]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF2 + 4.*gamma1gg_1[ii1]*gamma1qg_1[ii1]* gamma1qq_1[ii1]*LF2 + 4.*gamma1qg_1[ii1]*pow(gamma1qq_1[ii1],2)*LF2 - 24.*B2q*gamma1qg_1[ii1]*LQ + 12.*beta1*gamma1qg_1[ii1]*LQ - 24.*B1q*eC1qq_1*gamma1qg_1[ii1]*LQ + 36.*beta0*eC1qq_1*gamma1qg_1[ii1]*LQ - 24.*eC1qq_1*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LQ - 12.*eC1qq_1*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ - 12.*eC1qq_1*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ - 24.*gamma1qg_1[ii1]*gamma2qq_2[ii2]*LQ - 12.*gamma1qg_1[ii1]*gamma2gg_1[ii1]*LQ - 24.*B1q*gamma2qg_1[ii1]*LQ + 24.*beta0*gamma2qg_1[ii1]*LQ - 24.*gamma1qq_2[ii2]*gamma2qg_1[ii1]*LQ - 12.*gamma1gg_1[ii1]*gamma2qg_1[ii1]*LQ - 12.*gamma1qq_1[ii1]*gamma2qg_1[ii1]*LQ - 12.*gamma1qg_1[ii1]*gamma2qq_1[ii1]*LQ - 12.*gamma1qg_1[ii1]*gamma2qqb_1[ii1]*LQ + 12.*gamma1qg_1[ii1]*gamma2qqbp_1[ii1]*LQ + 12.*gamma1qg_1[ii1]*gamma2qqp_1[ii1]*LQ - 12.*B1q*beta0*gamma1qg_1[ii1]*LF*LQ + 8.*pow(beta0,2)*gamma1qg_1[ii1]*LF*LQ - 24.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qg_1[ii1]*LF*LQ - 24.*B1q*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LF* LQ - 24.*pow(gamma1qq_2[ii2],2)*gamma1qg_1[ii1]*LF*LQ - 12.*B1q*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF* LQ - 24.*gamma1qq_2[ii2]*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF*LQ - 8.*pow(gamma1gg_1[ii1],2)*gamma1qg_1[ii1]*LF*LQ - 12.*B1q*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ - 24.*gamma1qq_2[ii2]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ - 8.*gamma1gg_1[ii1]*gamma1qg_1[ii1]* gamma1qq_1[ii1]*LF*LQ - 8.*gamma1qg_1[ii1]*pow(gamma1qq_1[ii1],2)*LF*LQ - 12.*A2q*gamma1qg_1[ii1]*LQ2 + 12.*pow(B1q,2)*gamma1qg_1[ii1]*LQ2 - 24.*B1q*beta0*gamma1qg_1[ii1]*LQ2 + 8.*pow(beta0,2)*gamma1qg_1[ii1]*LQ2 - 12.*A1q*eC1qq_1*gamma1qg_1[ii1]*LQ2 + 12.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qg_1[ii1]*LQ2 + 24.*B1q*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LQ2 - 24.*beta0*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LQ2 + 12.*pow(gamma1qq_2[ii2],2)*gamma1qg_1[ii1]*LQ2 + 12.*B1q*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ2 - 12.*beta0*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ2 + 12.*gamma1qq_2[ii2]*gamma1gg_1[ii1]*gamma1qg_1[ii1]* LQ2 + 4.*pow(gamma1gg_1[ii1],2)*gamma1qg_1[ii1]*LQ2 + 12.*B1q*gamma1qg_1[ii1]*gamma1qq_1[ii1]* LQ2 - 12.*beta0*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ2 + 12.*gamma1qq_2[ii2]*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ2 + 4.*gamma1gg_1[ii1]*gamma1qg_1[ii1]* gamma1qq_1[ii1]*LQ2 + 4.*gamma1qg_1[ii1]*pow(gamma1qq_1[ii1],2)*LQ2 - 12.*A1q*gamma2qg_1[ii1]*LQ2 - 6.*A1q*beta0*gamma1qg_1[ii1]*LF*LQ2 - 12.*A1q*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LF*LQ2 - 6.*A1q*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF*LQ2 - 6.*A1q*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF*LQ2 + 12.*A1q*B1q*gamma1qg_1[ii1]*LQ3 - 14.*A1q*beta0*gamma1qg_1[ii1]*LQ3 + 12.*A1q*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LQ3 + 6.*A1q*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ3 + 6.*A1q*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ3 + 3.*pow(A1q,2)*gamma1qg_1[ii1]*LQ4 + 12.*eC1qq_2*2.*eC1qq_1*gamma1qg_1[ii1] + 12.*eC1qq_2*2.*gamma2qg_1[ii1] + 12.*eC1qq_2*beta0*gamma1qg_1[ii1]*LF + 12.*eC1qq_2*2.*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LF + 12.*eC1qq_2*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF + 12.*eC1qq_2*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF - 12.*eC1qq_2*2.*B1q*gamma1qg_1[ii1]*LQ + 12.*eC1qq_2*3.*beta0*gamma1qg_1[ii1]*LQ - 12.*eC1qq_2*2.*gamma1qq_2[ii2]*gamma1qg_1[ii1]*LQ - 12.*eC1qq_2*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ - 12.*eC1qq_2*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ - 12.*eC1qq_2*A1q*gamma1qg_1[ii1]*LQ2+ 12.*gamma1qg_1[ii1]*gamma2qqbp_1[ii1]*LF*nf + 12.*gamma1qg_1[ii1]*gamma2qqp_1[ii1]*LF*nf + 8.*gamma1gq_1[ii1]*pow(gamma1qg_1[ii1],2)*LF2*nf - 12.*gamma1qg_1[ii1]*gamma2qqbp_1[ii1]*LQ*nf - 12.*gamma1qg_1[ii1]*gamma2qqp_1[ii1]*LQ*nf - 16.*gamma1gq_1[ii1]*pow(gamma1qg_1[ii1],2)*LF*LQ*nf + 8.*gamma1gq_1[ii1]*pow(gamma1qg_1[ii1],2)*LQ2*nf))/24.;
	      H3st_qg_1_gg   [idx] = (LQF*(+ 24.*eC1qg_2*gamma1gq_2[ii2]*eC1qg_1 ))/24.;
	      H3st_qg_1_qg_1   [idx] = (LQF*(+ 24.*eC1qg_1*gamma2qq_2[ii2] + 24.*eC1qg_1*gamma2gg_1[ii1] + 12.*eC1qg_1*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF + 12.*beta0*eC1qg_1*gamma1qq_2[ii2]*LF + 12.*eC1qg_1*pow(gamma1qq_2[ii2],2)*LF + 12.*beta0*eC1qg_1*gamma1gg_1[ii1]*LF + 24.*eC1qg_1*gamma1qq_2[ii2]*gamma1gg_1[ii1]*LF + 12.*eC1qg_1*pow(gamma1gg_1[ii1],2)*LF - 12.*eC1qg_1*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 24.*B1q*eC1qg_1*gamma1qq_2[ii2]*LQ + 36.*beta0*eC1qg_1*gamma1qq_2[ii2]*LQ - 12.*eC1qg_1*pow(gamma1qq_2[ii2],2)*LQ - 24.*B1q*eC1qg_1*gamma1gg_1[ii1]*LQ + 36.*beta0*eC1qg_1*gamma1gg_1[ii1]*LQ - 24.*eC1qg_1*gamma1qq_2[ii2]*gamma1gg_1[ii1]*LQ - 12.*eC1qg_1*pow(gamma1gg_1[ii1],2)*LQ - 12.*A1q*eC1qg_1*gamma1qq_2[ii2]*LQ2 - 12.*A1q*eC1qg_1*gamma1gg_1[ii1]*LQ2 + 12.*eC1qq_2*2.*eC1qg_1*(gamma1qq_2[ii2] + gamma1gg_1[ii1]) + 24.*eC1qg_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF*nf - 24.*eC1qg_1*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ*nf + 24.*eC2qg_1*gamma1qq_2[ii2] + 24.*eC2qg_1*gamma1gg_1[ii1] ))/24.;
	      H3st_qg_1_qg_2   [idx] = (LQF*(+ 24.*eC1qg_2*gamma1gq_2[ii2]*gamma1qg_1[ii1]*LQF))/24.;
	      H3st_qg_1_qq_1   [idx] = (LQF*(+ 24.*eC2qqb_1*gamma1qg_1[ii1] ))/24.;
	      H3st_qg_1_qqbp_1   [idx] = (LQF*(- 24.*eC2qqp_1*gamma1qg_1[ii1] + 24.*eC2qqp_1*gamma1qg_1[ii1]*nf ))/24.;
	      H3st_qg_1_qqp_1   [idx] = (LQF*(- 24.*eC2qqbp_1*gamma1qg_1[ii1] + 24.*eC2qqbp_1*gamma1qg_1[ii1]*nf ))/24.;

	      H3st_qq_2_qqb   [idx] = (LQF*(+ 12.*eC1qq_2*gamma2qqb_2[ii2] + 12.*gamma3qqb_2[ii2] + 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF + 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LF + 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LF + 12.*beta0*gamma2qqb_2[ii2]*LF + 12.*gamma1qq_1[ii1]*gamma2qqb_2[ii2]*LF + 12.*gamma1qq_2[ii2]*gamma2qqb_2[ii2]*LF + 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]* LF2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 + 6.*eC1qq_1*2.*gamma2qqb_2[ii2] + 6.*eC1qq_1*gamma1gq_2[ii2]*gamma1qg_2[ii2]* LQF- 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LQ - 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LQ - 12.*B1q*gamma2qqb_2[ii2]*LQ + 12.*beta0*gamma2qqb_2[ii2]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqb_2[ii2]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqb_2[ii2]*LQ - 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 12.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF* LQ - 4.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 8.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ + 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 - 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]* gamma1qg_2[ii2]*LQ2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 - 6.*A1q*gamma2qqb_2[ii2]*LQ2 - 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ2 + 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ3))/12.;
	      H3st_qq_2_qg_2   [idx] = (LQF*(+ 12.*eC1qg_2*gamma2gq_2[ii2] + 6.*beta0*eC1qg_2*gamma1gq_2[ii2]*LF + 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LF + 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LF + 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LF + 6.*eC1qq_1*2.*eC1qg_2*gamma1gq_2[ii2] - 12.*B1q*eC1qg_2*gamma1gq_2[ii2]*LQ + 18.*beta0*eC1qg_2*gamma1gq_2[ii2]* LQ - 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LQ - 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]* LQ - 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]* LQ - 6.*A1q*eC1qg_2*gamma1gq_2[ii2]*LQ2 + 12.*eC2qg_2*gamma1gq_2[ii2] ))/12.;
	      H3st_qq_2_qq_2   [idx] = (LQF*(+ 12.*eC2qqb_2*(gamma1qq_1[ii1] + gamma1qq_2[ii2]) ))/12.;
	    
	      H3st_qqp_2_qqb  [idx] = (LQF*(+ 12.*eC1qq_2*gamma2qqbp_2[ii2] + 12.*gamma3qqbp_2[ii2] + 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF + 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LF + 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LF + 12.*beta0*gamma2qqbp_2[ii2]*LF + 12.*gamma1qq_1[ii1]*gamma2qqbp_2[ii2]*LF + 12.*gamma1qq_2[ii2]*gamma2qqbp_2[ii2]*LF + 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]* LF2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 + 6.*eC1qq_1*2.*gamma2qqbp_2[ii2] + 6.*eC1qq_1*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF- 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LQ - 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LQ - 12.*B1q*gamma2qqbp_2[ii2]*LQ + 12.*beta0*gamma2qqbp_2[ii2]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqbp_2[ii2]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqbp_2[ii2]*LQ - 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 12.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 4.*gamma1gg_2[ii2]*gamma1gq_2[ii2]* gamma1qg_2[ii2]*LF*LQ - 8.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ + 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 - 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]* LQ2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 - 6.*A1q*gamma2qqbp_2[ii2]*LQ2 - 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ2 + 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ3))/12.;
	      H3st_qqp_2_qg_2  [idx] = (LQF*(+ 12.*eC1qg_2*gamma2gq_2[ii2] + 6.*beta0*eC1qg_2*gamma1gq_2[ii2]*LF + 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LF + 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LF + 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LF + 6.*eC1qq_1*2.*eC1qg_2*gamma1gq_2[ii2] - 12.*B1q*eC1qg_2*gamma1gq_2[ii2]*LQ + 18.*beta0*eC1qg_2*gamma1gq_2[ii2]*LQ - 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LQ - 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LQ - 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LQ - 6.*A1q*eC1qg_2*gamma1gq_2[ii2]*LQ2 + 12.*eC2qg_2*gamma1gq_2[ii2] ))/12.;
	      H3st_qqp_2_qqp_2  [idx] = (LQF*(+ 12.*eC2qqbp_2*(gamma1qq_1[ii1] + gamma1qq_2[ii2]) ))/12.;	    
	      H3st_qqbp_2_qqb [idx] = (LQF*(+ 12.*eC1qq_2*gamma2qqp_2[ii2] + 12.*gamma3qqp_2[ii2] + 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF + 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LF + 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LF + 12.*beta0*gamma2qqp_2[ii2]*LF + 12.*gamma1qq_1[ii1]*gamma2qqp_2[ii2]*LF + 12.*gamma1qq_2[ii2]*gamma2qqp_2[ii2]*LF + 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]* LF2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 + 6.*eC1qq_1*2.*gamma2qqp_2[ii2] + 6.*eC1qq_1*gamma1gq_2[ii2]*gamma1qg_2[ii2]* LQF- 6.*eC1qq_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ - 6.*gamma1qg_2[ii2]*gamma2gq_2[ii2]*LQ - 6.*gamma1gq_2[ii2]*gamma2qg_2[ii2]*LQ - 12.*B1q*gamma2qqp_2[ii2]*LQ + 12.*beta0*gamma2qqp_2[ii2]*LQ - 12.*gamma1qq_1[ii1]*gamma2qqp_2[ii2]*LQ - 12.*gamma1qq_2[ii2]*gamma2qqp_2[ii2]*LQ - 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 12.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF* LQ - 4.*gamma1gg_2[ii2]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 8.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ + 6.*B1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 - 6.*beta0*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 6.*gamma1qq_1[ii1]*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ2 + 2.*gamma1gg_2[ii2]*gamma1gq_2[ii2]* gamma1qg_2[ii2]*LQ2 + 4.*gamma1gq_2[ii2]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 - 6.*A1q*gamma2qqp_2[ii2]*LQ2 - 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*LQ2 + 3.*A1q*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ3))/12.;
	      H3st_qqbp_2_qg_2 [idx] = (LQF*(+ 12.*eC1qg_2*gamma2gq_2[ii2] + 6.*beta0*eC1qg_2*gamma1gq_2[ii2]*LF + 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LF + 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]*LF + 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]*LF + 6.*eC1qq_1*2.*eC1qg_2*gamma1gq_2[ii2] - 12.*B1q*eC1qg_2*gamma1gq_2[ii2]*LQ + 18.*beta0*eC1qg_2*gamma1gq_2[ii2]* LQ - 12.*eC1qg_2*gamma1qq_1[ii1]*gamma1gq_2[ii2]*LQ - 6.*eC1qg_2*gamma1gg_2[ii2]*gamma1gq_2[ii2]* LQ - 6.*eC1qg_2*gamma1gq_2[ii2]*gamma1qq_2[ii2]* LQ - 6.*A1q*eC1qg_2*gamma1gq_2[ii2]*LQ2 + 12.*eC2qg_2*gamma1gq_2[ii2] ))/12.;
	      H3st_qqbp_2_qqbp_2 [idx] = (LQF*(+ 12.*eC2qqp_2*(gamma1qq_1[ii1] + gamma1qq_2[ii2]) ))/12.;

	      H3st_qg_2_qqb   [idx] = (LQF*(+ 24.*eC2qq_1*gamma1qg_2[ii2] + 24.*eC2qq_2*gamma1qg_2[ii2] + 24.*eC1qq_2*gamma2qg_2[ii2] + 24.*gamma3qg_2[ii2] + 12.*beta1*gamma1qg_2[ii2]*LF + 12.*beta0*eC1qq_2*gamma1qg_2[ii2]*LF + 24.*eC1qq_2*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LF + 12.*eC1qq_2*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF + 12.*eC1qq_2*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF + 24.*gamma1qg_2[ii2]*gamma2qq_1[ii1]*LF + 12.*gamma1qg_2[ii2]*gamma2gg_2[ii2]*LF + 24.*beta0*gamma2qg_2[ii2]*LF + 24.*gamma1qq_1[ii1]*gamma2qg_2[ii2]*LF + 12.*gamma1gg_2[ii2]*gamma2qg_2[ii2]*LF + 12.*gamma1qq_2[ii2]*gamma2qg_2[ii2]*LF + 12.*gamma1qg_2[ii2]*gamma2qq_2[ii2]*LF + 12.*gamma1qg_2[ii2]*gamma2qqb_2[ii2]*LF - 12.*gamma1qg_2[ii2]*gamma2qqbp_2[ii2]*LF - 12.*gamma1qg_2[ii2]*gamma2qqp_2[ii2]*LF + 8.*pow(beta0,2)*gamma1qg_2[ii2]*LF2 + 12.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF2 + 24.*beta0*gamma1qq_1[ii1]*gamma1qg_2[ii2]* LF2 + 12.*pow(gamma1qq_1[ii1],2)*gamma1qg_2[ii2]*LF2 + 12.*beta0*gamma1gg_2[ii2]*gamma1qg_2[ii2]* LF2 + 12.*gamma1qq_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF2 + 4.*pow(gamma1gg_2[ii2],2)*gamma1qg_2[ii2]*LF2 + 12.*beta0*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 + 12.*gamma1qq_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 + 4.*gamma1gg_2[ii2]*gamma1qg_2[ii2]* gamma1qq_2[ii2]*LF2 + 4.*gamma1qg_2[ii2]*pow(gamma1qq_2[ii2],2)*LF2 - 24.*B2q*gamma1qg_2[ii2]*LQ + 12.*beta1*gamma1qg_2[ii2]*LQ - 24.*B1q*eC1qq_2*gamma1qg_2[ii2]*LQ + 36.*beta0*eC1qq_2*gamma1qg_2[ii2]*LQ - 24.*eC1qq_2*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LQ - 12.*eC1qq_2*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ - 12.*eC1qq_2*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ - 24.*gamma1qg_2[ii2]*gamma2qq_1[ii1]*LQ - 12.*gamma1qg_2[ii2]*gamma2gg_2[ii2]*LQ - 24.*B1q*gamma2qg_2[ii2]*LQ + 24.*beta0*gamma2qg_2[ii2]*LQ - 24.*gamma1qq_1[ii1]*gamma2qg_2[ii2]*LQ - 12.*gamma1gg_2[ii2]*gamma2qg_2[ii2]*LQ - 12.*gamma1qq_2[ii2]*gamma2qg_2[ii2]*LQ - 12.*gamma1qg_2[ii2]*gamma2qq_2[ii2]*LQ - 12.*gamma1qg_2[ii2]*gamma2qqb_2[ii2]*LQ + 12.*gamma1qg_2[ii2]*gamma2qqbp_2[ii2]*LQ + 12.*gamma1qg_2[ii2]*gamma2qqp_2[ii2]*LQ - 12.*B1q*beta0*gamma1qg_2[ii2]*LF*LQ + 8.*pow(beta0,2)*gamma1qg_2[ii2]*LF*LQ - 24.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF*LQ - 24.*B1q*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LF* LQ - 24.*pow(gamma1qq_1[ii1],2)*gamma1qg_2[ii2]*LF*LQ - 12.*B1q*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF* LQ - 24.*gamma1qq_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 8.*pow(gamma1gg_2[ii2],2)*gamma1qg_2[ii2]*LF*LQ - 12.*B1q*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ - 24.*gamma1qq_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ - 8.*gamma1gg_2[ii2]*gamma1qg_2[ii2]* gamma1qq_2[ii2]*LF*LQ - 8.*gamma1qg_2[ii2]*pow(gamma1qq_2[ii2],2)*LF*LQ - 12.*A2q*gamma1qg_2[ii2]*LQ2 + 12.*pow(B1q,2)*gamma1qg_2[ii2]*LQ2 - 24.*B1q*beta0*gamma1qg_2[ii2]*LQ2 + 8.*pow(beta0,2)*gamma1qg_2[ii2]*LQ2 - 12.*A1q*eC1qq_2*gamma1qg_2[ii2]*LQ2 + 12.*gamma1gq_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ2 + 24.*B1q*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LQ2 - 24.*beta0*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LQ2 + 12.*pow(gamma1qq_1[ii1],2)*gamma1qg_2[ii2]*LQ2 + 12.*B1q*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ2 - 12.*beta0*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ2 + 12.*gamma1qq_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]* LQ2 + 4.*pow(gamma1gg_2[ii2],2)*gamma1qg_2[ii2]*LQ2 + 12.*B1q*gamma1qg_2[ii2]*gamma1qq_2[ii2]* LQ2 - 12.*beta0*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 + 12.*gamma1qq_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 + 4.*gamma1gg_2[ii2]*gamma1qg_2[ii2]* gamma1qq_2[ii2]*LQ2 + 4.*gamma1qg_2[ii2]*pow(gamma1qq_2[ii2],2)*LQ2 - 12.*A1q*gamma2qg_2[ii2]*LQ2 - 6.*A1q*beta0*gamma1qg_2[ii2]*LF*LQ2 - 12.*A1q*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LF*LQ2 - 6.*A1q*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF*LQ2 - 6.*A1q*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ2 + 12.*A1q*B1q*gamma1qg_2[ii2]*LQ3 - 14.*A1q*beta0*gamma1qg_2[ii2]*LQ3 + 12.*A1q*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LQ3 + 6.*A1q*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ3 + 6.*A1q*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ3 + 3.*pow(A1q,2)*gamma1qg_2[ii2]*LQ4 + 12.*eC1qq_1*2.*eC1qq_2*gamma1qg_2[ii2] + 12.*eC1qq_1*2.*gamma2qg_2[ii2] + 12.*eC1qq_1*beta0*gamma1qg_2[ii2]*LF + 12.*eC1qq_1*2.*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LF + 12.*eC1qq_1*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF + 12.*eC1qq_1*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF - 12.*eC1qq_1*2.*B1q*gamma1qg_2[ii2]*LQ + 12.*eC1qq_1*3.*beta0*gamma1qg_2[ii2]*LQ - 12.*eC1qq_1*2.*gamma1qq_1[ii1]*gamma1qg_2[ii2]*LQ - 12.*eC1qq_1*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ - 12.*eC1qq_1*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ - 12.*eC1qq_1*A1q*gamma1qg_2[ii2]*LQ2+ 12.*gamma1qg_2[ii2]*gamma2qqbp_2[ii2]*LF*nf + 12.*gamma1qg_2[ii2]*gamma2qqp_2[ii2]*LF*nf + 8.*gamma1gq_2[ii2]*pow(gamma1qg_2[ii2],2)*LF2*nf - 12.*gamma1qg_2[ii2]*gamma2qqbp_2[ii2]*LQ*nf - 12.*gamma1qg_2[ii2]*gamma2qqp_2[ii2]*LQ*nf - 16.*gamma1gq_2[ii2]*pow(gamma1qg_2[ii2],2)*LF*LQ*nf + 8.*gamma1gq_2[ii2]*pow(gamma1qg_2[ii2],2)*LQ2*nf))/24.;
	      H3st_qg_2_gg   [idx] = (LQF*(+ 24.*eC1qg_1*gamma1gq_1[ii1]*eC1qg_2 ))/24.;
	      H3st_qg_2_qg_2   [idx] = (LQF*(+ 24.*eC1qg_2*gamma2qq_1[ii1] + 24.*eC1qg_2*gamma2gg_2[ii2] + 12.*eC1qg_2*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LF + 12.*beta0*eC1qg_2*gamma1qq_1[ii1]*LF + 12.*eC1qg_2*pow(gamma1qq_1[ii1],2)*LF + 12.*beta0*eC1qg_2*gamma1gg_2[ii2]*LF + 24.*eC1qg_2*gamma1qq_1[ii1]*gamma1gg_2[ii2]*LF + 12.*eC1qg_2*pow(gamma1gg_2[ii2],2)*LF - 12.*eC1qg_2*gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQ - 24.*B1q*eC1qg_2*gamma1qq_1[ii1]*LQ + 36.*beta0*eC1qg_2*gamma1qq_1[ii1]*LQ - 12.*eC1qg_2*pow(gamma1qq_1[ii1],2)*LQ - 24.*B1q*eC1qg_2*gamma1gg_2[ii2]*LQ + 36.*beta0*eC1qg_2*gamma1gg_2[ii2]*LQ - 24.*eC1qg_2*gamma1qq_1[ii1]*gamma1gg_2[ii2]*LQ - 12.*eC1qg_2*pow(gamma1gg_2[ii2],2)*LQ - 12.*A1q*eC1qg_2*gamma1qq_1[ii1]*LQ2 - 12.*A1q*eC1qg_2*gamma1gg_2[ii2]*LQ2 + 12.*eC1qq_1*2.*eC1qg_2*(gamma1qq_1[ii1] + gamma1gg_2[ii2]) + 24.*eC1qg_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LF*nf - 24.*eC1qg_2*gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQ*nf + 24.*eC2qg_2*gamma1qq_1[ii1] + 24.*eC2qg_2*gamma1gg_2[ii2] ))/24.;
	      H3st_qg_2_qg_1   [idx] = (LQF*(+ 24.*eC1qg_1*gamma1gq_1[ii1]*gamma1qg_2[ii2]*LQF))/24.;
	      H3st_qg_2_qq_2   [idx] = (LQF*(+ 24.*eC2qqb_2*gamma1qg_2[ii2] ))/24.;
	      H3st_qg_2_qqbp_2   [idx] = (LQF*(- 24.*eC2qqp_2*gamma1qg_2[ii2] + 24.*eC2qqp_2*gamma1qg_2[ii2]*nf ))/24.;
	      H3st_qg_2_qqp_2   [idx] = (LQF*(- 24.*eC2qqbp_2*gamma1qg_2[ii2] + 24.*eC2qqbp_2*gamma1qg_2[ii2]*nf ))/24.;

	      H3st_gg_qqb     [idx] = (LQF*(+ 2.*eC1qq_2*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF + 2.*gamma1qg_2[ii2]*gamma2qg_1[ii1]*LF + 2.*gamma1qg_1[ii1]*gamma2qg_2[ii2]*LF + 2.*beta0*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF2 + gamma1gg_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF2 + gamma1qg_1[ii1]*gamma1qq_1[ii1]*gamma1qg_2[ii2]* LF2 + gamma1qg_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF2 + gamma1qg_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF2 - 2.*eC1qq_2*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ - 2.*gamma1qg_2[ii2]*gamma2qg_1[ii1]*LQ - 2.*gamma1qg_1[ii1]*gamma2qg_2[ii2]*LQ - 2.*B1q*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF*LQ - 2.*gamma1gg_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF*LQ - 2.*gamma1qg_1[ii1]*gamma1qq_1[ii1]* gamma1qg_2[ii2]*LF*LQ - 2.*gamma1qg_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF*LQ - 2.*gamma1qg_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF*LQ + 2.*B1q*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ2 - 2.*beta0*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ2 + gamma1gg_1[ii1]*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ2 + gamma1qg_1[ii1]*gamma1qq_1[ii1]*gamma1qg_2[ii2]* LQ2 + gamma1qg_1[ii1]*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ2 + gamma1qg_1[ii1]*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ2 - A1q*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF*LQ2 + A1q*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ3 + 2.*eC1qq_1*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LF - 2.*eC1qq_1*gamma1qg_1[ii1]*gamma1qg_2[ii2]*LQ))/2.;
	      H3st_gg_qg_1     [idx] = (LQF*(+ 2.*eC2qg_1*gamma1qg_2[ii2] + eC1qg_1*2.*eC1qq_2*gamma1qg_2[ii2] + eC1qg_1*2.*gamma2qg_2[ii2] + eC1qg_1*beta0*gamma1qg_2[ii2]*LF + eC1qg_1*2.*gamma1gg_1[ii1]*gamma1qg_2[ii2]*LF + eC1qg_1*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LF + eC1qg_1*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LF - eC1qg_1*2.*B1q*gamma1qg_2[ii2]*LQ + eC1qg_1*3.*beta0*gamma1qg_2[ii2]*LQ - eC1qg_1*2.*gamma1gg_1[ii1]*gamma1qg_2[ii2]* LQ - eC1qg_1*gamma1gg_2[ii2]*gamma1qg_2[ii2]*LQ - eC1qg_1*gamma1qg_2[ii2]*gamma1qq_2[ii2]*LQ - eC1qg_1*A1q*gamma1qg_2[ii2]*LQ2))/2.;
	      H3st_gg_qg_2     [idx] = (LQF*(+ 2.*eC2qg_2*gamma1qg_1[ii1] + 2.*eC1qg_2*gamma2qg_1[ii1] + beta0*eC1qg_2*gamma1qg_1[ii1]*LF + eC1qg_2*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LF + eC1qg_2*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LF + 2.*eC1qg_2*gamma1qg_1[ii1]*gamma1gg_2[ii2]*LF - 2.*B1q*eC1qg_2*gamma1qg_1[ii1]*LQ + 3.*beta0*eC1qg_2*gamma1qg_1[ii1]*LQ - eC1qg_2*gamma1gg_1[ii1]*gamma1qg_1[ii1]*LQ - eC1qg_2*gamma1qg_1[ii1]*gamma1qq_1[ii1]*LQ - 2.*eC1qg_2*gamma1qg_1[ii1]*gamma1gg_2[ii2]*LQ - A1q*eC1qg_2*gamma1qg_1[ii1]*LQ2 + 2.*eC1qq_1*gamma1qg_1[ii1]*eC1qg_2 ))/2.;
	      H3st_gg_gg     [idx] = (LQF*(+ eC1qg_1*2.*eC1qg_2*(gamma1gg_1[ii1] + gamma1gg_2[ii2]) ))/2.;

	      H3st_qbg_1_qqb  [idx] = ((+ 2.*gamma2qqb_2[ii2]*gamma1qg_1[ii1]*LQF+ gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	      H3st_qbg_1_qg_1  [idx] = ((+ 2.*gamma2qqb_2[ii2]*eC1qg_1+ gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF*eC1qg_1)* LQF)/2.;
	      H3st_qbg_1_qg_2  [idx] = ((+ 2.*eC1qg_2*gamma1gq_2[ii2]*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	      H3st_qbg_1_qq_2  [idx] = ((+ 2.*eC2qqb_2*gamma1qg_1[ii1] )* LQF)/2.;
	      H3st_qbg_1_gg  [idx] = ((+ 2.*eC1qg_2*gamma1gq_2[ii2]*eC1qg_1)* LQF)/2.;
	    
	      H3st_qbg_2_qqb  [idx] = ((+ 2.*gamma2qqb_1[ii1]*gamma1qg_2[ii2]*LQF+ gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	      H3st_qbg_2_qg_2  [idx] = ((+ 2.*gamma2qqb_1[ii1]*eC1qg_2+ gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF*eC1qg_2)* LQF)/2.;
	      H3st_qbg_2_qg_1  [idx] = ((+ 2.*eC1qg_1*gamma1gq_1[ii1]*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	      H3st_qbg_2_qq_1  [idx] = ((+ 2.*eC2qqb_1*gamma1qg_2[ii2] )* LQF)/2.;
	      H3st_qbg_2_gg  [idx] = ((+ 2.*eC1qg_1*gamma1gq_1[ii1]*eC1qg_2)* LQF)/2.;
	    
	      H3st_qpg_1_qqb  [idx] = ((+ 2.*gamma2qqp_2[ii2]*gamma1qg_1[ii1]*LQF+ gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	      H3st_qpg_1_qg_1  [idx] = ((+ 2.*gamma2qqp_2[ii2]*eC1qg_1+ gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF*eC1qg_1)* LQF)/2.;
	      H3st_qpg_1_qg_2  [idx] = ((+ 2.*eC1qg_2*gamma1gq_2[ii2]*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	      H3st_qpg_1_qqbp_2  [idx] = ((+ 2.*eC2qqp_2*gamma1qg_1[ii1] )* LQF)/2.;
	      H3st_qpg_1_gg  [idx] = ((+ 2.*eC1qg_2*gamma1gq_2[ii2]*eC1qg_1)* LQF)/2.;

	      H3st_qpg_2_qqb  [idx] = ((+ 2.*gamma2qqp_1[ii1]*gamma1qg_2[ii2]*LQF+ gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	      H3st_qpg_2_qg_2  [idx] = ((+ 2.*gamma2qqp_1[ii1]*eC1qg_2+ gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF*eC1qg_2)* LQF)/2.;
	      H3st_qpg_2_qg_1  [idx] = ((+ 2.*eC1qg_1*gamma1gq_1[ii1]*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	      H3st_qpg_2_qqbp_1  [idx] = ((+ 2.*eC2qqp_1*gamma1qg_2[ii2] )* LQF)/2.;
	      H3st_qpg_2_gg  [idx] = ((+ 2.*eC1qg_1*gamma1gq_1[ii1]*eC1qg_2)* LQF)/2.;

	      H3st_qbpg_1_qqb [idx] = ((+ 2.*gamma2qqbp_2[ii2]*gamma1qg_1[ii1]*LQF+ gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	      H3st_qbpg_1_qg_1 [idx] = ((+ 2.*gamma2qqbp_2[ii2]*eC1qg_1+ gamma1gq_2[ii2]*gamma1qg_2[ii2]*LQF*eC1qg_1)* LQF)/2.;
	      H3st_qbpg_1_qg_2 [idx] = ((+ 2.*eC1qg_2*gamma1gq_2[ii2]*gamma1qg_1[ii1]*LQF)* LQF)/2.;
	      H3st_qbpg_1_qqp_2 [idx] = ((+ 2.*eC2qqbp_2*gamma1qg_1[ii1] )* LQF)/2.;
	      H3st_qbpg_1_gg [idx] = ((+ 2.*eC1qg_2*gamma1gq_2[ii2]*eC1qg_1)* LQF)/2.;

	      H3st_qbpg_2_qqb [idx] = ((+ 2.*gamma2qqbp_1[ii1]*gamma1qg_2[ii2]*LQF+ gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	      H3st_qbpg_2_qg_2 [idx] = ((+ 2.*gamma2qqbp_1[ii1]*eC1qg_2+ gamma1gq_1[ii1]*gamma1qg_1[ii1]*LQF*eC1qg_2)* LQF)/2.;
	      H3st_qbpg_2_qg_1 [idx] = ((+ 2.*eC1qg_1*gamma1gq_1[ii1]*gamma1qg_2[ii2]*LQF)* LQF)/2.;
	      H3st_qbpg_2_qqp_1 [idx] = ((+ 2.*eC2qqbp_1*gamma1qg_2[ii2] )* LQF)/2.;
	      H3st_qbpg_2_gg [idx] = ((+ 2.*eC1qg_1*gamma1gq_1[ii1]*eC1qg_2)* LQF)/2.;
	    
	      //Mixed LR*LQF variations
	      H3st_qqb_qqb    [idx] += -2*beta0*H2st_qqb_qqb[idx]*LR + (-beta1*LR - beta0*LR2)*H1st_qqb_qqb[idx];
	      H3st_qqb_qg_1   [idx] += -2*beta0*H2st_qqb_qg_1[idx]*LR;
	      H3st_qqb_qg_2   [idx] += -2*beta0*H2st_qqb_qg_2[idx]*LR;

	      H3st_qg_1_qqb   [idx] += -2*beta0*H2st_qg_1_qqb[idx]*LR + (-beta1*LR - beta0*LR2)*H1st_qg_1_qqb[idx];
	      H3st_qg_1_qg_1  [idx] += -2*beta0*H2st_qg_1_qg_1[idx]*LR;
	    
	      H3st_qg_2_qqb   [idx] += -2*beta0*H2st_qg_2_qqb[idx]*LR + (-beta1*LR - beta0*LR2)*H1st_qg_2_qqb[idx];
	      H3st_qg_2_qg_2  [idx] += -2*beta0*H2st_qg_2_qg_2[idx]*LR;
	    
	      H3st_qq_1_qqb   [idx] += -2*beta0*H2st_qq_1_qqb[idx]*LR;
	      H3st_qq_1_qg_1  [idx] += -2*beta0*H2st_qq_1_qg_1[idx]*LR;
	    
	      H3st_qq_2_qqb   [idx] += -2*beta0*H2st_qq_2_qqb[idx]*LR;
	      H3st_qq_2_qg_2  [idx] += -2*beta0*H2st_qq_2_qg_2[idx]*LR;
	    
	      H3st_qqp_1_qqb  [idx] += -2*beta0*H2st_qqp_1_qqb[idx]*LR;
	      H3st_qqp_1_qg_1 [idx] += -2*beta0*H2st_qqp_1_qg_1[idx]*LR;
	    
	      H3st_qqp_2_qqb  [idx] += -2*beta0*H2st_qqp_2_qqb[idx]*LR;
	      H3st_qqp_2_qg_2 [idx] += -2*beta0*H2st_qqp_2_qg_2[idx]*LR;
	    
	      H3st_qqbp_1_qqb  [idx] += -2*beta0*H2st_qqbp_1_qqb[idx]*LR;
	      H3st_qqbp_1_qg_1 [idx] += -2*beta0*H2st_qqbp_1_qg_1[idx]*LR;
	    
	      H3st_qqbp_2_qqb  [idx] += -2*beta0*H2st_qqbp_2_qqb[idx]*LR;
	      H3st_qqbp_2_qg_2 [idx] += -2*beta0*H2st_qqbp_2_qg_2[idx]*LR;
	    
	      H3st_gg_qqb      [idx] += -2*beta0*H2st_gg_qqb[idx]*LR;
	      H3st_gg_qg_1     [idx] += -2*beta0*H2st_gg_qg_1[idx]*LR;
	      H3st_gg_qg_2     [idx] += -2*beta0*H2st_gg_qg_2[idx]*LR;
	    
	      qqb_eqqb[idx]    += as3*H3st_qqb_qqb[idx];
	      qg_1_eqqb[idx]   += as3*H3st_qg_1_qqb[idx];
	      qg_2_eqqb[idx]   += as3*H3st_qg_2_qqb[idx];
	      qq_1_eqqb[idx]   += as3*H3st_qq_1_qqb[idx];
	      qq_2_eqqb[idx]   += as3*H3st_qq_2_qqb[idx];
	      qqp_1_eqqb[idx]  += as3*H3st_qqp_1_qqb[idx];
	      qqp_2_eqqb[idx]  += as3*H3st_qqp_2_qqb[idx];
	      qqbp_1_eqqb[idx] += as3*H3st_qqbp_1_qqb[idx];
	      qqbp_2_eqqb[idx] += as3*H3st_qqbp_2_qqb[idx];
	      gg_eqqb[idx]     += as3*H3st_gg_qqb[idx];
	      qbg_1_eqqb[idx]  += as3*H3st_qbg_1_qqb[idx];
	      qbg_2_eqqb[idx]  += as3*H3st_qbg_2_qqb[idx];
	      qpg_1_eqqb[idx]  += as3*H3st_qpg_1_qqb[idx];
	      qpg_2_eqqb[idx]  += as3*H3st_qpg_2_qqb[idx];
	      qbpg_1_eqqb[idx] += as3*H3st_qbpg_1_qqb[idx];
	      qbpg_2_eqqb[idx] += as3*H3st_qbpg_2_qqb[idx];

	      qqb_eqg_1[idx]    += as3*H3st_qqb_qg_1[idx];
	      qg_1_eqg_1[idx]   += as3*H3st_qg_1_qg_1[idx];
	      qg_2_eqg_1[idx]   += as3*H3st_qg_2_qg_1[idx];
	      qq_1_eqg_1[idx]   += as3*H3st_qq_1_qg_1[idx];
	      qqp_1_eqg_1[idx]  += as3*H3st_qqp_1_qg_1[idx];
	      qqbp_1_eqg_1[idx] += as3*H3st_qqbp_1_qg_1[idx];
	      gg_eqg_1[idx]     += as3*H3st_gg_qg_1[idx];
	      qbg_1_eqg_1[idx]  += as3*H3st_qbg_1_qg_1[idx];
	      qbg_2_eqg_1[idx]  += as3*H3st_qbg_2_qg_1[idx];
	      qpg_1_eqg_1[idx]  += as3*H3st_qpg_1_qg_1[idx];
	      qpg_2_eqg_1[idx]  += as3*H3st_qpg_2_qg_1[idx];
	      qbpg_1_eqg_1[idx] += as3*H3st_qbpg_1_qg_1[idx];
	      qbpg_2_eqg_1[idx] += as3*H3st_qbpg_2_qg_1[idx];

	      qqb_eqg_2[idx]    += as3*H3st_qqb_qg_2[idx];
	      qg_1_eqg_2[idx]   += as3*H3st_qg_1_qg_2[idx];
	      qg_2_eqg_2[idx]   += as3*H3st_qg_2_qg_2[idx];
	      qq_2_eqg_2[idx]   += as3*H3st_qq_2_qg_2[idx];
	      qqp_2_eqg_2[idx]  += as3*H3st_qqp_2_qg_2[idx];
	      qqbp_2_eqg_2[idx] += as3*H3st_qqbp_2_qg_2[idx];
	      gg_eqg_2[idx]     += as3*H3st_gg_qg_2[idx];
	      qbg_1_eqg_2[idx]  += as3*H3st_qbg_1_qg_2[idx];
	      qbg_2_eqg_2[idx]  += as3*H3st_qbg_2_qg_2[idx];
	      qpg_1_eqg_2[idx]  += as3*H3st_qpg_1_qg_2[idx];
	      qpg_2_eqg_2[idx]  += as3*H3st_qpg_2_qg_2[idx];
	      qbpg_1_eqg_2[idx] += as3*H3st_qbpg_1_qg_2[idx];
	      qbpg_2_eqg_2[idx] += as3*H3st_qbpg_2_qg_2[idx];

	      qq_1_eqq_1[idx]   += as3*H3st_qq_1_qq_1[idx];
	      qg_1_eqq_1[idx]   += as3*H3st_qg_1_qq_1[idx];
	      qbg_2_eqq_1[idx]  += as3*H3st_qbg_2_qq_1[idx];
	      qq_2_eqq_2[idx]   += as3*H3st_qq_2_qq_2[idx];
	      qg_2_eqq_2[idx]   += as3*H3st_qg_2_qq_2[idx];
	      qbg_1_eqq_2[idx]  += as3*H3st_qbg_1_qq_2[idx];

	      qqp_1_eqqp_1[idx]  += as3*H3st_qqp_1_qqp_1[idx];
	      qg_1_eqqp_1[idx]   += as3*H3st_qg_1_qqp_1[idx];
	      qbpg_2_eqqp_1[idx]  += as3*H3st_qbpg_2_qqp_1[idx];
	      qqp_2_eqqp_2[idx]  += as3*H3st_qqp_2_qqp_2[idx];
	      qg_2_eqqp_2[idx]   += as3*H3st_qg_2_qqp_2[idx];
	      qbpg_1_eqqp_2[idx]  += as3*H3st_qbpg_1_qqp_2[idx];

	      qqbp_1_eqqbp_1[idx]  += as3*H3st_qqbp_1_qqbp_1[idx];
	      qg_1_eqqbp_1[idx]    += as3*H3st_qg_1_qqbp_1[idx];
	      qpg_2_eqqbp_1[idx]   += as3*H3st_qpg_2_qqbp_1[idx];
	      qqbp_2_eqqbp_2[idx]  += as3*H3st_qqbp_2_qqbp_2[idx];
	      qg_2_eqqbp_2[idx]    += as3*H3st_qg_2_qqbp_2[idx];
	      qpg_1_eqqbp_2[idx]  += as3*H3st_qpg_1_qqbp_2[idx];

	      qg_1_egg[idx]   += as3*H3st_qg_1_gg[idx];
	      qg_2_egg[idx]   += as3*H3st_qg_2_gg[idx];
	      gg_egg[idx]   += as3*H3st_gg_gg[idx];
	      qbg_1_egg[idx]  += as3*H3st_qbg_1_gg[idx];
	      qbg_2_egg[idx]  += as3*H3st_qbg_2_gg[idx];
	      qpg_1_egg[idx]  += as3*H3st_qpg_1_gg[idx];
	      qpg_2_egg[idx]  += as3*H3st_qpg_2_gg[idx];
	      qbpg_1_egg[idx] += as3*H3st_qbpg_1_gg[idx];
	      qbpg_2_egg[idx] += as3*H3st_qbpg_2_gg[idx];
	    }
    }

  /*
  //NNNNLO --> to be implemented
  if (max(opts.order_hcoef,opts.order_evol) >= 4)
    {
      if (opts.order_hcoef < 4)
	{
	  LR  = 0;
	  LR2 = 0;
	  LR3 = 0;
	  LF  = 0;
	  LF2 = 0;
	  LF3 = 0;
	  LF4 = 0;
	  LQF  = -LQ;
	  LQF2 = pow(LQF,2);
	  LQF3 = pow(LQF,3);
	  LQF3 = pow(LQF,4);
	}

      for (int sign = mesq::positive; sign <= mesq::negative; sign++)
	for (int i1 = 0; i1 < mellinint::mdim; i1++)
	  for (int i2 = 0; i2 < mellinint::mdim; i2++)
	    {
	      int ii1 = anomalous::index(i1,mesq::positive);
	      int ii2 = anomalous::index(i2,sign);
	      int idx = hcoeff::index(i1,i2,sign);
	    }
    }
  */  

  delete[] H1st_qqb_qqb ;
  delete[] H1st_qg_1_qqb;
  delete[] H1st_qg_2_qqb;
  delete[] H2st_qqb_qqb     ;
  delete[] H2st_qqb_qg_1    ;
  delete[] H2st_qqb_qg_2    ;
  delete[] H2st_qg_1_qqb    ;
  delete[] H2st_qg_1_qg_1   ;
  delete[] H2st_qg_2_qqb    ;
  delete[] H2st_qg_2_qg_2   ;
  delete[] H2st_qq_1_qqb    ;
  delete[] H2st_qq_1_qg_1   ;
  delete[] H2st_qq_2_qqb    ;
  delete[] H2st_qq_2_qg_2   ;
  delete[] H2st_qqp_1_qqb   ;
  delete[] H2st_qqp_1_qg_1  ;
  delete[] H2st_qqp_2_qqb   ;
  delete[] H2st_qqp_2_qg_2  ;
  delete[] H2st_qqbp_1_qqb  ;
  delete[] H2st_qqbp_1_qg_1 ;
  delete[] H2st_qqbp_2_qqb  ;
  delete[] H2st_qqbp_2_qg_2 ;
  delete[] H2st_gg_qqb      ;
  delete[] H2st_gg_qg_1     ;
  delete[] H2st_gg_qg_2     ;
  delete[] H3st_qqb_qqb        ;
  delete[] H3st_qqb_qg_1       ;
  delete[] H3st_qqb_qg_2       ;
  delete[] H3st_qq_1_qqb       ;
  delete[] H3st_qq_1_qg_1      ;
  delete[] H3st_qq_1_qq_1      ;
  delete[] H3st_qqp_1_qqb      ;
  delete[] H3st_qqp_1_qg_1     ;
  delete[] H3st_qqp_1_qqp_1    ;
  delete[] H3st_qqbp_1_qqb     ;
  delete[] H3st_qqbp_1_qg_1    ;
  delete[] H3st_qqbp_1_qqbp_1  ;
  delete[] H3st_qg_1_qqb       ;
  delete[] H3st_qg_1_gg        ;
  delete[] H3st_qg_1_qg_1      ;
  delete[] H3st_qg_1_qg_2      ;
  delete[] H3st_qg_1_qq_1      ;
  delete[] H3st_qg_1_qqbp_1    ;
  delete[] H3st_qg_1_qqp_1     ;
  delete[] H3st_qq_2_qqb       ;
  delete[] H3st_qq_2_qg_2      ;
  delete[] H3st_qq_2_qq_2      ;
  delete[] H3st_qqp_2_qqb      ;
  delete[] H3st_qqp_2_qg_2     ;
  delete[] H3st_qqp_2_qqp_2    ;
  delete[] H3st_qqbp_2_qqb     ;
  delete[] H3st_qqbp_2_qg_2    ;
  delete[] H3st_qqbp_2_qqbp_2  ;
  delete[] H3st_qg_2_qqb       ;
  delete[] H3st_qg_2_gg        ;
  delete[] H3st_qg_2_qg_2      ;
  delete[] H3st_qg_2_qg_1      ;
  delete[] H3st_qg_2_qq_2      ;
  delete[] H3st_qg_2_qqbp_2    ;
  delete[] H3st_qg_2_qqp_2     ;
  delete[] H3st_gg_qqb         ;
  delete[] H3st_gg_qg_1        ;
  delete[] H3st_gg_qg_2        ;
  delete[] H3st_gg_gg          ;
  delete[] H3st_qbg_1_qqb      ;
  delete[] H3st_qbg_1_qg_1     ;
  delete[] H3st_qbg_1_qg_2     ;
  delete[] H3st_qbg_1_qq_2     ;
  delete[] H3st_qbg_1_gg       ;
  delete[] H3st_qbg_2_qqb      ;
  delete[] H3st_qbg_2_qg_2     ;
  delete[] H3st_qbg_2_qg_1     ;
  delete[] H3st_qbg_2_qq_1     ;
  delete[] H3st_qbg_2_gg       ;
  delete[] H3st_qpg_1_qqb      ;
  delete[] H3st_qpg_1_qg_1     ;
  delete[] H3st_qpg_1_qg_2     ;
  delete[] H3st_qpg_1_qqbp_2   ;
  delete[] H3st_qpg_1_gg       ;
  delete[] H3st_qpg_2_qqb      ;
  delete[] H3st_qpg_2_qg_2     ;
  delete[] H3st_qpg_2_qg_1     ;
  delete[] H3st_qpg_2_qqbp_1   ;
  delete[] H3st_qpg_2_gg       ;
  delete[] H3st_qbpg_1_qqb     ;
  delete[] H3st_qbpg_1_qg_1    ;
  delete[] H3st_qbpg_1_qg_2    ;
  delete[] H3st_qbpg_1_qqp_2   ;
  delete[] H3st_qbpg_1_gg      ;
  delete[] H3st_qbpg_2_qqb     ;
  delete[] H3st_qbpg_2_qg_2    ;
  delete[] H3st_qbpg_2_qg_1    ;
  delete[] H3st_qbpg_2_qqp_1   ;
  delete[] H3st_qbpg_2_gg      ;
}
