#include "muf.h"
#include "resint.h"
#include "pmom.h"
#include "mesq.h"
#include "settings.h"
#include "ccoeff.h"
#include "resconst.h"

using namespace pmom;
using namespace resconst;
using namespace ccoeff;

void muf::ccalc1d()
{
  LR  = resint::LR;
  LR2 = pow(LR,2);
  LR3 = pow(LR,3);

  LF  = resint::LF;
  LF2 = pow(LF,2);
  LF3 = pow(LF,3);
  LF4 = pow(LF,4);

  LQ  = resint::LQ;
  LQ2 = pow(LQ,2);
  LQ3 = pow(LQ,3);
  LQ4 = pow(LQ,4);
  LQ5 = pow(LQ,5);
  LQ6 = pow(LQ,6);
  LQ7 = pow(LQ,7);

  LQF  = LF - LQ;
  LQF2 = pow(LQF,2);
  LQF3 = pow(LQF,3);
  LQF4 = pow(LQF,4);

  double as = resint::aass;
  double as2 = pow(as,2);
  double as3 = pow(as,3);
  double as4 = pow(as,4);

  A1 = A1q;
  A2 = A2q;
  A3 = A3q;
  A4 = A4q;

  B1 = B1q;
  B2 = B2q;
  B3 = B3q;
  B4 = B4q;
  
  complex <double> H1st_qqb_qqb[mellinint::mdim*2];
  complex <double> H1st_qg_qqb[mellinint::mdim*2];

  //NLO
  if (max(opts.order_hcoef,opts.order_evol) >= 1) //use order_evol to match the order of the PDF evolution from Q to b0/b, could use order_sudak instead
    {
      if (opts.order_hcoef < 1)
	{
	  LR  = 0;
	  LR2 = 0;
	  LR3 = 0;
	  LF  = 0;
	  LF2 = 0;
	  LF3 = 0;
	  LF4 = 0;
	  LQF  = -LQ;
	  LQF2 = pow(LQF,2);
	  LQF3 = pow(LQF,3);
	  LQF4 = pow(LQF,4);
	}
	  
      for (int sign = mesq::positive; sign <= mesq::negative; sign++)
	for (int i = 0; i < mellinint::mdim; i++)
	  {
	    int idx = pmom::index(i,sign);
	    H1st_qqb_qqb[idx] = 2.*gamma1qq[idx]*LQF;
	    H1st_qg_qqb[idx]  = gamma1qg[idx]*LQF;
	    qqb_eqqb[idx] = as*H1st_qqb_qqb[idx];
	    qg_eqqb[idx]  = as*H1st_qg_qqb[idx];
	    //cout << " in ccalc " << idx << " qqb_eqq " << qqb_eqq[idx] << endl;
	  }
    }
      
  //NNLO
  complex <double> H2st_qqb_qqb  [mellinint::mdim*2];
  complex <double> H2st_qqb_qg   [mellinint::mdim*2];
  complex <double> H2st_qq_qqb   [mellinint::mdim*2];
  complex <double> H2st_qq_qg    [mellinint::mdim*2];
  complex <double> H2st_qqp_qqb  [mellinint::mdim*2];
  complex <double> H2st_qqp_qg   [mellinint::mdim*2];
  complex <double> H2st_qqbp_qqb [mellinint::mdim*2];
  complex <double> H2st_qqbp_qg  [mellinint::mdim*2];
  complex <double> H2st_qg_qqb   [mellinint::mdim*2];
  complex <double> H2st_qg_qg    [mellinint::mdim*2];
  complex <double> H2st_gg_qqb   [mellinint::mdim*2];
  complex <double> H2st_gg_qg    [mellinint::mdim*2];
  if (max(opts.order_hcoef,opts.order_evol) >= 2)
    {
      if (opts.order_hcoef < 2)
	{
	  LR  = 0;
	  LR2 = 0;
	  LF  = 0;
	  LF2 = 0;
	  LF3 = 0;
	  LF4 = 0;
	  LQF  = -LQ;
	  LQF2 = pow(LQF,2);
	  LQF3 = pow(LQF,3);
	  LQF4 = pow(LQF,4);
	}
	  
      for (int sign = mesq::positive; sign <= mesq::negative; sign++)
	for (int i = 0; i < mellinint::mdim; i++)
	  {
	    int idx = pmom::index(i,sign);

	    //Include exponentiation in the C coefficients
	    complex <double> eC1qq   = ccoeff::C1qq[idx] + H1q/2.;
	    complex <double> eC1qg   = ccoeff::C1qg[idx];
	    H2st_qqb_qqb  [idx] = LQF*(2.*gamma2qq[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF + gamma1qq[idx]*(4.*eC1qq + 2.*gamma1qq[idx]*LF - 2.*(B1q + gamma1qq[idx])*LQ - A1q*LQ2 + beta0*(LF + LQ)));
	    H2st_qqb_qg   [idx] = LQF*2.*eC1qg*gamma1gq[idx];
	    H2st_qq_qqb   [idx] = ((2.*gamma2qqb[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF)*LQF)/2.;
	    H2st_qq_qg    [idx] = (2.*eC1qg*gamma1gq[idx]*LQF)/2.;
	    H2st_qqp_qqb  [idx] = ((2.*gamma2qqp[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF)*LQF)/2.;
	    H2st_qqp_qg   [idx] = (2.*eC1qg*gamma1gq[idx]*LQF)/2.;
	    H2st_qqbp_qqb [idx] = ((2.*gamma2qqp[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF)*LQF)/2.;
	    H2st_qqbp_qg  [idx] = (2.*eC1qg*gamma1gq[idx]*LQF)/2.;
	    H2st_qg_qqb   [idx] = (LQF*(4.*eC1qq*gamma1qg[idx] + 2.*gamma2qg[idx] + beta0*gamma1qg[idx]*LF + gamma1gg[idx]*gamma1qg[idx]*LF + 3.*gamma1qg[idx]*gamma1qq[idx]*LF - 2.*B1q*gamma1qg[idx]*LQ + beta0*gamma1qg[idx]*LQ - gamma1gg[idx]*gamma1qg[idx]*LQ - 3.*gamma1qg[idx]*gamma1qq[idx]*LQ - A1q*gamma1qg[idx]*LQ2))/2.;
	    H2st_qg_qg    [idx] = (LQF*2.*eC1qg*(gamma1gg[idx] + gamma1qq[idx]))/2.;
	    H2st_gg_qqb   [idx] = gamma1qg[idx]*(gamma1qg[idx]*LQF)*LQF;
	    H2st_gg_qg    [idx] = gamma1qg[idx]*(2.*eC1qg)*LQF;

	    //Mixed LR*LQF variations
	    H2st_qqb_qqb[idx] += -beta0*H1st_qqb_qqb[idx]*LR;
	    H2st_qg_qqb[idx]  += -beta0*H1st_qg_qqb[idx]*LR;

	    qqb_eqqb[idx]  += as2*H2st_qqb_qqb[idx];
	    qg_eqqb[idx]   += as2*H2st_qg_qqb[idx];
	    qq_eqqb[idx]   += as2*H2st_qq_qqb[idx];
	    qqp_eqqb[idx]  += as2*H2st_qqp_qqb[idx];
	    qqbp_eqqb[idx] += as2*H2st_qqbp_qqb[idx];
	    gg_eqqb[idx]   += as2*H2st_gg_qqb[idx];

	    qqb_eqg[idx]  += as2*H2st_qqb_qg[idx];
	    qg_eqg[idx]   += as2*H2st_qg_qg[idx];
	    qq_eqg[idx]   += as2*H2st_qq_qg[idx];
	    qqp_eqg[idx]  += as2*H2st_qqp_qg[idx];
	    qqbp_eqg[idx] += as2*H2st_qqbp_qg[idx];
	    gg_eqg[idx]   += as2*H2st_gg_qg[idx];
	  }
    }
  //NNNLO
  complex <double> H3st_qqb_qqb   [mellinint::mdim*2];
  complex <double> H3st_qqb_qg    [mellinint::mdim*2];
  complex <double> H3st_qq_qqb    [mellinint::mdim*2];
  complex <double> H3st_qq_qq     [mellinint::mdim*2];
  complex <double> H3st_qq_qg     [mellinint::mdim*2];
  complex <double> H3st_qqp_qqb   [mellinint::mdim*2];
  complex <double> H3st_qqp_qqp   [mellinint::mdim*2];
  complex <double> H3st_qqp_qg    [mellinint::mdim*2];
  complex <double> H3st_qqbp_qqb  [mellinint::mdim*2];
  complex <double> H3st_qqbp_qqbp [mellinint::mdim*2];
  complex <double> H3st_qqbp_qg   [mellinint::mdim*2];
  complex <double> H3st_qg_qqb    [mellinint::mdim*2];
  complex <double> H3st_qg_qg     [mellinint::mdim*2];
  complex <double> H3st_qg_gg     [mellinint::mdim*2];
  complex <double> H3st_qg_qq     [mellinint::mdim*2];
  complex <double> H3st_qg_qqp    [mellinint::mdim*2];
  complex <double> H3st_qg_qqbp   [mellinint::mdim*2];
  complex <double> H3st_qbg_qqb   [mellinint::mdim*2];
  complex <double> H3st_qbg_qg    [mellinint::mdim*2];
  complex <double> H3st_qbg_qq    [mellinint::mdim*2];
  complex <double> H3st_qbg_gg    [mellinint::mdim*2];
  complex <double> H3st_qpg_qqb   [mellinint::mdim*2];
  complex <double> H3st_qpg_qg    [mellinint::mdim*2];
  complex <double> H3st_qpg_qqbp  [mellinint::mdim*2];
  complex <double> H3st_qpg_gg    [mellinint::mdim*2];
  complex <double> H3st_qbpg_qqb  [mellinint::mdim*2];
  complex <double> H3st_qbpg_qg   [mellinint::mdim*2];
  complex <double> H3st_qbpg_qqp  [mellinint::mdim*2];
  complex <double> H3st_qbpg_gg   [mellinint::mdim*2];
  complex <double> H3st_gg_qqb    [mellinint::mdim*2];
  complex <double> H3st_gg_qg     [mellinint::mdim*2];
  complex <double> H3st_gg_gg     [mellinint::mdim*2];
  if (max(opts.order_hcoef,opts.order_evol) >= 3)
    {
      if (opts.order_hcoef < 3)
	{
	  LR  = 0;
	  LR2 = 0;
	  LR3 = 0;
	  LF  = 0;
	  LF2 = 0;
	  LF3 = 0;
	  LF4 = 0;
	  LQF  = -LQ;
	  LQF2 = pow(LQF,2);
	  LQF3 = pow(LQF,3);
	  LQF4 = pow(LQF,4);
	}
	  
      for (int sign = mesq::positive; sign <= mesq::negative; sign++)
	for (int i = 0; i < mellinint::mdim; i++)
	  {
	    int idx = pmom::index(i,sign);

	    //Include exponentiation in the C coefficients
	    complex <double> eC1qq   = ccoeff::C1qq[idx] + H1q/2.;
	    complex <double> eC2qq   = ccoeff::C2qq[idx] + H1q/2.*C1qq[idx] + (H2q/2.-pow(H1q,2)/8.);
	    complex <double> eC1qg   = ccoeff::C1qg[idx];
	    complex <double> eC2qg   = ccoeff::C2qg[idx] + H1q/2.*C1qg[idx];
	    complex <double> eC2qqp  = ccoeff::C2qqp[idx];
	    complex <double> eC2qqb  = ccoeff::C2qqb[idx];
	    complex <double> eC2qqbp = ccoeff::C2qqbp[idx];
		
	    double nf = 5.;
	    H3st_qqb_qqb[idx]  = (LQF*(24.*pow(eC1qq,2)*gamma1qq[idx] + 48.*eC2qq*gamma1qq[idx] + 48.*eC1qq*gamma2qq[idx] + 24.*gamma3qq[idx] + 24.*eC1qq*gamma1gq[idx]*gamma1qg[idx]*LF + 12.*beta1*gamma1qq[idx]*LF + 24.*beta0*eC1qq*gamma1qq[idx]*LF + 48.*eC1qq*pow(gamma1qq[idx],2)*LF + 12.*gamma1qg[idx]*gamma2gq[idx]*LF + 12.*gamma1gq[idx]*gamma2qg[idx]*LF + 24.*beta0*gamma2qq[idx]*LF + 48.*gamma1qq[idx]*gamma2qq[idx]*LF + 12.*beta0*gamma1gq[idx]*gamma1qg[idx]*LF2 + 4.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LF2 + 8.*pow(beta0,2)*gamma1qq[idx]*LF2 + 20.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LF2 + 24.*beta0*pow(gamma1qq[idx],2)*LF2 + 16.*pow(gamma1qq[idx],3)*LF2 - 24.*eC1qq*gamma1gq[idx]*gamma1qg[idx]*LQ - 24.*B2q*gamma1qq[idx]*LQ + 12.*beta1*gamma1qq[idx]*LQ - 48.*B1q*eC1qq*gamma1qq[idx]*LQ + 72.*beta0*eC1qq*gamma1qq[idx]*LQ - 48.*eC1qq*pow(gamma1qq[idx],2)*LQ - 12.*gamma1qg[idx]*gamma2gq[idx]*LQ - 12.*gamma1gq[idx]*gamma2qg[idx]*LQ - 24.*B1q*gamma2qq[idx]*LQ + 24.*beta0*gamma2qq[idx]*LQ - 48.*gamma1qq[idx]*gamma2qq[idx]*LQ - 12.*B1q*gamma1gq[idx]*gamma1qg[idx]*LF*LQ - 8.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LF*LQ - 12.*B1q*beta0*gamma1qq[idx]*LF*LQ + 8.*pow(beta0,2)*gamma1qq[idx]*LF*LQ - 40.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LF*LQ - 24.*B1q*pow(gamma1qq[idx],2)*LF*LQ - 32.*pow(gamma1qq[idx],3)*LF*LQ + 12.*B1q*gamma1gq[idx]*gamma1qg[idx]*LQ2 - 12.*beta0*gamma1gq[idx]*gamma1qg[idx]*LQ2 + 4.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LQ2 - 12.*A2q*gamma1qq[idx]*LQ2 + 12.*pow(B1q,2)*gamma1qq[idx]*LQ2 - 24.*B1q*beta0*gamma1qq[idx]*LQ2 + 8.*pow(beta0,2)*gamma1qq[idx]*LQ2 - 24.*A1q*eC1qq*gamma1qq[idx]*LQ2 + 20.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LQ2 + 24.*B1q*pow(gamma1qq[idx],2)*LQ2 - 24.*beta0*pow(gamma1qq[idx],2)*LQ2 + 16.*pow(gamma1qq[idx],3)*LQ2 - 12.*A1q*gamma2qq[idx]*LQ2 - 6.*A1q*gamma1gq[idx]*gamma1qg[idx]*LF*LQ2 - 6.*A1q*beta0*gamma1qq[idx]*LF*LQ2 - 12.*A1q*pow(gamma1qq[idx],2)*LF*LQ2 + 6.*A1q*gamma1gq[idx]*gamma1qg[idx]*LQ3 + 12.*A1q*B1q*gamma1qq[idx]*LQ3 - 14.*A1q*beta0*gamma1qq[idx]*LQ3 + 12.*A1q*pow(gamma1qq[idx],2)*LQ3 + 3.*pow(A1q,2)*gamma1qq[idx]*LQ4))/12.;
	    H3st_qqb_qg[idx]  = (LQF*(24.*eC2qg*gamma1gq[idx] + 12.*eC1qg*(2.*eC1qq*gamma1gq[idx] + 2.*gamma2gq[idx] + gamma1gq[idx]*(beta0*LF + gamma1gg[idx]*LF + 3.*gamma1qq[idx]*LF - 2.*B1q*LQ + 3.*beta0*LQ - gamma1gg[idx]*LQ - 3.*gamma1qq[idx]*LQ - A1q*LQ2))))/12.;

	    H3st_qq_qqb[idx]   = (LQF*(12.*(2.*eC1qq*gamma2qqb[idx] + gamma3qqb[idx]) + 6.*(2.*eC1qq*gamma1gq[idx]*gamma1qg[idx] + gamma1qg[idx]*gamma2gq[idx] + gamma1gq[idx]*gamma2qg[idx] + 2.*beta0*gamma2qqb[idx] + 4.*gamma1qq[idx]*gamma2qqb[idx])*LF + 2.*gamma1gq[idx]*gamma1qg[idx]*(3.*beta0 + gamma1gg[idx] + 5.*gamma1qq[idx])*LF2 - 2.*(6.*eC1qq*gamma1gq[idx]*gamma1qg[idx] + 3.*gamma1qg[idx]*gamma2gq[idx] + 3.*gamma1gq[idx]*gamma2qg[idx] + 6.*(B1q - beta0 + 2.*gamma1qq[idx])*gamma2qqb[idx] + gamma1gq[idx]*gamma1qg[idx]*(3.*B1q + 2.*gamma1gg[idx] + 10.*gamma1qq[idx])*LF)*LQ + (-6.*A1q*gamma2qqb[idx] + gamma1gq[idx]*gamma1qg[idx]*(6.*B1q - 6.*beta0 + 2.*gamma1gg[idx] + 10.*gamma1qq[idx] - 3.*A1q*LF))*LQ2 + 3.*A1q*gamma1gq[idx]*gamma1qg[idx]*LQ3))/12.;
	    H3st_qq_qq[idx]   = (LQF*(12.*(2.*eC2qqb*gamma1qq[idx])))/12.;
	    H3st_qq_qg[idx]   = (LQF*(12.*(eC2qg*gamma1gq[idx]) + 6.*eC1qg*(2.*gamma2gq[idx] + gamma1gq[idx]*(2.*eC1qq + (beta0 + gamma1gg[idx] + 3.*gamma1qq[idx])*LF - (2.*B1q - 3.*beta0 + gamma1gg[idx] + 3.*gamma1qq[idx])*LQ - A1q*LQ2))))/12.;
	  
	    H3st_qqp_qqb[idx]  = (LQF*(24.*eC1qq*gamma2qqbp[idx] + 12.*gamma3qqbp[idx] + 12.*eC1qq*gamma1gq[idx]*gamma1qg[idx]*LF + 6.*gamma1qg[idx]*gamma2gq[idx]*LF + 6.*gamma1gq[idx]*gamma2qg[idx]*LF + 12.*beta0*gamma2qqbp[idx]*LF + 24.*gamma1qq[idx]*gamma2qqbp[idx]*LF + 6.*beta0*gamma1gq[idx]*gamma1qg[idx]*LF2 + 2.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LF2 + 10.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LF2 - 12.*eC1qq*gamma1gq[idx]*gamma1qg[idx]*LQ - 6.*gamma1qg[idx]*gamma2gq[idx]*LQ - 6.*gamma1gq[idx]*gamma2qg[idx]*LQ - 12.*B1q*gamma2qqbp[idx]*LQ + 12.*beta0*gamma2qqbp[idx]*LQ - 24.*gamma1qq[idx]*gamma2qqbp[idx]*LQ - 6.*B1q*gamma1gq[idx]*gamma1qg[idx]*LF*LQ - 4.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LF*LQ - 20.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LF*LQ + 6.*B1q*gamma1gq[idx]*gamma1qg[idx]*LQ2 - 6.*beta0*gamma1gq[idx]*gamma1qg[idx]*LQ2 + 2.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LQ2 + 10.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LQ2 - 6.*A1q*gamma2qqbp[idx]*LQ2 - 3.*A1q*gamma1gq[idx]*gamma1qg[idx]*LF*LQ2 + 3.*A1q*gamma1gq[idx]*gamma1qg[idx]*LQ3))/12.;
	    H3st_qqp_qqp[idx]  = (LQF*(24.*eC2qqbp*gamma1qq[idx]))/12.;
	    H3st_qqp_qg[idx]  = (LQF*(12.*eC2qg*gamma1gq[idx] + 6.*eC1qg*(2.*eC1qq*gamma1gq[idx] + 2.*gamma2gq[idx] + gamma1gq[idx]*(beta0*LF + gamma1gg[idx]*LF + 3.*gamma1qq[idx]*LF - 2.*B1q*LQ + 3.*beta0*LQ - gamma1gg[idx]*LQ - 3.*gamma1qq[idx]*LQ - A1q*LQ2))))/12.;

	    H3st_qqbp_qqb[idx] = (LQF*(12.*(2.*eC1qq*gamma2qqp[idx] + gamma3qqp[idx]) + 6.*(2.*eC1qq*gamma1gq[idx]*gamma1qg[idx] + gamma1qg[idx]*gamma2gq[idx] + gamma1gq[idx]*gamma2qg[idx] + 2.*beta0*gamma2qqp[idx] + 4.*gamma1qq[idx]*gamma2qqp[idx])*LF + 2.*gamma1gq[idx]*gamma1qg[idx]*(3.*beta0 + gamma1gg[idx] + 5.*gamma1qq[idx])*LF2 - 2.*(6.*eC1qq*gamma1gq[idx]*gamma1qg[idx] + 3.*gamma1qg[idx]*gamma2gq[idx] + 3.*gamma1gq[idx]*gamma2qg[idx] + 6.*(B1q - beta0 + 2.*gamma1qq[idx])*gamma2qqp[idx] + gamma1gq[idx]*gamma1qg[idx]*(3.*B1q + 2.*gamma1gg[idx] + 10.*gamma1qq[idx])*LF)*LQ + (-6.*A1q*gamma2qqp[idx] + gamma1gq[idx]*gamma1qg[idx]*(6.*B1q - 6.*beta0 + 2.*gamma1gg[idx] + 10.*gamma1qq[idx] - 3.*A1q*LF))*LQ2 + 3.*A1q*gamma1gq[idx]*gamma1qg[idx]*LQ3 ))/12.;
	    H3st_qqbp_qqbp[idx] = (LQF*(12.*(2.*eC2qqp*gamma1qq[idx])))/12.;
	    H3st_qqbp_qg[idx] = (LQF*(12.*(eC2qg*gamma1gq[idx]) + 6.*eC1qg*(2.*gamma2gq[idx] + gamma1gq[idx]*(2.*eC1qq + (beta0 + gamma1gg[idx] + 3.*gamma1qq[idx])*LF - (2.*B1q - 3.*beta0 + gamma1gg[idx] + 3.*gamma1qq[idx])*LQ - A1q*LQ2))))/12.;
	
	    H3st_qg_qqb[idx]   = (LQF*(24.*((pow(eC1qq,2) + 2.*eC2qq)*gamma1qg[idx] + 2.*eC1qq*gamma2qg[idx] + gamma3qg[idx]) + 12.*(beta1 + 2.*beta0*eC1qq)*gamma1qg[idx]*LF + 24.*eC1qq*gamma1gg[idx]*gamma1qg[idx]*LF + 72.*eC1qq*gamma1qg[idx]*gamma1qq[idx]*LF + 12.*gamma1qg[idx]*gamma2gg[idx]*LF + 24.*beta0*gamma2qg[idx]*LF + 12.*gamma1gg[idx]*gamma2qg[idx]*LF + 36.*gamma1qq[idx]*gamma2qg[idx]*LF + 36.*gamma1qg[idx]*gamma2qq[idx]*LF + 12.*gamma1qg[idx]*gamma2qqb[idx]*LF - 12.*gamma1qg[idx]*gamma2qqbp[idx]*LF - 12.*gamma1qg[idx]*gamma2qqp[idx]*LF + 8.*pow(beta0,2)*gamma1qg[idx]*LF2 + 12.*beta0*gamma1gg[idx]*gamma1qg[idx]*LF2 + 4.*pow(gamma1gg[idx],2)*gamma1qg[idx]*LF2 + 12.*gamma1gq[idx]*pow(gamma1qg[idx],2)*LF2 + 36.*beta0*gamma1qg[idx]*gamma1qq[idx]*LF2 + 16.*gamma1gg[idx]*gamma1qg[idx]*gamma1qq[idx]*LF2 + 28.*gamma1qg[idx]*pow(gamma1qq[idx],2)*LF2 - 24.*B2q*gamma1qg[idx]*LQ + 12.*beta1*gamma1qg[idx]*LQ - 48.*B1q*eC1qq*gamma1qg[idx]*LQ + 72.*beta0*eC1qq*gamma1qg[idx]*LQ - 24.*eC1qq*gamma1gg[idx]*gamma1qg[idx]*LQ - 72.*eC1qq*gamma1qg[idx]*gamma1qq[idx]*LQ - 12.*gamma1qg[idx]*gamma2gg[idx]*LQ - 24.*B1q*gamma2qg[idx]*LQ + 24.*beta0*gamma2qg[idx]*LQ - 12.*gamma1gg[idx]*gamma2qg[idx]*LQ - 36.*gamma1qq[idx]*gamma2qg[idx]*LQ - 36.*gamma1qg[idx]*gamma2qq[idx]*LQ - 12.*gamma1qg[idx]*gamma2qqb[idx]*LQ + 12.*gamma1qg[idx]*gamma2qqbp[idx]*LQ + 12.*gamma1qg[idx]*gamma2qqp[idx]*LQ - 12.*B1q*beta0*gamma1qg[idx]*LF*LQ + 8.*pow(beta0,2)*gamma1qg[idx]*LF*LQ - 12.*B1q*gamma1gg[idx]*gamma1qg[idx]*LF*LQ - 8.*pow(gamma1gg[idx],2)*gamma1qg[idx]*LF*LQ - 24.*gamma1gq[idx]*pow(gamma1qg[idx],2)*LF*LQ - 36.*B1q*gamma1qg[idx]*gamma1qq[idx]*LF*LQ - 32.*gamma1gg[idx]*gamma1qg[idx]*gamma1qq[idx]*LF*LQ - 56.*gamma1qg[idx]*pow(gamma1qq[idx],2)*LF*LQ - 12.*A2q*gamma1qg[idx]*LQ2 + 12.*pow(B1q,2)*gamma1qg[idx]*LQ2 - 24.*B1q*beta0*gamma1qg[idx]*LQ2 + 8.*pow(beta0,2)*gamma1qg[idx]*LQ2 - 24.*A1q*eC1qq*gamma1qg[idx]*LQ2 + 12.*B1q*gamma1gg[idx]*gamma1qg[idx]*LQ2 - 12.*beta0*gamma1gg[idx]*gamma1qg[idx]*LQ2 + 4.*pow(gamma1gg[idx],2)*gamma1qg[idx]*LQ2 + 12.*gamma1gq[idx]*pow(gamma1qg[idx],2)*LQ2 + 36.*B1q*gamma1qg[idx]*gamma1qq[idx]*LQ2 - 36.*beta0*gamma1qg[idx]*gamma1qq[idx]*LQ2 + 16.*gamma1gg[idx]*gamma1qg[idx]*gamma1qq[idx]*LQ2 + 28.*gamma1qg[idx]*pow(gamma1qq[idx],2)*LQ2 - 12.*A1q*gamma2qg[idx]*LQ2 - 6.*A1q*beta0*gamma1qg[idx]*LF*LQ2 - 6.*A1q*gamma1gg[idx]*gamma1qg[idx]*LF*LQ2 - 18.*A1q*gamma1qg[idx]*gamma1qq[idx]*LF*LQ2 + 12.*A1q*B1q*gamma1qg[idx]*LQ3 - 14.*A1q*beta0*gamma1qg[idx]*LQ3 + 6.*A1q*gamma1gg[idx]*gamma1qg[idx]*LQ3 + 18.*A1q*gamma1qg[idx]*gamma1qq[idx]*LQ3 + 3.*pow(A1q,2)*gamma1qg[idx]*LQ4 + 4.*gamma1qg[idx]*((3.*gamma2qqbp[idx] + 3.*gamma2qqp[idx] + 2.*gamma1gq[idx]*gamma1qg[idx]*LQF)*LQF)*nf))/24.;
	    H3st_qg_qg[idx]   = (LQF*(24.*eC2qg*(gamma1gg[idx] + gamma1qq[idx]) + 12.*eC1qg*(2.*eC1qq*(gamma1gg[idx] + gamma1qq[idx]) + 2.*(gamma2gg[idx] + gamma2qq[idx]) + pow(gamma1gg[idx],2)*LF + 3.*gamma1gq[idx]*gamma1qg[idx]*LF + 2.*gamma1gg[idx]*gamma1qq[idx]*LF + pow(gamma1qq[idx],2)*LF - 2.*B1q*gamma1gg[idx]*LQ - pow(gamma1gg[idx],2)*LQ - 3.*gamma1gq[idx]*gamma1qg[idx]*LQ - 2.*B1q*gamma1qq[idx]*LQ - 2.*gamma1gg[idx]*gamma1qq[idx]*LQ - pow(gamma1qq[idx],2)*LQ - A1q*gamma1gg[idx]*LQ2 - A1q*gamma1qq[idx]*LQ2 + beta0*(gamma1gg[idx] + gamma1qq[idx])*(LF + 3.*LQ) + 2.*gamma1gq[idx]*gamma1qg[idx]*LQF*nf)))/24.;
	    H3st_qg_gg[idx]   = (LQF*(24.*pow(eC1qg,2)*gamma1gq[idx]))/24.;
	    H3st_qg_qq[idx]   = (LQF*(24.*(eC2qqb*gamma1qg[idx])))/24.;
	    H3st_qg_qqp[idx]  = (LQF*(24.*(-eC2qqbp*gamma1qg[idx]) + 4.*gamma1qg[idx]*(6.*eC2qqbp)*nf))/24.;
	    H3st_qg_qqbp[idx]   = (LQF*(24.*(-eC2qqp*gamma1qg[idx]) + 4.*gamma1qg[idx]*(6.*eC2qqp)*nf))/24.;

	    H3st_qbg_qqb[idx]   = (((gamma1qg[idx]*LQF)*(2.*gamma2qqb[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF))*LQF)/2.;
	    H3st_qbg_qg[idx]   = (((gamma1qg[idx]*LQF*2.*eC1qg*gamma1gq[idx]) +  eC1qg*(2.*gamma2qqb[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF))*LQF)/2.;
	    H3st_qbg_qq[idx]  = ((2.*eC2qqb*gamma1qg[idx])*LQF)/2.;
	    H3st_qbg_gg[idx]   = (((eC1qg)*(2.*eC1qg*gamma1gq[idx]))*LQF)/2.;
	  
	    H3st_qpg_qqb[idx]  = (((gamma1qg[idx]*LQF)*(2.*gamma2qqp[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF))*LQF)/2.;
	    H3st_qpg_qg[idx]  = (((gamma1qg[idx]*LQF*2.*eC1qg*gamma1gq[idx]) +  eC1qg*(2.*gamma2qqp[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF))*LQF)/2.;
	    H3st_qpg_qqbp[idx] = ((2.*eC2qqp*gamma1qg[idx])*LQF)/2.;
	    H3st_qpg_gg[idx]  = (((eC1qg)*(2.*eC1qg*gamma1gq[idx]))*LQF)/2.;

	    H3st_qbpg_qqb[idx]   = (((gamma1qg[idx]*LQF)*(2.*gamma2qqbp[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF))*LQF)/2.;
	    H3st_qbpg_qg[idx]   = ((gamma1qg[idx]*LQF*2.*eC1qg*gamma1gq[idx] + (eC1qg)*(2.*gamma2qqbp[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF))*LQF)/2.;
	    H3st_qbpg_qqp[idx] = ((2.*eC2qqbp*gamma1qg[idx])*LQF)/2.;
	    H3st_qbpg_gg[idx]   = (((eC1qg)*(2.*eC1qg*gamma1gq[idx]))*LQF)/2.;

	    H3st_gg_qqb[idx]   = (LQF*(gamma1qg[idx]*LQF*(4.*eC1qq*gamma1qg[idx] + 4.*gamma2qg[idx] + gamma1qg[idx]*(2.*gamma1gg[idx]*LF + 2.*gamma1qq[idx]*LF - 2.*B1q*LQ - 2.*gamma1gg[idx]*LQ - 2.*gamma1qq[idx]*LQ - A1q*LQ2 + 2.*beta0*(LF + LQ)))))/2.;
	    H3st_gg_qg[idx]   = (LQF*(4.*eC2qg*gamma1qg[idx] + 2.*eC1qg*(2.*eC1qq*gamma1qg[idx] + 2.*gamma2qg[idx] + gamma1qg[idx]*(beta0*LF + 3.*gamma1gg[idx]*LF + gamma1qq[idx]*LF - 2.*B1q*LQ + 3.*beta0*LQ - 3.*gamma1gg[idx]*LQ - gamma1qq[idx]*LQ - A1q*LQ2))))/2.;
	    H3st_gg_gg[idx]   = (LQF*(4.*pow(eC1qg,2)*gamma1gg[idx]))/2.;

	    //Mixed LR*LQF variations
	    H3st_qqb_qqb [idx] += -2*beta0*H2st_qqb_qqb[idx]*LR + (-beta1*LR - pow(beta0,2)*LR2)*H1st_qqb_qqb[idx];
	    H3st_qqb_qg  [idx] += -2*beta0*H2st_qqb_qg[idx]*LR;

	    H3st_qg_qqb  [idx] += -2*beta0*H2st_qg_qqb[idx]*LR  + (-beta1*LR - pow(beta0,2)*LR2)*H1st_qg_qqb[idx];
	    H3st_qg_qg   [idx] += -2*beta0*H2st_qg_qg[idx]*LR;

	    H3st_qq_qqb  [idx] += -2*beta0*H2st_qq_qqb[idx]*LR;
	    H3st_qq_qg   [idx] += -2*beta0*H2st_qq_qg[idx]*LR;

	    H3st_qqp_qqb [idx] += -2*beta0*H2st_qqp_qqb[idx]*LR;
	    H3st_qqp_qg  [idx] += -2*beta0*H2st_qqp_qg[idx]*LR;

	    H3st_qqbp_qqb[idx] += -2*beta0*H2st_qqbp_qqb[idx]*LR;
	    H3st_qqbp_qg [idx] += -2*beta0*H2st_qqbp_qg[idx]*LR;

	    H3st_gg_qqb  [idx] += -2*beta0*H2st_gg_qqb[idx]*LR;
	    H3st_gg_qg   [idx] += -2*beta0*H2st_gg_qg[idx]*LR;

	    qqb_eqqb[idx]  += as3*H3st_qqb_qqb[idx];
	    qg_eqqb[idx]   += as3*H3st_qg_qqb[idx];
	    qq_eqqb[idx]   += as3*H3st_qq_qqb[idx];
	    qqp_eqqb[idx]  += as3*H3st_qqp_qqb[idx];
	    qqbp_eqqb[idx] += as3*H3st_qqbp_qqb[idx];
	    gg_eqqb[idx]   += as3*H3st_gg_qqb[idx];
	    qbg_eqqb[idx]  += as3*H3st_qbg_qqb[idx];
	    qpg_eqqb[idx]  += as3*H3st_qpg_qqb[idx];
	    qbpg_eqqb[idx] += as3*H3st_qbpg_qqb[idx];

	    qqb_eqg[idx]  += as3*H3st_qqb_qg[idx];
	    qg_eqg[idx]   += as3*H3st_qg_qg[idx];
	    qq_eqg[idx]   += as3*H3st_qq_qg[idx];
	    qqp_eqg[idx]  += as3*H3st_qqp_qg[idx];
	    qqbp_eqg[idx] += as3*H3st_qqbp_qg[idx];
	    gg_eqg[idx]   += as3*H3st_gg_qg[idx];
	    qbg_eqg[idx]  += as3*H3st_qbg_qg[idx];
	    qpg_eqg[idx]  += as3*H3st_qpg_qg[idx];
	    qbpg_eqg[idx] += as3*H3st_qbpg_qg[idx];

	    qg_eqq[idx]   += as3*H3st_qg_qq[idx];
	    qq_eqq[idx]   += as3*H3st_qq_qq[idx];
	    qbg_eqq[idx]  += as3*H3st_qbg_qq[idx];

	    qg_eqqp[idx]   += as3*H3st_qg_qqp[idx];
	    qqp_eqqp[idx] += as3*H3st_qqp_qqp[idx];
	    qbpg_eqqp[idx]  += as3*H3st_qbpg_qqp[idx];

	    qg_eqqbp[idx]    += as3*H3st_qg_qqbp[idx];
	    qqbp_eqqbp[idx]   += as3*H3st_qqbp_qqbp[idx];
	    qpg_eqqbp[idx]  += as3*H3st_qpg_qqbp[idx];

	    qg_egg[idx]   += as3*H3st_qg_gg[idx];
	    gg_egg[idx]   += as3*H3st_gg_gg[idx];
	    qbg_egg[idx]  += as3*H3st_qbg_gg[idx];
	    qpg_egg[idx]  += as3*H3st_qpg_gg[idx];
	    qbpg_egg[idx] += as3*H3st_qbpg_gg[idx];
	  }
    }
  //NNNNLO
  if (max(opts.order_hcoef,opts.order_evol) >= 4)
    {
      if (opts.order_hcoef < 4)
	{
	  LR  = 0;
	  LR2 = 0;
	  LR3 = 0;
	  LF  = 0;
	  LF2 = 0;
	  LF3 = 0;
	  LF4 = 0;
	  LQF  = -LQ;
	  LQF2 = pow(LQF,2);
	  LQF3 = pow(LQF,3);
	  LQF3 = pow(LQF,4);
	}
	  
      for (int sign = mesq::positive; sign <= mesq::negative; sign++)
	for (int i = 0; i < mellinint::mdim; i++)
	  {
	    int idx = pmom::index(i,sign);
	    double nf = 5.;
	    
	    //complex <double> H4st_qqb_qqb=+pow(gamma2qqbp[idx],2)*LF2+pow(gamma2qqbp[idx],2)*LQ2-2.*pow(gamma2qqbp[idx],2)*LQ*LF+pow(gamma2qqp[idx],2)*LF2+pow(gamma2qqp[idx],2)*LQ2-2.*pow(gamma2qqp[idx],2)*LQ*LF+pow(gamma2qqb[idx],2)*LF2+pow(gamma2qqb[idx],2)*LQ2-2.*pow(gamma2qqb[idx],2)*LQ*LF+gamma2qg[idx]*gamma1gq[idx]*LF2+gamma2qg[idx]*gamma1gq[idx]*LQ2-2.*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+2.*gamma1qg[idx]*gamma3gq[idx]*LF2+2.*gamma1qg[idx]*gamma3gq[idx]*LQ2-2.*gamma1qg[idx]*gamma3gq[idx]*LQ*LF+gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LF3-gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ3+gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ2*LF-2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ*LF2+1./12.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LF4+1./12.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ4-1./3.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ3*LF+1./2.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ2*LF2-1./3.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ*LF3+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+2.*gamma4qq[idx]*LF-2.*gamma4qq[idx]*LQ+2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+4.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-5.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+2.*pow(gamma2qq[idx],2)*LF2+2.*pow(gamma2qq[idx],2)*LQ2-4.*pow(gamma2qq[idx],2)*LQ*LF+2.*gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-2.*gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF3-3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ3+5.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-6.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4+1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4-2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+3.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+4.*gamma1qq[idx]*gamma3qq[idx]*LF2+4.*gamma1qq[idx]*gamma3qq[idx]*LQ2-6.*gamma1qq[idx]*gamma3qq[idx]*LQ*LF+17./12.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF4+17./12.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-17./3.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+17./2.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-17./3.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+4.*pow(gamma1qq[idx],2)*gamma2qq[idx]*LF3-4.*pow(gamma1qq[idx],2)*gamma2qq[idx]*LQ3+10.*pow(gamma1qq[idx],2)*gamma2qq[idx]*LQ2*LF-11.*pow(gamma1qq[idx],2)*gamma2qq[idx]*LQ*LF2+2./3.*pow(gamma1qq[idx],4)*LF4+2./3.*pow(gamma1qq[idx],4)*LQ4-8./3.*pow(gamma1qq[idx],4)*LQ3*LF+4.*pow(gamma1qq[idx],4)*LQ2*LF2-8./3.*pow(gamma1qq[idx],4)*LQ*LF3+2.*C3qq[idx]*gamma1qq[idx]*LF-2.*C3qq[idx]*gamma1qq[idx]*LQ+C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-2.*C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+2.*C2qq[idx]*gamma2qq[idx]*LF-2.*C2qq[idx]*gamma2qq[idx]*LQ+3.*C2qq[idx]*pow(gamma1qq[idx],2)*LF2+3.*C2qq[idx]*pow(gamma1qq[idx],2)*LQ2-6.*C2qq[idx]*pow(gamma1qq[idx],2)*LQ*LF-2.*C1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+2.*C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+2.*C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2-2.*C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+1./3.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-1./3.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+2.*C1qq[idx]*gamma3qq[idx]*LF-2.*C1qq[idx]*gamma3qq[idx]*LQ+8./3.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-8./3.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+8.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-8.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+6.*C1qq[idx]*gamma1qq[idx]*gamma2qq[idx]*LF2+6.*C1qq[idx]*gamma1qq[idx]*gamma2qq[idx]*LQ2-12.*C1qq[idx]*gamma1qq[idx]*gamma2qq[idx]*LQ*LF+7./3.*C1qq[idx]*pow(gamma1qq[idx],3)*LF3-7./3.*C1qq[idx]*pow(gamma1qq[idx],3)*LQ3+7.*C1qq[idx]*pow(gamma1qq[idx],3)*LQ2*LF-7.*C1qq[idx]*pow(gamma1qq[idx],3)*LQ*LF2+4.*C1qq[idx]*C2qq[idx]*gamma1qq[idx]*LF-4.*C1qq[idx]*C2qq[idx]*gamma1qq[idx]*LQ+pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF2+pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2-2.*pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+2.*pow(C1qq[idx],2)*gamma2qq[idx]*LF-2.*pow(C1qq[idx],2)*gamma2qq[idx]*LQ+2.*pow(C1qq[idx],2)*pow(gamma1qq[idx],2)*LF2+2.*pow(C1qq[idx],2)*pow(gamma1qq[idx],2)*LQ2-4.*pow(C1qq[idx],2)*pow(gamma1qq[idx],2)*LQ*LF+1./6.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./6.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-2./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-2./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+beta2*gamma1qq[idx]*LF2-beta2*gamma1qq[idx]*LQ2+beta1*gamma1qg[idx]*gamma1gq[idx]*LF3+beta1*gamma1qg[idx]*gamma1gq[idx]*LQ3-beta1*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-beta1*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+2.*beta1*gamma2qq[idx]*LF2-2.*beta1*gamma2qq[idx]*LQ2+2.*beta1*pow(gamma1qq[idx],2)*LF3+2.*beta1*pow(gamma1qq[idx],2)*LQ3-2.*beta1*pow(gamma1qq[idx],2)*LQ2*LF-2.*beta1*pow(gamma1qq[idx],2)*LQ*LF2+2.*beta1*C1qq[idx]*gamma1qq[idx]*LF2-6.*beta1*C1qq[idx]*gamma1qq[idx]*LQ2+4.*beta1*C1qq[idx]*gamma1qq[idx]*LQ*LF-2.*beta0*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-beta0*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+3.*beta0*gamma1qg[idx]*gamma2gq[idx]*LF3+3.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ3-beta0*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4-1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF-beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+3.*beta0*gamma3qq[idx]*LF2-3.*beta0*gamma3qq[idx]*LQ2+5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF4-5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+5.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-5.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+6.*beta0*gamma1qq[idx]*gamma2qq[idx]*LF3+6.*beta0*gamma1qq[idx]*gamma2qq[idx]*LQ3-6.*beta0*gamma1qq[idx]*gamma2qq[idx]*LQ2*LF-6.*beta0*gamma1qq[idx]*gamma2qq[idx]*LQ*LF2+2.*beta0*pow(gamma1qq[idx],3)*LF4-2.*beta0*pow(gamma1qq[idx],3)*LQ4+4.*beta0*pow(gamma1qq[idx],3)*LQ3*LF-4.*beta0*pow(gamma1qq[idx],3)*LQ*LF3+2.*beta0*C2qq[idx]*gamma1qq[idx]*LF2-10.*beta0*C2qq[idx]*gamma1qq[idx]*LQ2+8.*beta0*C2qq[idx]*gamma1qq[idx]*LQ*LF+2.*beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+4.*beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3-6.*beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+4.*beta0*C1qq[idx]*gamma2qq[idx]*LF2-8.*beta0*C1qq[idx]*gamma2qq[idx]*LQ2+4.*beta0*C1qq[idx]*gamma2qq[idx]*LQ*LF+4.*beta0*C1qq[idx]*pow(gamma1qq[idx],2)*LF3+8.*beta0*C1qq[idx]*pow(gamma1qq[idx],2)*LQ3-12.*beta0*C1qq[idx]*pow(gamma1qq[idx],2)*LQ2*LF+beta0*pow(C1qq[idx],2)*gamma1qq[idx]*LF2-5.*beta0*pow(C1qq[idx],2)*gamma1qq[idx]*LQ2+4.*beta0*pow(C1qq[idx],2)*gamma1qq[idx]*LQ*LF+5./3.*beta0*beta1*gamma1qq[idx]*LF3-5./3.*beta0*beta1*gamma1qq[idx]*LQ3+11./12.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LF4+11./12.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-2./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./2.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-2./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+2.*pow(beta0,2)*gamma2qq[idx]*LF3-2.*pow(beta0,2)*gamma2qq[idx]*LQ3+11./6.*pow(beta0,2)*pow(gamma1qq[idx],2)*LF4+11./6.*pow(beta0,2)*pow(gamma1qq[idx],2)*LQ4-4./3.*pow(beta0,2)*pow(gamma1qq[idx],2)*LQ3*LF-pow(beta0,2)*pow(gamma1qq[idx],2)*LQ2*LF2-4./3.*pow(beta0,2)*pow(gamma1qq[idx],2)*LQ*LF3+4./3.*pow(beta0,2)*C1qq[idx]*gamma1qq[idx]*LF3-22./3.*pow(beta0,2)*C1qq[idx]*gamma1qq[idx]*LQ3+4.*pow(beta0,2)*C1qq[idx]*gamma1qq[idx]*LQ2*LF+2.*pow(beta0,2)*C1qq[idx]*gamma1qq[idx]*LQ*LF2+1./2.*pow(beta0,3)*gamma1qq[idx]*LF4-1./2.*pow(beta0,3)*gamma1qq[idx]*LQ4+2.*B3*gamma1qq[idx]*LQ2-2.*B3*gamma1qq[idx]*LQ*LF-B2*gamma1qg[idx]*gamma1gq[idx]*LQ3+2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-B2*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+2.*B2*gamma2qq[idx]*LQ2-2.*B2*gamma2qq[idx]*LQ*LF-2.*B2*pow(gamma1qq[idx],2)*LQ3+4.*B2*pow(gamma1qq[idx],2)*LQ2*LF-2.*B2*pow(gamma1qq[idx],2)*LQ*LF2+4.*B2*C1qq[idx]*gamma1qq[idx]*LQ2-4.*B2*C1qq[idx]*gamma1qq[idx]*LQ*LF+3.*B2*beta0*gamma1qq[idx]*LQ3-2.*B2*beta0*gamma1qq[idx]*LQ2*LF-B2*beta0*gamma1qq[idx]*LQ*LF2+2.*B1*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-2.*B1*gamma1qg[idx]*gamma2gq[idx]*LQ3+2.*B1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-2.*B1*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./3.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4-B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-1./3.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+2.*B1*gamma3qq[idx]*LQ2-2.*B1*gamma3qq[idx]*LQ*LF+5./3.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4-5.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+5.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-5./3.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3-4.*B1*gamma1qq[idx]*gamma2qq[idx]*LQ3+8.*B1*gamma1qq[idx]*gamma2qq[idx]*LQ2*LF-4.*B1*gamma1qq[idx]*gamma2qq[idx]*LQ*LF2+4./3.*B1*pow(gamma1qq[idx],3)*LQ4-4.*B1*pow(gamma1qq[idx],3)*LQ3*LF+4.*B1*pow(gamma1qq[idx],3)*LQ2*LF2-4./3.*B1*pow(gamma1qq[idx],3)*LQ*LF3+4.*B1*C2qq[idx]*gamma1qq[idx]*LQ2-4.*B1*C2qq[idx]*gamma1qq[idx]*LQ*LF-2.*B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+4.*B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-2.*B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+4.*B1*C1qq[idx]*gamma2qq[idx]*LQ2-4.*B1*C1qq[idx]*gamma2qq[idx]*LQ*LF-4.*B1*C1qq[idx]*pow(gamma1qq[idx],2)*LQ3+8.*B1*C1qq[idx]*pow(gamma1qq[idx],2)*LQ2*LF-4.*B1*C1qq[idx]*pow(gamma1qq[idx],2)*LQ*LF2+2.*B1*pow(C1qq[idx],2)*gamma1qq[idx]*LQ2-2.*B1*pow(C1qq[idx],2)*gamma1qq[idx]*LQ*LF+2.*B1*beta1*gamma1qq[idx]*LQ3-B1*beta1*gamma1qq[idx]*LQ2*LF-B1*beta1*gamma1qq[idx]*LQ*LF2-3./2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4+2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+1./2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+3.*B1*beta0*gamma2qq[idx]*LQ3-B1*beta0*gamma2qq[idx]*LQ2*LF-2.*B1*beta0*gamma2qq[idx]*LQ*LF2-3.*B1*beta0*pow(gamma1qq[idx],2)*LQ4+4.*B1*beta0*pow(gamma1qq[idx],2)*LQ3*LF+B1*beta0*pow(gamma1qq[idx],2)*LQ2*LF2-2.*B1*beta0*pow(gamma1qq[idx],2)*LQ*LF3+8.*B1*beta0*C1qq[idx]*gamma1qq[idx]*LQ3-6.*B1*beta0*C1qq[idx]*gamma1qq[idx]*LQ2*LF-2.*B1*beta0*C1qq[idx]*gamma1qq[idx]*LQ*LF2+11./6.*B1*pow(beta0,2)*gamma1qq[idx]*LQ4-2./3.*B1*pow(beta0,2)*gamma1qq[idx]*LQ3*LF-1./2.*B1*pow(beta0,2)*gamma1qq[idx]*LQ2*LF2-2./3.*B1*pow(beta0,2)*gamma1qq[idx]*LQ*LF3-2.*B1*B2*gamma1qq[idx]*LQ3+2.*B1*B2*gamma1qq[idx]*LQ2*LF+1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-pow(B1,2)*gamma2qq[idx]*LQ3+pow(B1,2)*gamma2qq[idx]*LQ2*LF+pow(B1,2)*pow(gamma1qq[idx],2)*LQ4-2.*pow(B1,2)*pow(gamma1qq[idx],2)*LQ3*LF+pow(B1,2)*pow(gamma1qq[idx],2)*LQ2*LF2-2.*pow(B1,2)*C1qq[idx]*gamma1qq[idx]*LQ3+2.*pow(B1,2)*C1qq[idx]*gamma1qq[idx]*LQ2*LF-3./2.*pow(B1,2)*beta0*gamma1qq[idx]*LQ4+pow(B1,2)*beta0*gamma1qq[idx]*LQ3*LF+1./2.*pow(B1,2)*beta0*gamma1qq[idx]*LQ2*LF2+1./3.*pow(B1,3)*gamma1qq[idx]*LQ4-1./3.*pow(B1,3)*gamma1qq[idx]*LQ3*LF+A3*gamma1qq[idx]*LQ3-A3*gamma1qq[idx]*LQ2*LF-1./2.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ4+A2*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./2.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+A2*gamma2qq[idx]*LQ3-A2*gamma2qq[idx]*LQ2*LF-A2*pow(gamma1qq[idx],2)*LQ4+2.*A2*pow(gamma1qq[idx],2)*LQ3*LF-A2*pow(gamma1qq[idx],2)*LQ2*LF2+2.*A2*C1qq[idx]*gamma1qq[idx]*LQ3-2.*A2*C1qq[idx]*gamma1qq[idx]*LQ2*LF+11./6.*A2*beta0*gamma1qq[idx]*LQ4-4./3.*A2*beta0*gamma1qq[idx]*LQ3*LF-1./2.*A2*beta0*gamma1qq[idx]*LQ2*LF2-A2*B1*gamma1qq[idx]*LQ4+A2*B1*gamma1qq[idx]*LQ3*LF+A1*gamma2qg[idx]*gamma1gq[idx]*LQ3*LF-A1*gamma1qg[idx]*gamma2gq[idx]*LQ4+A1*gamma1qg[idx]*gamma2gq[idx]*LQ3*LF-A1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF2+1./6.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ5-1./2.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4*LF+1./2.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF2-1./6.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF3+A1*gamma3qq[idx]*LQ3-A1*gamma3qq[idx]*LQ2*LF+5./6.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ5-5./2.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+5./2.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-5./6.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3-2.*A1*gamma1qq[idx]*gamma2qq[idx]*LQ4+4.*A1*gamma1qq[idx]*gamma2qq[idx]*LQ3*LF-2.*A1*gamma1qq[idx]*gamma2qq[idx]*LQ2*LF2+2./3.*A1*pow(gamma1qq[idx],3)*LQ5-2.*A1*pow(gamma1qq[idx],3)*LQ4*LF+2.*A1*pow(gamma1qq[idx],3)*LQ3*LF2-2./3.*A1*pow(gamma1qq[idx],3)*LQ2*LF3+2.*A1*C2qq[idx]*gamma1qq[idx]*LQ3-2.*A1*C2qq[idx]*gamma1qq[idx]*LQ2*LF-A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+2.*A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+2.*A1*C1qq[idx]*gamma2qq[idx]*LQ3-2.*A1*C1qq[idx]*gamma2qq[idx]*LQ2*LF-2.*A1*C1qq[idx]*pow(gamma1qq[idx],2)*LQ4+4.*A1*C1qq[idx]*pow(gamma1qq[idx],2)*LQ3*LF-2.*A1*C1qq[idx]*pow(gamma1qq[idx],2)*LQ2*LF2+A1*pow(C1qq[idx],2)*gamma1qq[idx]*LQ3-A1*pow(C1qq[idx],2)*gamma1qq[idx]*LQ2*LF+7./6.*A1*beta1*gamma1qq[idx]*LQ4-2./3.*A1*beta1*gamma1qq[idx]*LQ3*LF-1./2.*A1*beta1*gamma1qq[idx]*LQ2*LF2-5./6.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ5+7./6.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+1./6.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-1./2.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3+5./3.*A1*beta0*gamma2qq[idx]*LQ4-2./3.*A1*beta0*gamma2qq[idx]*LQ3*LF-A1*beta0*gamma2qq[idx]*LQ2*LF2-5./3.*A1*beta0*pow(gamma1qq[idx],2)*LQ5+7./3.*A1*beta0*pow(gamma1qq[idx],2)*LQ4*LF+1./3.*A1*beta0*pow(gamma1qq[idx],2)*LQ3*LF2-A1*beta0*pow(gamma1qq[idx],2)*LQ2*LF3+13./3.*A1*beta0*C1qq[idx]*gamma1qq[idx]*LQ4-10./3.*A1*beta0*C1qq[idx]*gamma1qq[idx]*LQ3*LF-A1*beta0*C1qq[idx]*gamma1qq[idx]*LQ2*LF2+7./6.*A1*pow(beta0,2)*gamma1qq[idx]*LQ5-1./2.*A1*pow(beta0,2)*gamma1qq[idx]*LQ4*LF-1./3.*A1*pow(beta0,2)*gamma1qq[idx]*LQ3*LF2-1./3.*A1*pow(beta0,2)*gamma1qq[idx]*LQ2*LF3-A1*B2*gamma1qq[idx]*LQ4+A1*B2*gamma1qq[idx]*LQ3*LF+1./2.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ5-A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+1./2.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-A1*B1*gamma2qq[idx]*LQ4+A1*B1*gamma2qq[idx]*LQ3*LF+A1*B1*pow(gamma1qq[idx],2)*LQ5-2.*A1*B1*pow(gamma1qq[idx],2)*LQ4*LF+A1*B1*pow(gamma1qq[idx],2)*LQ3*LF2-2.*A1*B1*C1qq[idx]*gamma1qq[idx]*LQ4+2.*A1*B1*C1qq[idx]*gamma1qq[idx]*LQ3*LF-5./3.*A1*B1*beta0*gamma1qq[idx]*LQ5+7./6.*A1*B1*beta0*gamma1qq[idx]*LQ4*LF+1./2.*A1*B1*beta0*gamma1qq[idx]*LQ3*LF2+1./2.*A1*pow(B1,2)*gamma1qq[idx]*LQ5-1./2.*A1*pow(B1,2)*gamma1qq[idx]*LQ4*LF-1./2.*A1*A2*gamma1qq[idx]*LQ5+1./2.*A1*A2*gamma1qq[idx]*LQ4*LF+1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ6-1./4.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ5*LF+1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF2-1./4.*pow(A1,2)*gamma2qq[idx]*LQ5+1./4.*pow(A1,2)*gamma2qq[idx]*LQ4*LF+1./4.*pow(A1,2)*pow(gamma1qq[idx],2)*LQ6-1./2.*pow(A1,2)*pow(gamma1qq[idx],2)*LQ5*LF+1./4.*pow(A1,2)*pow(gamma1qq[idx],2)*LQ4*LF2-1./2.*pow(A1,2)*C1qq[idx]*gamma1qq[idx]*LQ5+1./2.*pow(A1,2)*C1qq[idx]*gamma1qq[idx]*LQ4*LF-11./24.*pow(A1,2)*beta0*gamma1qq[idx]*LQ6+1./3.*pow(A1,2)*beta0*gamma1qq[idx]*LQ5*LF+1./8.*pow(A1,2)*beta0*gamma1qq[idx]*LQ4*LF2+1./4.*pow(A1,2)*B1*gamma1qq[idx]*LQ6-1./4.*pow(A1,2)*B1*gamma1qq[idx]*LQ5*LF+1./24.*pow(A1,3)*gamma1qq[idx]*LQ7-1./24.*pow(A1,3)*gamma1qq[idx]*LQ6*LF;
	    //complex <double> H4st_qg_qqb=+gamma4qg[idx]*LF-gamma4qg[idx]*LQ+1./2.*gamma2qg[idx]*gamma2qqbp[idx]*LF2+1./2.*gamma2qg[idx]*gamma2qqbp[idx]*LQ2-gamma2qg[idx]*gamma2qqbp[idx]*LQ*LF+1./2.*gamma2qg[idx]*gamma2qqp[idx]*LF2+1./2.*gamma2qg[idx]*gamma2qqp[idx]*LQ2-gamma2qg[idx]*gamma2qqp[idx]*LQ*LF+1./2.*gamma2qg[idx]*gamma2qqb[idx]*LF2+1./2.*gamma2qg[idx]*gamma2qqb[idx]*LQ2-gamma2qg[idx]*gamma2qqb[idx]*LQ*LF+1./2.*gamma2qg[idx]*gamma1gg[idx]*LF2+1./2.*gamma2qg[idx]*gamma1gg[idx]*LQ2-gamma2qg[idx]*gamma1gg[idx]*LQ*LF+gamma1qg[idx]*gamma3gg[idx]*LF2+gamma1qg[idx]*gamma3gg[idx]*LQ2-gamma1qg[idx]*gamma3gg[idx]*LQ*LF+1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2gg[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2gg[idx]*LQ3+1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2gg[idx]*LQ2*LF-gamma1qg[idx]*gamma1gg[idx]*gamma2gg[idx]*LQ*LF2+1./24.*gamma1qg[idx]*pow(gamma1gg[idx],3)*LF4+1./24.*gamma1qg[idx]*pow(gamma1gg[idx],3)*LQ4-1./6.*gamma1qg[idx]*pow(gamma1gg[idx],3)*LQ3*LF+1./4.*gamma1qg[idx]*pow(gamma1gg[idx],3)*LQ2*LF2-1./6.*gamma1qg[idx]*pow(gamma1gg[idx],3)*LQ*LF3+1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF3-1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ3+5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+pow(gamma1qg[idx],2)*gamma2gq[idx]*LF3-pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ3+2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ2*LF-2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ*LF2+5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF4+5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ4-5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+5./2.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+gamma3qq[idx]*gamma1qg[idx]*LF2+gamma3qq[idx]*gamma1qg[idx]*LQ2-2.*gamma3qq[idx]*gamma1qg[idx]*LQ*LF+3./2.*gamma2qq[idx]*gamma2qg[idx]*LF2+3./2.*gamma2qq[idx]*gamma2qg[idx]*LQ2-3.*gamma2qq[idx]*gamma2qg[idx]*LQ*LF+1./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF3-1./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3+3./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF-3./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2+2.*gamma1qq[idx]*gamma3qg[idx]*LF2+2.*gamma1qq[idx]*gamma3qg[idx]*LQ2-3.*gamma1qq[idx]*gamma3qg[idx]*LQ*LF+gamma1qq[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ2*LF-gamma1qq[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ*LF2+gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF-gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF2+gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF-gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF2+gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF-gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF2+3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LF3-3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ3+5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ2*LF-3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ*LF2+5./24.*gamma1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LF4+5./24.*gamma1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ4-5./6.*gamma1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3*LF+5./4.*gamma1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF2-5./6.*gamma1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF3+7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4+7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4-7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+7./2.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2-7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+3./2.*gamma1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LF3-3./2.*gamma1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ3+11./2.*gamma1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ2*LF-11./2.*gamma1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ*LF2+2.*pow(gamma1qq[idx],2)*gamma2qg[idx]*LF3-2.*pow(gamma1qq[idx],2)*gamma2qg[idx]*LQ3+4.*pow(gamma1qq[idx],2)*gamma2qg[idx]*LQ2*LF-9./2.*pow(gamma1qq[idx],2)*gamma2qg[idx]*LQ*LF2+11./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LF4+11./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ4-11./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF+11./4.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2-11./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ*LF3+5./8.*pow(gamma1qq[idx],3)*gamma1qg[idx]*LF4+5./8.*pow(gamma1qq[idx],3)*gamma1qg[idx]*LQ4-5./2.*pow(gamma1qq[idx],3)*gamma1qg[idx]*LQ3*LF+15./4.*pow(gamma1qq[idx],3)*gamma1qg[idx]*LQ2*LF2-5./2.*pow(gamma1qq[idx],3)*gamma1qg[idx]*LQ*LF3+C3qq[idx]*gamma1qg[idx]*LF-C3qq[idx]*gamma1qg[idx]*LQ+C2qq[idx]*gamma2qg[idx]*LF-C2qq[idx]*gamma2qg[idx]*LQ+1./2.*C2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+1./2.*C2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2-C2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+5./2.*C2qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+5./2.*C2qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2-5.*C2qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+C1qq[idx]*gamma3qg[idx]*LF-C1qq[idx]*gamma3qg[idx]*LQ-C1qq[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ*LF-C1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF-C1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF-C1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF+C1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LF2+C1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ2-C1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ*LF+1./6.*C1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LF3-1./6.*C1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3+1./2.*C1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF-1./2.*C1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF2+C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3-C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+3.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF-3.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+2.*C1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LF2+2.*C1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ2-5.*C1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ*LF+3.*C1qq[idx]*gamma1qq[idx]*gamma2qg[idx]*LF2+3.*C1qq[idx]*gamma1qq[idx]*gamma2qg[idx]*LQ2-5.*C1qq[idx]*gamma1qq[idx]*gamma2qg[idx]*LQ*LF+7./6.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF3-7./6.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3+7./2.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF-7./2.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2+13./6.*C1qq[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LF3-13./6.*C1qq[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ3+13./2.*C1qq[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ2*LF-13./2.*C1qq[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ*LF2+2.*C1qq[idx]*C2qq[idx]*gamma1qg[idx]*LF-2.*C1qq[idx]*C2qq[idx]*gamma1qg[idx]*LQ+pow(C1qq[idx],2)*gamma2qg[idx]*LF-pow(C1qq[idx],2)*gamma2qg[idx]*LQ+1./2.*pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LF2+1./2.*pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ2-pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+3./2.*pow(C1qq[idx],2)*gamma1qq[idx]*gamma1qg[idx]*LF2+3./2.*pow(C1qq[idx],2)*gamma1qq[idx]*gamma1qg[idx]*LQ2-3.*pow(C1qq[idx],2)*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+nf*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF3-nf*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ3+nf*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-2.*nf*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+1./12.*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF4+1./12.*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ4-1./3.*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+1./2.*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-1./3.*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+1./2.*nf*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4+1./2.*nf*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4-2.*nf*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+3.*nf*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2-2.*nf*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+1./3.*nf*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3-1./3.*nf*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+nf*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF-nf*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+1./2.*beta2*gamma1qg[idx]*LF2-1./2.*beta2*gamma1qg[idx]*LQ2+beta1*gamma2qg[idx]*LF2-beta1*gamma2qg[idx]*LQ2+1./2.*beta1*gamma1qg[idx]*gamma1gg[idx]*LF3+1./2.*beta1*gamma1qg[idx]*gamma1gg[idx]*LQ3-1./2.*beta1*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF-1./2.*beta1*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2+3./2.*beta1*gamma1qq[idx]*gamma1qg[idx]*LF3+3./2.*beta1*gamma1qq[idx]*gamma1qg[idx]*LQ3-3./2.*beta1*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF-3./2.*beta1*gamma1qq[idx]*gamma1qg[idx]*LQ*LF2+beta1*C1qq[idx]*gamma1qg[idx]*LF2-3.*beta1*C1qq[idx]*gamma1qg[idx]*LQ2+2.*beta1*C1qq[idx]*gamma1qg[idx]*LQ*LF+3./2.*beta0*gamma3qg[idx]*LF2-3./2.*beta0*gamma3qg[idx]*LQ2-beta0*gamma2qg[idx]*gamma1gg[idx]*LQ2*LF-1./2.*beta0*gamma2qg[idx]*gamma1gg[idx]*LQ*LF2-beta0*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF-1./2.*beta0*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF2-beta0*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF-1./2.*beta0*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF2-beta0*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF-1./2.*beta0*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF2+3./2.*beta0*gamma1qg[idx]*gamma2gg[idx]*LF3+3./2.*beta0*gamma1qg[idx]*gamma2gg[idx]*LQ3-1./2.*beta0*gamma1qg[idx]*gamma2gg[idx]*LQ2*LF-beta0*gamma1qg[idx]*gamma2gg[idx]*LQ*LF2+1./4.*beta0*gamma1qg[idx]*pow(gamma1gg[idx],2)*LF4-1./4.*beta0*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ4+1./2.*beta0*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3*LF-1./2.*beta0*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF3+3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4-3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4+3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF-3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+3./2.*beta0*gamma2qq[idx]*gamma1qg[idx]*LF3+3./2.*beta0*gamma2qq[idx]*gamma1qg[idx]*LQ3-5./2.*beta0*gamma2qq[idx]*gamma1qg[idx]*LQ2*LF-2.*beta0*gamma2qq[idx]*gamma1qg[idx]*LQ*LF2+3.*beta0*gamma1qq[idx]*gamma2qg[idx]*LF3+3.*beta0*gamma1qq[idx]*gamma2qg[idx]*LQ3-2.*beta0*gamma1qq[idx]*gamma2qg[idx]*LQ2*LF-5./2.*beta0*gamma1qq[idx]*gamma2qg[idx]*LQ*LF2+beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF4-beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ4+2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF-2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF3+7./4.*beta0*pow(gamma1qq[idx],2)*gamma1qg[idx]*LF4-7./4.*beta0*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ4+7./2.*beta0*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ3*LF-7./2.*beta0*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ*LF3+beta0*C2qq[idx]*gamma1qg[idx]*LF2-5.*beta0*C2qq[idx]*gamma1qg[idx]*LQ2+4.*beta0*C2qq[idx]*gamma1qg[idx]*LQ*LF+2.*beta0*C1qq[idx]*gamma2qg[idx]*LF2-4.*beta0*C1qq[idx]*gamma2qg[idx]*LQ2+2.*beta0*C1qq[idx]*gamma2qg[idx]*LQ*LF+beta0*C1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF3+2.*beta0*C1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3-3.*beta0*C1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF+3.*beta0*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LF3+6.*beta0*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3-9.*beta0*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF+1./2.*beta0*pow(C1qq[idx],2)*gamma1qg[idx]*LF2-5./2.*beta0*pow(C1qq[idx],2)*gamma1qg[idx]*LQ2+2.*beta0*pow(C1qq[idx],2)*gamma1qg[idx]*LQ*LF+1./2.*beta0*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4-1./2.*beta0*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4+beta0*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF-beta0*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+5./6.*beta0*beta1*gamma1qg[idx]*LF3-5./6.*beta0*beta1*gamma1qg[idx]*LQ3+pow(beta0,2)*gamma2qg[idx]*LF3-pow(beta0,2)*gamma2qg[idx]*LQ3+11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gg[idx]*LF4+11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gg[idx]*LQ4-1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF-1./4.*pow(beta0,2)*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2-1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gg[idx]*LQ*LF3+11./8.*pow(beta0,2)*gamma1qq[idx]*gamma1qg[idx]*LF4+11./8.*pow(beta0,2)*gamma1qq[idx]*gamma1qg[idx]*LQ4-pow(beta0,2)*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF-3./4.*pow(beta0,2)*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2-pow(beta0,2)*gamma1qq[idx]*gamma1qg[idx]*LQ*LF3+2./3.*pow(beta0,2)*C1qq[idx]*gamma1qg[idx]*LF3-11./3.*pow(beta0,2)*C1qq[idx]*gamma1qg[idx]*LQ3+2.*pow(beta0,2)*C1qq[idx]*gamma1qg[idx]*LQ2*LF+pow(beta0,2)*C1qq[idx]*gamma1qg[idx]*LQ*LF2+1./4.*pow(beta0,3)*gamma1qg[idx]*LF4-1./4.*pow(beta0,3)*gamma1qg[idx]*LQ4+B3*gamma1qg[idx]*LQ2-B3*gamma1qg[idx]*LQ*LF+B2*gamma2qg[idx]*LQ2-B2*gamma2qg[idx]*LQ*LF-1./2.*B2*gamma1qg[idx]*gamma1gg[idx]*LQ3+B2*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF-1./2.*B2*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2-3./2.*B2*gamma1qq[idx]*gamma1qg[idx]*LQ3+3.*B2*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF-3./2.*B2*gamma1qq[idx]*gamma1qg[idx]*LQ*LF2+2.*B2*C1qq[idx]*gamma1qg[idx]*LQ2-2.*B2*C1qq[idx]*gamma1qg[idx]*LQ*LF+3./2.*B2*beta0*gamma1qg[idx]*LQ3-B2*beta0*gamma1qg[idx]*LQ2*LF-1./2.*B2*beta0*gamma1qg[idx]*LQ*LF2+B1*gamma3qg[idx]*LQ2-B1*gamma3qg[idx]*LQ*LF+B1*gamma2qg[idx]*gamma1gg[idx]*LQ2*LF+B1*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF+B1*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF+B1*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF-B1*gamma1qg[idx]*gamma2gg[idx]*LQ3+B1*gamma1qg[idx]*gamma2gg[idx]*LQ2*LF-B1*gamma1qg[idx]*gamma2gg[idx]*LQ*LF2+1./6.*B1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ4-1./2.*B1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3*LF+1./2.*B1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF2-1./6.*B1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF3+1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4-3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2-1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3-B1*gamma2qq[idx]*gamma1qg[idx]*LQ3+3.*B1*gamma2qq[idx]*gamma1qg[idx]*LQ2*LF-B1*gamma2qq[idx]*gamma1qg[idx]*LQ*LF2-2.*B1*gamma1qq[idx]*gamma2qg[idx]*LQ3+3.*B1*gamma1qq[idx]*gamma2qg[idx]*LQ2*LF-2.*B1*gamma1qq[idx]*gamma2qg[idx]*LQ*LF2+2./3.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ4-2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF+2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2-2./3.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF3+7./6.*B1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ4-7./2.*B1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ3*LF+7./2.*B1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ2*LF2-7./6.*B1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ*LF3+2.*B1*C2qq[idx]*gamma1qg[idx]*LQ2-2.*B1*C2qq[idx]*gamma1qg[idx]*LQ*LF+2.*B1*C1qq[idx]*gamma2qg[idx]*LQ2-2.*B1*C1qq[idx]*gamma2qg[idx]*LQ*LF-B1*C1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3+2.*B1*C1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF-B1*C1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2-3.*B1*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3+6.*B1*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF-3.*B1*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF2+B1*pow(C1qq[idx],2)*gamma1qg[idx]*LQ2-B1*pow(C1qq[idx],2)*gamma1qg[idx]*LQ*LF+1./3.*B1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4-B1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+B1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2-1./3.*B1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+B1*beta1*gamma1qg[idx]*LQ3-1./2.*B1*beta1*gamma1qg[idx]*LQ2*LF-1./2.*B1*beta1*gamma1qg[idx]*LQ*LF2+3./2.*B1*beta0*gamma2qg[idx]*LQ3-1./2.*B1*beta0*gamma2qg[idx]*LQ2*LF-B1*beta0*gamma2qg[idx]*LQ*LF2-3./4.*B1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ4+B1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF+1./4.*B1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2-1./2.*B1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ*LF3-9./4.*B1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ4+3.*B1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF+3./4.*B1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2-3./2.*B1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ*LF3+4.*B1*beta0*C1qq[idx]*gamma1qg[idx]*LQ3-3.*B1*beta0*C1qq[idx]*gamma1qg[idx]*LQ2*LF-B1*beta0*C1qq[idx]*gamma1qg[idx]*LQ*LF2+11./12.*B1*pow(beta0,2)*gamma1qg[idx]*LQ4-1./3.*B1*pow(beta0,2)*gamma1qg[idx]*LQ3*LF-1./4.*B1*pow(beta0,2)*gamma1qg[idx]*LQ2*LF2-1./3.*B1*pow(beta0,2)*gamma1qg[idx]*LQ*LF3-B1*B2*gamma1qg[idx]*LQ3+B1*B2*gamma1qg[idx]*LQ2*LF-1./2.*pow(B1,2)*gamma2qg[idx]*LQ3+1./2.*pow(B1,2)*gamma2qg[idx]*LQ2*LF+1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ4-1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF+1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2+3./4.*pow(B1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ4-3./2.*pow(B1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF+3./4.*pow(B1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2-pow(B1,2)*C1qq[idx]*gamma1qg[idx]*LQ3+pow(B1,2)*C1qq[idx]*gamma1qg[idx]*LQ2*LF-3./4.*pow(B1,2)*beta0*gamma1qg[idx]*LQ4+1./2.*pow(B1,2)*beta0*gamma1qg[idx]*LQ3*LF+1./4.*pow(B1,2)*beta0*gamma1qg[idx]*LQ2*LF2+1./6.*pow(B1,3)*gamma1qg[idx]*LQ4-1./6.*pow(B1,3)*gamma1qg[idx]*LQ3*LF+1./2.*A3*gamma1qg[idx]*LQ3-1./2.*A3*gamma1qg[idx]*LQ2*LF+1./2.*A2*gamma2qg[idx]*LQ3-1./2.*A2*gamma2qg[idx]*LQ2*LF-1./4.*A2*gamma1qg[idx]*gamma1gg[idx]*LQ4+1./2.*A2*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF-1./4.*A2*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2-3./4.*A2*gamma1qq[idx]*gamma1qg[idx]*LQ4+3./2.*A2*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF-3./4.*A2*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2+A2*C1qq[idx]*gamma1qg[idx]*LQ3-A2*C1qq[idx]*gamma1qg[idx]*LQ2*LF+11./12.*A2*beta0*gamma1qg[idx]*LQ4-2./3.*A2*beta0*gamma1qg[idx]*LQ3*LF-1./4.*A2*beta0*gamma1qg[idx]*LQ2*LF2-1./2.*A2*B1*gamma1qg[idx]*LQ4+1./2.*A2*B1*gamma1qg[idx]*LQ3*LF+1./2.*A1*gamma3qg[idx]*LQ3-1./2.*A1*gamma3qg[idx]*LQ2*LF+1./2.*A1*gamma2qg[idx]*gamma1gg[idx]*LQ3*LF+1./2.*A1*gamma1qg[idx]*gamma2qqbp[idx]*LQ3*LF+1./2.*A1*gamma1qg[idx]*gamma2qqp[idx]*LQ3*LF+1./2.*A1*gamma1qg[idx]*gamma2qqb[idx]*LQ3*LF-1./2.*A1*gamma1qg[idx]*gamma2gg[idx]*LQ4+1./2.*A1*gamma1qg[idx]*gamma2gg[idx]*LQ3*LF-1./2.*A1*gamma1qg[idx]*gamma2gg[idx]*LQ2*LF2+1./12.*A1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ5-1./4.*A1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ4*LF+1./4.*A1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3*LF2-1./12.*A1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF3+1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ5-3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4*LF+3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF2-1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF3-1./2.*A1*gamma2qq[idx]*gamma1qg[idx]*LQ4+3./2.*A1*gamma2qq[idx]*gamma1qg[idx]*LQ3*LF-1./2.*A1*gamma2qq[idx]*gamma1qg[idx]*LQ2*LF2-A1*gamma1qq[idx]*gamma2qg[idx]*LQ4+3./2.*A1*gamma1qq[idx]*gamma2qg[idx]*LQ3*LF-A1*gamma1qq[idx]*gamma2qg[idx]*LQ2*LF2+1./3.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ5-A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ4*LF+A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF2-1./3.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF3+7./12.*A1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ5-7./4.*A1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ4*LF+7./4.*A1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ3*LF2-7./12.*A1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ2*LF3+A1*C2qq[idx]*gamma1qg[idx]*LQ3-A1*C2qq[idx]*gamma1qg[idx]*LQ2*LF+A1*C1qq[idx]*gamma2qg[idx]*LQ3-A1*C1qq[idx]*gamma2qg[idx]*LQ2*LF-1./2.*A1*C1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ4+A1*C1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF-1./2.*A1*C1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2-3./2.*A1*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ4+3.*A1*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF-3./2.*A1*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2+1./2.*A1*pow(C1qq[idx],2)*gamma1qg[idx]*LQ3-1./2.*A1*pow(C1qq[idx],2)*gamma1qg[idx]*LQ2*LF+1./6.*A1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ5-1./2.*A1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4*LF+1./2.*A1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF2-1./6.*A1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF3+7./12.*A1*beta1*gamma1qg[idx]*LQ4-1./3.*A1*beta1*gamma1qg[idx]*LQ3*LF-1./4.*A1*beta1*gamma1qg[idx]*LQ2*LF2+5./6.*A1*beta0*gamma2qg[idx]*LQ4-1./3.*A1*beta0*gamma2qg[idx]*LQ3*LF-1./2.*A1*beta0*gamma2qg[idx]*LQ2*LF2-5./12.*A1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ5+7./12.*A1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ4*LF+1./12.*A1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF2-1./4.*A1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF3-5./4.*A1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ5+7./4.*A1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ4*LF+1./4.*A1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF2-3./4.*A1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF3+13./6.*A1*beta0*C1qq[idx]*gamma1qg[idx]*LQ4-5./3.*A1*beta0*C1qq[idx]*gamma1qg[idx]*LQ3*LF-1./2.*A1*beta0*C1qq[idx]*gamma1qg[idx]*LQ2*LF2+7./12.*A1*pow(beta0,2)*gamma1qg[idx]*LQ5-1./4.*A1*pow(beta0,2)*gamma1qg[idx]*LQ4*LF-1./6.*A1*pow(beta0,2)*gamma1qg[idx]*LQ3*LF2-1./6.*A1*pow(beta0,2)*gamma1qg[idx]*LQ2*LF3-1./2.*A1*B2*gamma1qg[idx]*LQ4+1./2.*A1*B2*gamma1qg[idx]*LQ3*LF-1./2.*A1*B1*gamma2qg[idx]*LQ4+1./2.*A1*B1*gamma2qg[idx]*LQ3*LF+1./4.*A1*B1*gamma1qg[idx]*gamma1gg[idx]*LQ5-1./2.*A1*B1*gamma1qg[idx]*gamma1gg[idx]*LQ4*LF+1./4.*A1*B1*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF2+3./4.*A1*B1*gamma1qq[idx]*gamma1qg[idx]*LQ5-3./2.*A1*B1*gamma1qq[idx]*gamma1qg[idx]*LQ4*LF+3./4.*A1*B1*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF2-A1*B1*C1qq[idx]*gamma1qg[idx]*LQ4+A1*B1*C1qq[idx]*gamma1qg[idx]*LQ3*LF-5./6.*A1*B1*beta0*gamma1qg[idx]*LQ5+7./12.*A1*B1*beta0*gamma1qg[idx]*LQ4*LF+1./4.*A1*B1*beta0*gamma1qg[idx]*LQ3*LF2+1./4.*A1*pow(B1,2)*gamma1qg[idx]*LQ5-1./4.*A1*pow(B1,2)*gamma1qg[idx]*LQ4*LF-1./4.*A1*A2*gamma1qg[idx]*LQ5+1./4.*A1*A2*gamma1qg[idx]*LQ4*LF-1./8.*pow(A1,2)*gamma2qg[idx]*LQ5+1./8.*pow(A1,2)*gamma2qg[idx]*LQ4*LF+1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ6-1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ5*LF+1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ4*LF2+3./16.*pow(A1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ6-3./8.*pow(A1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ5*LF+3./16.*pow(A1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ4*LF2-1./4.*pow(A1,2)*C1qq[idx]*gamma1qg[idx]*LQ5+1./4.*pow(A1,2)*C1qq[idx]*gamma1qg[idx]*LQ4*LF-11./48.*pow(A1,2)*beta0*gamma1qg[idx]*LQ6+1./6.*pow(A1,2)*beta0*gamma1qg[idx]*LQ5*LF+1./16.*pow(A1,2)*beta0*gamma1qg[idx]*LQ4*LF2+1./8.*pow(A1,2)*B1*gamma1qg[idx]*LQ6-1./8.*pow(A1,2)*B1*gamma1qg[idx]*LQ5*LF+1./48.*pow(A1,3)*gamma1qg[idx]*LQ7-1./48.*pow(A1,3)*gamma1qg[idx]*LQ6*LF;
	    //complex <double> H4st_qq_qqb=+gamma2qqp[idx]*gamma2qqbp[idx]*LF2+gamma2qqp[idx]*gamma2qqbp[idx]*LQ2-2.*gamma2qqp[idx]*gamma2qqbp[idx]*LQ*LF+gamma4qqb[idx]*LF-gamma4qqb[idx]*LQ+1./2.*gamma2qg[idx]*gamma1gq[idx]*LF2+1./2.*gamma2qg[idx]*gamma1gq[idx]*LQ2-gamma2qg[idx]*gamma1gq[idx]*LQ*LF+gamma1qg[idx]*gamma3gq[idx]*LF2+gamma1qg[idx]*gamma3gq[idx]*LQ2-gamma1qg[idx]*gamma3gq[idx]*LQ*LF+1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LF3-1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ3+1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ2*LF-gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF-gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF-gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3-gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF-5./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LF4+1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ4-1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ3*LF+1./4.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ2*LF2-1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ*LF3+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+2.*gamma2qq[idx]*gamma2qqb[idx]*LF2+2.*gamma2qq[idx]*gamma2qqb[idx]*LQ2-4.*gamma2qq[idx]*gamma2qqb[idx]*LQ*LF+gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-5./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+2.*gamma1qq[idx]*gamma3qqb[idx]*LF2+2.*gamma1qq[idx]*gamma3qqb[idx]*LQ2-3.*gamma1qq[idx]*gamma3qqb[idx]*LQ*LF+gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF3-3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ3+5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4+1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4-gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+pow(gamma1qq[idx],2)*gamma2qqb[idx]*LF3-pow(gamma1qq[idx],2)*gamma2qqb[idx]*LQ3+3.*pow(gamma1qq[idx],2)*gamma2qqb[idx]*LQ2*LF-7./2.*pow(gamma1qq[idx],2)*gamma2qqb[idx]*LQ*LF2+17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF4+17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+17./4.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+C2qq[idx]*gamma2qqb[idx]*LF-C2qq[idx]*gamma2qqb[idx]*LQ+1./2.*C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+C1qq[idx]*gamma3qqb[idx]*LF-C1qq[idx]*gamma3qqb[idx]*LQ-C1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2-C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+1./6.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-1./6.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+1./2.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-1./2.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+2.*C1qq[idx]*gamma1qq[idx]*gamma2qqb[idx]*LF2+2.*C1qq[idx]*gamma1qq[idx]*gamma2qqb[idx]*LQ2-5.*C1qq[idx]*gamma1qq[idx]*gamma2qqb[idx]*LQ*LF+4./3.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-4./3.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+4.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-4.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+pow(C1qq[idx],2)*gamma2qqb[idx]*LF-pow(C1qq[idx],2)*gamma2qqb[idx]*LQ+1./2.*pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2-pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+1./2.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+beta1*gamma2qqb[idx]*LF2-beta1*gamma2qqb[idx]*LQ2+1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LF3+1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ3-1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*beta0*gamma3qqb[idx]*LF2-3./2.*beta0*gamma3qqb[idx]*LQ2-beta0*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*beta0*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LF3+3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ3-1./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-beta0*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4-1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF-1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+3./2.*beta0*gamma1qq[idx]*gamma2qqb[idx]*LF3+3./2.*beta0*gamma1qq[idx]*gamma2qqb[idx]*LQ3-5./2.*beta0*gamma1qq[idx]*gamma2qqb[idx]*LQ2*LF-2.*beta0*gamma1qq[idx]*gamma2qqb[idx]*LQ*LF2+5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF4-5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+2.*beta0*C1qq[idx]*gamma2qqb[idx]*LF2-4.*beta0*C1qq[idx]*gamma2qqb[idx]*LQ2+2.*beta0*C1qq[idx]*gamma2qqb[idx]*LQ*LF+beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+2.*beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3-3.*beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+pow(beta0,2)*gamma2qqb[idx]*LF3-pow(beta0,2)*gamma2qqb[idx]*LQ3+11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LF4+11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./4.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+B2*gamma2qqb[idx]*LQ2-B2*gamma2qqb[idx]*LQ*LF-1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ3+B2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+B1*gamma3qqb[idx]*LQ2-B1*gamma3qqb[idx]*LQ*LF+B1*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-B1*gamma1qg[idx]*gamma2gq[idx]*LQ3+B1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-B1*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4-1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3-B1*gamma1qq[idx]*gamma2qqb[idx]*LQ3+3.*B1*gamma1qq[idx]*gamma2qqb[idx]*LQ2*LF-B1*gamma1qq[idx]*gamma2qqb[idx]*LQ*LF2+5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4-5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+2.*B1*C1qq[idx]*gamma2qqb[idx]*LQ2-2.*B1*C1qq[idx]*gamma2qqb[idx]*LQ*LF-B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+2.*B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*B1*beta0*gamma2qqb[idx]*LQ3-1./2.*B1*beta0*gamma2qqb[idx]*LQ2*LF-B1*beta0*gamma2qqb[idx]*LQ*LF2-3./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4+B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+1./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-1./2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3-1./2.*pow(B1,2)*gamma2qqb[idx]*LQ3+1./2.*pow(B1,2)*gamma2qqb[idx]*LQ2*LF+1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+1./2.*A2*gamma2qqb[idx]*LQ3-1./2.*A2*gamma2qqb[idx]*LQ2*LF-1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ4+1./2.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+1./2.*A1*gamma3qqb[idx]*LQ3-1./2.*A1*gamma3qqb[idx]*LQ2*LF+1./2.*A1*gamma2qg[idx]*gamma1gq[idx]*LQ3*LF-1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ4+1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ3*LF-1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF2+1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ5-1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4*LF+1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF2-1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF3-1./2.*A1*gamma1qq[idx]*gamma2qqb[idx]*LQ4+3./2.*A1*gamma1qq[idx]*gamma2qqb[idx]*LQ3*LF-1./2.*A1*gamma1qq[idx]*gamma2qqb[idx]*LQ2*LF2+5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ5-5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3+A1*C1qq[idx]*gamma2qqb[idx]*LQ3-A1*C1qq[idx]*gamma2qqb[idx]*LQ2*LF-1./2.*A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./2.*A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+5./6.*A1*beta0*gamma2qqb[idx]*LQ4-1./3.*A1*beta0*gamma2qqb[idx]*LQ3*LF-1./2.*A1*beta0*gamma2qqb[idx]*LQ2*LF2-5./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ5+7./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+1./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-1./4.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3-1./2.*A1*B1*gamma2qqb[idx]*LQ4+1./2.*A1*B1*gamma2qqb[idx]*LQ3*LF+1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ5-1./2.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-1./8.*pow(A1,2)*gamma2qqb[idx]*LQ5+1./8.*pow(A1,2)*gamma2qqb[idx]*LQ4*LF+1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ6-1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ5*LF+1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF2;
	    //complex <double> H4st_qqp_qqb=+gamma4qqbp[idx]*LF-gamma4qqbp[idx]*LQ+gamma2qqb[idx]*gamma2qqp[idx]*LF2+gamma2qqb[idx]*gamma2qqp[idx]*LQ2-2.*gamma2qqb[idx]*gamma2qqp[idx]*LQ*LF+1./2.*gamma2qg[idx]*gamma1gq[idx]*LF2+1./2.*gamma2qg[idx]*gamma1gq[idx]*LQ2-gamma2qg[idx]*gamma1gq[idx]*LQ*LF+gamma1qg[idx]*gamma3gq[idx]*LF2+gamma1qg[idx]*gamma3gq[idx]*LQ2-gamma1qg[idx]*gamma3gq[idx]*LQ*LF+1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LF3-1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ3+1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ2*LF-gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ*LF2+gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3-gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF-5./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF-gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF-gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LF4+1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ4-1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ3*LF+1./4.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ2*LF2-1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ*LF3+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+2.*gamma2qq[idx]*gamma2qqbp[idx]*LF2+2.*gamma2qq[idx]*gamma2qqbp[idx]*LQ2-4.*gamma2qq[idx]*gamma2qqbp[idx]*LQ*LF+gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-5./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+2.*gamma1qq[idx]*gamma3qqbp[idx]*LF2+2.*gamma1qq[idx]*gamma3qqbp[idx]*LQ2-3.*gamma1qq[idx]*gamma3qqbp[idx]*LQ*LF+gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF3-3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ3+5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4+1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4-gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+pow(gamma1qq[idx],2)*gamma2qqbp[idx]*LF3-pow(gamma1qq[idx],2)*gamma2qqbp[idx]*LQ3+3.*pow(gamma1qq[idx],2)*gamma2qqbp[idx]*LQ2*LF-7./2.*pow(gamma1qq[idx],2)*gamma2qqbp[idx]*LQ*LF2+17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF4+17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+17./4.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+C2qq[idx]*gamma2qqbp[idx]*LF-C2qq[idx]*gamma2qqbp[idx]*LQ+1./2.*C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+C1qq[idx]*gamma3qqbp[idx]*LF-C1qq[idx]*gamma3qqbp[idx]*LQ-C1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2-C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+1./6.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-1./6.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+1./2.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-1./2.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+2.*C1qq[idx]*gamma1qq[idx]*gamma2qqbp[idx]*LF2+2.*C1qq[idx]*gamma1qq[idx]*gamma2qqbp[idx]*LQ2-5.*C1qq[idx]*gamma1qq[idx]*gamma2qqbp[idx]*LQ*LF+4./3.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-4./3.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+4.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-4.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+pow(C1qq[idx],2)*gamma2qqbp[idx]*LF-pow(C1qq[idx],2)*gamma2qqbp[idx]*LQ+1./2.*pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2-pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+1./2.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+beta1*gamma2qqbp[idx]*LF2-beta1*gamma2qqbp[idx]*LQ2+1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LF3+1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ3-1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*beta0*gamma3qqbp[idx]*LF2-3./2.*beta0*gamma3qqbp[idx]*LQ2-beta0*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*beta0*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LF3+3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ3-1./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-beta0*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4-1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF-1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+3./2.*beta0*gamma1qq[idx]*gamma2qqbp[idx]*LF3+3./2.*beta0*gamma1qq[idx]*gamma2qqbp[idx]*LQ3-5./2.*beta0*gamma1qq[idx]*gamma2qqbp[idx]*LQ2*LF-2.*beta0*gamma1qq[idx]*gamma2qqbp[idx]*LQ*LF2+5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF4-5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+2.*beta0*C1qq[idx]*gamma2qqbp[idx]*LF2-4.*beta0*C1qq[idx]*gamma2qqbp[idx]*LQ2+2.*beta0*C1qq[idx]*gamma2qqbp[idx]*LQ*LF+beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+2.*beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3-3.*beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+pow(beta0,2)*gamma2qqbp[idx]*LF3-pow(beta0,2)*gamma2qqbp[idx]*LQ3+11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LF4+11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./4.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+B2*gamma2qqbp[idx]*LQ2-B2*gamma2qqbp[idx]*LQ*LF-1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ3+B2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+B1*gamma3qqbp[idx]*LQ2-B1*gamma3qqbp[idx]*LQ*LF+B1*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-B1*gamma1qg[idx]*gamma2gq[idx]*LQ3+B1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-B1*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4-1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3-B1*gamma1qq[idx]*gamma2qqbp[idx]*LQ3+3.*B1*gamma1qq[idx]*gamma2qqbp[idx]*LQ2*LF-B1*gamma1qq[idx]*gamma2qqbp[idx]*LQ*LF2+5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4-5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+2.*B1*C1qq[idx]*gamma2qqbp[idx]*LQ2-2.*B1*C1qq[idx]*gamma2qqbp[idx]*LQ*LF-B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+2.*B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*B1*beta0*gamma2qqbp[idx]*LQ3-1./2.*B1*beta0*gamma2qqbp[idx]*LQ2*LF-B1*beta0*gamma2qqbp[idx]*LQ*LF2-3./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4+B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+1./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-1./2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3-1./2.*pow(B1,2)*gamma2qqbp[idx]*LQ3+1./2.*pow(B1,2)*gamma2qqbp[idx]*LQ2*LF+1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+1./2.*A2*gamma2qqbp[idx]*LQ3-1./2.*A2*gamma2qqbp[idx]*LQ2*LF-1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ4+1./2.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+1./2.*A1*gamma3qqbp[idx]*LQ3-1./2.*A1*gamma3qqbp[idx]*LQ2*LF+1./2.*A1*gamma2qg[idx]*gamma1gq[idx]*LQ3*LF-1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ4+1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ3*LF-1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF2+1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ5-1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4*LF+1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF2-1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF3-1./2.*A1*gamma1qq[idx]*gamma2qqbp[idx]*LQ4+3./2.*A1*gamma1qq[idx]*gamma2qqbp[idx]*LQ3*LF-1./2.*A1*gamma1qq[idx]*gamma2qqbp[idx]*LQ2*LF2+5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ5-5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3+A1*C1qq[idx]*gamma2qqbp[idx]*LQ3-A1*C1qq[idx]*gamma2qqbp[idx]*LQ2*LF-1./2.*A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./2.*A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+5./6.*A1*beta0*gamma2qqbp[idx]*LQ4-1./3.*A1*beta0*gamma2qqbp[idx]*LQ3*LF-1./2.*A1*beta0*gamma2qqbp[idx]*LQ2*LF2-5./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ5+7./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+1./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-1./4.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3-1./2.*A1*B1*gamma2qqbp[idx]*LQ4+1./2.*A1*B1*gamma2qqbp[idx]*LQ3*LF+1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ5-1./2.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-1./8.*pow(A1,2)*gamma2qqbp[idx]*LQ5+1./8.*pow(A1,2)*gamma2qqbp[idx]*LQ4*LF+1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ6-1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ5*LF+1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF2;
	    //complex <double> H4st_qqbp_qqb=+gamma4qqp[idx]*LF-gamma4qqp[idx]*LQ+gamma2qqb[idx]*gamma2qqbp[idx]*LF2+gamma2qqb[idx]*gamma2qqbp[idx]*LQ2-2.*gamma2qqb[idx]*gamma2qqbp[idx]*LQ*LF+1./2.*gamma2qg[idx]*gamma1gq[idx]*LF2+1./2.*gamma2qg[idx]*gamma1gq[idx]*LQ2-gamma2qg[idx]*gamma1gq[idx]*LQ*LF+gamma1qg[idx]*gamma3gq[idx]*LF2+gamma1qg[idx]*gamma3gq[idx]*LQ2-gamma1qg[idx]*gamma3gq[idx]*LQ*LF+1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LF3-1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ3+1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ2*LF-gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF-gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3-gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF-5./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF-gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LF4+1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ4-1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ3*LF+1./4.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ2*LF2-1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ*LF3+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+2.*gamma2qq[idx]*gamma2qqp[idx]*LF2+2.*gamma2qq[idx]*gamma2qqp[idx]*LQ2-4.*gamma2qq[idx]*gamma2qqp[idx]*LQ*LF+gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-5./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+2.*gamma1qq[idx]*gamma3qqp[idx]*LF2+2.*gamma1qq[idx]*gamma3qqp[idx]*LQ2-3.*gamma1qq[idx]*gamma3qqp[idx]*LQ*LF+gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF3-3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ3+5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4+1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4-gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+pow(gamma1qq[idx],2)*gamma2qqp[idx]*LF3-pow(gamma1qq[idx],2)*gamma2qqp[idx]*LQ3+3.*pow(gamma1qq[idx],2)*gamma2qqp[idx]*LQ2*LF-7./2.*pow(gamma1qq[idx],2)*gamma2qqp[idx]*LQ*LF2+17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF4+17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+17./4.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+C2qq[idx]*gamma2qqp[idx]*LF-C2qq[idx]*gamma2qqp[idx]*LQ+1./2.*C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+C1qq[idx]*gamma3qqp[idx]*LF-C1qq[idx]*gamma3qqp[idx]*LQ-C1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2-C1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+1./6.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-1./6.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+1./2.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-1./2.*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+2.*C1qq[idx]*gamma1qq[idx]*gamma2qqp[idx]*LF2+2.*C1qq[idx]*gamma1qq[idx]*gamma2qqp[idx]*LQ2-5.*C1qq[idx]*gamma1qq[idx]*gamma2qqp[idx]*LQ*LF+4./3.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-4./3.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+4.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-4.*C1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+pow(C1qq[idx],2)*gamma2qqp[idx]*LF-pow(C1qq[idx],2)*gamma2qqp[idx]*LQ+1./2.*pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2-pow(C1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+1./2.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+beta1*gamma2qqp[idx]*LF2-beta1*gamma2qqp[idx]*LQ2+1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LF3+1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ3-1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*beta0*gamma3qqp[idx]*LF2-3./2.*beta0*gamma3qqp[idx]*LQ2-beta0*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*beta0*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LF3+3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ3-1./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-beta0*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4-1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF-1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+3./2.*beta0*gamma1qq[idx]*gamma2qqp[idx]*LF3+3./2.*beta0*gamma1qq[idx]*gamma2qqp[idx]*LQ3-5./2.*beta0*gamma1qq[idx]*gamma2qqp[idx]*LQ2*LF-2.*beta0*gamma1qq[idx]*gamma2qqp[idx]*LQ*LF2+5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF4-5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+2.*beta0*C1qq[idx]*gamma2qqp[idx]*LF2-4.*beta0*C1qq[idx]*gamma2qqp[idx]*LQ2+2.*beta0*C1qq[idx]*gamma2qqp[idx]*LQ*LF+beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+2.*beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3-3.*beta0*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+pow(beta0,2)*gamma2qqp[idx]*LF3-pow(beta0,2)*gamma2qqp[idx]*LQ3+11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LF4+11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./4.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+B2*gamma2qqp[idx]*LQ2-B2*gamma2qqp[idx]*LQ*LF-1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ3+B2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+B1*gamma3qqp[idx]*LQ2-B1*gamma3qqp[idx]*LQ*LF+B1*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-B1*gamma1qg[idx]*gamma2gq[idx]*LQ3+B1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF-B1*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4-1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3-B1*gamma1qq[idx]*gamma2qqp[idx]*LQ3+3.*B1*gamma1qq[idx]*gamma2qqp[idx]*LQ2*LF-B1*gamma1qq[idx]*gamma2qqp[idx]*LQ*LF2+5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4-5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+2.*B1*C1qq[idx]*gamma2qqp[idx]*LQ2-2.*B1*C1qq[idx]*gamma2qqp[idx]*LQ*LF-B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+2.*B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-B1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+3./2.*B1*beta0*gamma2qqp[idx]*LQ3-1./2.*B1*beta0*gamma2qqp[idx]*LQ2*LF-B1*beta0*gamma2qqp[idx]*LQ*LF2-3./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4+B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+1./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2-1./2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3-1./2.*pow(B1,2)*gamma2qqp[idx]*LQ3+1./2.*pow(B1,2)*gamma2qqp[idx]*LQ2*LF+1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4-1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+1./2.*A2*gamma2qqp[idx]*LQ3-1./2.*A2*gamma2qqp[idx]*LQ2*LF-1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ4+1./2.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+1./2.*A1*gamma3qqp[idx]*LQ3-1./2.*A1*gamma3qqp[idx]*LQ2*LF+1./2.*A1*gamma2qg[idx]*gamma1gq[idx]*LQ3*LF-1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ4+1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ3*LF-1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF2+1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ5-1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4*LF+1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF2-1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF3-1./2.*A1*gamma1qq[idx]*gamma2qqp[idx]*LQ4+3./2.*A1*gamma1qq[idx]*gamma2qqp[idx]*LQ3*LF-1./2.*A1*gamma1qq[idx]*gamma2qqp[idx]*LQ2*LF2+5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ5-5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3+A1*C1qq[idx]*gamma2qqp[idx]*LQ3-A1*C1qq[idx]*gamma2qqp[idx]*LQ2*LF-1./2.*A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./2.*A1*C1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+5./6.*A1*beta0*gamma2qqp[idx]*LQ4-1./3.*A1*beta0*gamma2qqp[idx]*LQ3*LF-1./2.*A1*beta0*gamma2qqp[idx]*LQ2*LF2-5./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ5+7./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+1./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-1./4.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3-1./2.*A1*B1*gamma2qqp[idx]*LQ4+1./2.*A1*B1*gamma2qqp[idx]*LQ3*LF+1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ5-1./2.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2-1./8.*pow(A1,2)*gamma2qqp[idx]*LQ5+1./8.*pow(A1,2)*gamma2qqp[idx]*LQ4*LF+1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ6-1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ5*LF+1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF2;
	    //complex <double> H4st_gg_qqb=+pow(gamma2qg[idx],2)*LF2+pow(gamma2qg[idx],2)*LQ2-2.*pow(gamma2qg[idx],2)*LQ*LF+2.*gamma1qg[idx]*gamma3qg[idx]*LF2+2.*gamma1qg[idx]*gamma3qg[idx]*LQ2-4.*gamma1qg[idx]*gamma3qg[idx]*LQ*LF+gamma1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LF3-gamma1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ3+5.*gamma1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ2*LF-5.*gamma1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ*LF2+2.*pow(gamma1qg[idx],2)*gamma2qqbp[idx]*LQ2*LF-2.*pow(gamma1qg[idx],2)*gamma2qqbp[idx]*LQ*LF2+2.*pow(gamma1qg[idx],2)*gamma2qqp[idx]*LQ2*LF-2.*pow(gamma1qg[idx],2)*gamma2qqp[idx]*LQ*LF2+2.*pow(gamma1qg[idx],2)*gamma2qqb[idx]*LQ2*LF-2.*pow(gamma1qg[idx],2)*gamma2qqb[idx]*LQ*LF2+2.*pow(gamma1qg[idx],2)*gamma2gg[idx]*LF3-2.*pow(gamma1qg[idx],2)*gamma2gg[idx]*LQ3+4.*pow(gamma1qg[idx],2)*gamma2gg[idx]*LQ2*LF-4.*pow(gamma1qg[idx],2)*gamma2gg[idx]*LQ*LF2+7./12.*pow(gamma1qg[idx],2)*pow(gamma1gg[idx],2)*LF4+7./12.*pow(gamma1qg[idx],2)*pow(gamma1gg[idx],2)*LQ4-7./3.*pow(gamma1qg[idx],2)*pow(gamma1gg[idx],2)*LQ3*LF+7./2.*pow(gamma1qg[idx],2)*pow(gamma1gg[idx],2)*LQ2*LF2-7./3.*pow(gamma1qg[idx],2)*pow(gamma1gg[idx],2)*LQ*LF3+2.*gamma2qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF-2.*gamma2qq[idx]*pow(gamma1qg[idx],2)*LQ*LF2+3.*gamma1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LF3-3.*gamma1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LQ3+7.*gamma1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LQ2*LF-7.*gamma1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LQ*LF2+5./6.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LF4+5./6.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ4-10./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ3*LF+5.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ2*LF2-10./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ*LF3+7./12.*pow(gamma1qq[idx],2)*pow(gamma1qg[idx],2)*LF4+7./12.*pow(gamma1qq[idx],2)*pow(gamma1qg[idx],2)*LQ4-7./3.*pow(gamma1qq[idx],2)*pow(gamma1qg[idx],2)*LQ3*LF+7./2.*pow(gamma1qq[idx],2)*pow(gamma1qg[idx],2)*LQ2*LF2-7./3.*pow(gamma1qq[idx],2)*pow(gamma1qg[idx],2)*LQ*LF3+2.*C2qq[idx]*pow(gamma1qg[idx],2)*LF2+2.*C2qq[idx]*pow(gamma1qg[idx],2)*LQ2-4.*C2qq[idx]*pow(gamma1qg[idx],2)*LQ*LF+4.*C1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LF2+4.*C1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LQ2-8.*C1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LQ*LF+2.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LF3-2.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ3+6.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ2*LF-6.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ*LF2+2.*C1qq[idx]*gamma1qq[idx]*pow(gamma1qg[idx],2)*LF3-2.*C1qq[idx]*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ3+6.*C1qq[idx]*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF-6.*C1qq[idx]*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ*LF2+pow(C1qq[idx],2)*pow(gamma1qg[idx],2)*LF2+pow(C1qq[idx],2)*pow(gamma1qg[idx],2)*LQ2-2.*pow(C1qq[idx],2)*pow(gamma1qg[idx],2)*LQ*LF+2./3.*nf*pow(gamma1qg[idx],3)*gamma1gq[idx]*LF4+2./3.*nf*pow(gamma1qg[idx],3)*gamma1gq[idx]*LQ4-8./3.*nf*pow(gamma1qg[idx],3)*gamma1gq[idx]*LQ3*LF+4.*nf*pow(gamma1qg[idx],3)*gamma1gq[idx]*LQ2*LF2-8./3.*nf*pow(gamma1qg[idx],3)*gamma1gq[idx]*LQ*LF3+beta1*pow(gamma1qg[idx],2)*LF3+beta1*pow(gamma1qg[idx],2)*LQ3-beta1*pow(gamma1qg[idx],2)*LQ2*LF-beta1*pow(gamma1qg[idx],2)*LQ*LF2+3.*beta0*gamma1qg[idx]*gamma2qg[idx]*LF3+3.*beta0*gamma1qg[idx]*gamma2qg[idx]*LQ3-3.*beta0*gamma1qg[idx]*gamma2qg[idx]*LQ2*LF-3.*beta0*gamma1qg[idx]*gamma2qg[idx]*LQ*LF2+3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gg[idx]*LF4-3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ4+3.*beta0*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ3*LF-3.*beta0*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ*LF3+3./2.*beta0*gamma1qq[idx]*pow(gamma1qg[idx],2)*LF4-3./2.*beta0*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ4+3.*beta0*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ3*LF-3.*beta0*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ*LF3+2.*beta0*C1qq[idx]*pow(gamma1qg[idx],2)*LF3+4.*beta0*C1qq[idx]*pow(gamma1qg[idx],2)*LQ3-6.*beta0*C1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF+11./12.*pow(beta0,2)*pow(gamma1qg[idx],2)*LF4+11./12.*pow(beta0,2)*pow(gamma1qg[idx],2)*LQ4-2./3.*pow(beta0,2)*pow(gamma1qg[idx],2)*LQ3*LF-1./2.*pow(beta0,2)*pow(gamma1qg[idx],2)*LQ2*LF2-2./3.*pow(beta0,2)*pow(gamma1qg[idx],2)*LQ*LF3-B2*pow(gamma1qg[idx],2)*LQ3+2.*B2*pow(gamma1qg[idx],2)*LQ2*LF-B2*pow(gamma1qg[idx],2)*LQ*LF2-2.*B1*gamma1qg[idx]*gamma2qg[idx]*LQ3+4.*B1*gamma1qg[idx]*gamma2qg[idx]*LQ2*LF-2.*B1*gamma1qg[idx]*gamma2qg[idx]*LQ*LF2+B1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ4-3.*B1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ3*LF+3.*B1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ2*LF2-B1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ*LF3+B1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ4-3.*B1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ3*LF+3.*B1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF2-B1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ*LF3-2.*B1*C1qq[idx]*pow(gamma1qg[idx],2)*LQ3+4.*B1*C1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF-2.*B1*C1qq[idx]*pow(gamma1qg[idx],2)*LQ*LF2-3./2.*B1*beta0*pow(gamma1qg[idx],2)*LQ4+2.*B1*beta0*pow(gamma1qg[idx],2)*LQ3*LF+1./2.*B1*beta0*pow(gamma1qg[idx],2)*LQ2*LF2-B1*beta0*pow(gamma1qg[idx],2)*LQ*LF3+1./2.*pow(B1,2)*pow(gamma1qg[idx],2)*LQ4-pow(B1,2)*pow(gamma1qg[idx],2)*LQ3*LF+1./2.*pow(B1,2)*pow(gamma1qg[idx],2)*LQ2*LF2-1./2.*A2*pow(gamma1qg[idx],2)*LQ4+A2*pow(gamma1qg[idx],2)*LQ3*LF-1./2.*A2*pow(gamma1qg[idx],2)*LQ2*LF2-A1*gamma1qg[idx]*gamma2qg[idx]*LQ4+2.*A1*gamma1qg[idx]*gamma2qg[idx]*LQ3*LF-A1*gamma1qg[idx]*gamma2qg[idx]*LQ2*LF2+1./2.*A1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ5-3./2.*A1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ4*LF+3./2.*A1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ3*LF2-1./2.*A1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ2*LF3+1./2.*A1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ5-3./2.*A1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ4*LF+3./2.*A1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ3*LF2-1./2.*A1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF3-A1*C1qq[idx]*pow(gamma1qg[idx],2)*LQ4+2.*A1*C1qq[idx]*pow(gamma1qg[idx],2)*LQ3*LF-A1*C1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF2-5./6.*A1*beta0*pow(gamma1qg[idx],2)*LQ5+7./6.*A1*beta0*pow(gamma1qg[idx],2)*LQ4*LF+1./6.*A1*beta0*pow(gamma1qg[idx],2)*LQ3*LF2-1./2.*A1*beta0*pow(gamma1qg[idx],2)*LQ2*LF3+1./2.*A1*B1*pow(gamma1qg[idx],2)*LQ5-A1*B1*pow(gamma1qg[idx],2)*LQ4*LF+1./2.*A1*B1*pow(gamma1qg[idx],2)*LQ3*LF2+1./8.*pow(A1,2)*pow(gamma1qg[idx],2)*LQ6-1./4.*pow(A1,2)*pow(gamma1qg[idx],2)*LQ5*LF+1./8.*pow(A1,2)*pow(gamma1qg[idx],2)*LQ4*LF2;
	    //complex <double> H4st_qbg_qqb=+gamma2qg[idx]*gamma2qqb[idx]*LF2+gamma2qg[idx]*gamma2qqb[idx]*LQ2-2.*gamma2qg[idx]*gamma2qqb[idx]*LQ*LF+gamma1qg[idx]*gamma3qqb[idx]*LF2+gamma1qg[idx]*gamma3qqb[idx]*LQ2-2.*gamma1qg[idx]*gamma3qqb[idx]*LQ*LF+1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LQ3+3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LQ2*LF-3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF3-1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ3+5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+pow(gamma1qg[idx],2)*gamma2gq[idx]*LF3-pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ3+2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ2*LF-2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ*LF2+5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF4+5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ4-5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+5./2.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LF3-1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ3+5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF-5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF2+7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4+7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4-7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+7./2.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2-7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+2.*C1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LF2+2.*C1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ2-4.*C1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF+C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3-C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+3.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF-3.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+3./2.*beta0*gamma1qg[idx]*gamma2qqb[idx]*LF3+3./2.*beta0*gamma1qg[idx]*gamma2qqb[idx]*LQ3-3./2.*beta0*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF-3./2.*beta0*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF2+3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4-3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4+3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF-3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3-B1*gamma1qg[idx]*gamma2qqb[idx]*LQ3+2.*B1*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF-B1*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF2+1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4-3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2-1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3-1./2.*A1*gamma1qg[idx]*gamma2qqb[idx]*LQ4+A1*gamma1qg[idx]*gamma2qqb[idx]*LQ3*LF-1./2.*A1*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF2+1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ5-3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4*LF+3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF2-1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF3;
	    //complex <double> H4st_qpg_qqb=+gamma2qg[idx]*gamma2qqp[idx]*LF2+gamma2qg[idx]*gamma2qqp[idx]*LQ2-2.*gamma2qg[idx]*gamma2qqp[idx]*LQ*LF+gamma1qg[idx]*gamma3qqp[idx]*LF2+gamma1qg[idx]*gamma3qqp[idx]*LQ2-2.*gamma1qg[idx]*gamma3qqp[idx]*LQ*LF+1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LQ3+3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LQ2*LF-3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF3-1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ3+5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+pow(gamma1qg[idx],2)*gamma2gq[idx]*LF3-pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ3+2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ2*LF-2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ*LF2+5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF4+5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ4-5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+5./2.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LF3-1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ3+5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF-5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF2+7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4+7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4-7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+7./2.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2-7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+2.*C1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LF2+2.*C1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ2-4.*C1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF+C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3-C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+3.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF-3.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+3./2.*beta0*gamma1qg[idx]*gamma2qqp[idx]*LF3+3./2.*beta0*gamma1qg[idx]*gamma2qqp[idx]*LQ3-3./2.*beta0*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF-3./2.*beta0*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF2+3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4-3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4+3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF-3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3-B1*gamma1qg[idx]*gamma2qqp[idx]*LQ3+2.*B1*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF-B1*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF2+1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4-3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2-1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3-1./2.*A1*gamma1qg[idx]*gamma2qqp[idx]*LQ4+A1*gamma1qg[idx]*gamma2qqp[idx]*LQ3*LF-1./2.*A1*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF2+1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ5-3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4*LF+3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF2-1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF3;
	    //complex <double> H4st_qbpg_qqb=+gamma2qg[idx]*gamma2qqbp[idx]*LF2+gamma2qg[idx]*gamma2qqbp[idx]*LQ2-2.*gamma2qg[idx]*gamma2qqbp[idx]*LQ*LF+gamma1qg[idx]*gamma3qqbp[idx]*LF2+gamma1qg[idx]*gamma3qqbp[idx]*LQ2-2.*gamma1qg[idx]*gamma3qqbp[idx]*LQ*LF+1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LQ3+3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LQ2*LF-3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF3-1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ3+5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF-5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+pow(gamma1qg[idx],2)*gamma2gq[idx]*LF3-pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ3+2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ2*LF-2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ*LF2+5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF4+5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ4-5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+5./2.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LF3-1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ3+5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF-5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF2+7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4+7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4-7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+7./2.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2-7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+2.*C1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LF2+2.*C1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ2-4.*C1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF+C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3-C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+3.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF-3.*C1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+3./2.*beta0*gamma1qg[idx]*gamma2qqbp[idx]*LF3+3./2.*beta0*gamma1qg[idx]*gamma2qqbp[idx]*LQ3-3./2.*beta0*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF-3./2.*beta0*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF2+3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4-3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4+3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF-3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3-B1*gamma1qg[idx]*gamma2qqbp[idx]*LQ3+2.*B1*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF-B1*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF2+1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4-3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2-1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3-1./2.*A1*gamma1qg[idx]*gamma2qqbp[idx]*LQ4+A1*gamma1qg[idx]*gamma2qqbp[idx]*LQ3*LF-1./2.*A1*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF2+1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ5-3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4*LF+3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF2-1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF3;
	    //complex <double> H4st_qbq_qqb=+pow(gamma2qqb[idx],2)*LF2+pow(gamma2qqb[idx],2)*LQ2-2.*pow(gamma2qqb[idx],2)*LQ*LF+gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3-gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF-3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3;
	    //complex <double> H4st_qbqp_qqb=+gamma2qqb[idx]*gamma2qqbp[idx]*LF2+gamma2qqb[idx]*gamma2qqbp[idx]*LQ2-2.*gamma2qqb[idx]*gamma2qqbp[idx]*LQ*LF+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF-3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF-3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3;
	    //complex <double> H4st_qbqbp_qqb=+gamma2qqb[idx]*gamma2qqp[idx]*LF2+gamma2qqb[idx]*gamma2qqp[idx]*LQ2-2.*gamma2qqb[idx]*gamma2qqp[idx]*LQ*LF+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF-3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF-3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3;
	    //complex <double> H4st_qpqp_qqb=+gamma2qqp[idx]*gamma2qqbp[idx]*LF2+gamma2qqp[idx]*gamma2qqbp[idx]*LQ2-2.*gamma2qqp[idx]*gamma2qqbp[idx]*LQ*LF+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF-3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3-1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF-3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3;
	    //complex <double> H4st_qpqbp_qqb=+pow(gamma2qqp[idx],2)*LF2+pow(gamma2qqp[idx],2)*LQ2-2.*pow(gamma2qqp[idx],2)*LQ*LF+gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3-gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF-3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3;
	    //complex <double> H4st_qbpqp_qqb=+pow(gamma2qqbp[idx],2)*LF2+pow(gamma2qqbp[idx],2)*LQ2-2.*pow(gamma2qqbp[idx],2)*LQ*LF+gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3-gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF-3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2-pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3;
	    //complex <double> H4st_qqb_qg   =+2.*C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+2.*C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2-4.*C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3-C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF-3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+2.*C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LF2+2.*C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ2-4.*C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ*LF+2.*C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LF2+2.*C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ2-4.*C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ*LF+C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+3.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-3.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+2.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LF3-2.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ3+6.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ2*LF-6.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ*LF2+2.*C2qq[idx]*C1qg[idx]*gamma1gq[idx]*LF-2.*C2qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ+2.*C1qq[idx]*C2qg[idx]*gamma1gq[idx]*LF-2.*C1qq[idx]*C2qg[idx]*gamma1gq[idx]*LQ+2.*C1qq[idx]*C1qg[idx]*gamma2gq[idx]*LF-2.*C1qq[idx]*C1qg[idx]*gamma2gq[idx]*LQ+C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF2+C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2-2.*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+3.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+3.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2-6.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+beta1*C1qg[idx]*gamma1gq[idx]*LF2-3.*beta1*C1qg[idx]*gamma1gq[idx]*LQ2+2.*beta1*C1qg[idx]*gamma1gq[idx]*LQ*LF+beta0*C2qg[idx]*gamma1gq[idx]*LF2-5.*beta0*C2qg[idx]*gamma1gq[idx]*LQ2+4.*beta0*C2qg[idx]*gamma1gq[idx]*LQ*LF+2.*beta0*C1qg[idx]*gamma2gq[idx]*LF2-4.*beta0*C1qg[idx]*gamma2gq[idx]*LQ2+2.*beta0*C1qg[idx]*gamma2gq[idx]*LQ*LF+beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3+2.*beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3-3.*beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF+3.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF3+6.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3-9.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF+beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LF2-5.*beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2+4.*beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ*LF+2./3.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LF3-11./3.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ3+2.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ2*LF+pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ*LF2+2.*B2*C1qg[idx]*gamma1gq[idx]*LQ2-2.*B2*C1qg[idx]*gamma1gq[idx]*LQ*LF+2.*B1*C2qg[idx]*gamma1gq[idx]*LQ2-2.*B1*C2qg[idx]*gamma1gq[idx]*LQ*LF+2.*B1*C1qg[idx]*gamma2gq[idx]*LQ2-2.*B1*C1qg[idx]*gamma2gq[idx]*LQ*LF-B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+2.*B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2-3.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3+6.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF-3.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF2+2.*B1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2-2.*B1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ*LF+4.*B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ3-3.*B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ2*LF-B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ*LF2-pow(B1,2)*C1qg[idx]*gamma1gq[idx]*LQ3+pow(B1,2)*C1qg[idx]*gamma1gq[idx]*LQ2*LF+A2*C1qg[idx]*gamma1gq[idx]*LQ3-A2*C1qg[idx]*gamma1gq[idx]*LQ2*LF+A1*C2qg[idx]*gamma1gq[idx]*LQ3-A1*C2qg[idx]*gamma1gq[idx]*LQ2*LF+A1*C1qg[idx]*gamma2gq[idx]*LQ3-A1*C1qg[idx]*gamma2gq[idx]*LQ2*LF-1./2.*A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF-1./2.*A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-3./2.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ4+3.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3*LF-3./2.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF2+A1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ3-A1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2*LF+13./6.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ4-5./3.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ3*LF-1./2.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ2*LF2-A1*B1*C1qg[idx]*gamma1gq[idx]*LQ4+A1*B1*C1qg[idx]*gamma1gq[idx]*LQ3*LF-1./4.*pow(A1,2)*C1qg[idx]*gamma1gq[idx]*LQ5+1./4.*pow(A1,2)*C1qg[idx]*gamma1gq[idx]*LQ4*LF;
	    //complex <double> H4st_qg_qg    =+C3qg[idx]*gamma1qq[idx]*LF-C3qg[idx]*gamma1qq[idx]*LQ+3./2.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+3./2.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-3.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+C2qg[idx]*gamma2qq[idx]*LF-C2qg[idx]*gamma2qq[idx]*LQ+C2qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LF2+C2qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ2-2.*C2qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ*LF+1./2.*C2qg[idx]*pow(gamma1qq[idx],2)*LF2+1./2.*C2qg[idx]*pow(gamma1qq[idx],2)*LQ2-C2qg[idx]*pow(gamma1qq[idx],2)*LQ*LF+C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF2+C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2-3.*C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+2.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+2.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2-3.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+5./3.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-5./3.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+5.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-5.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+C1qg[idx]*gamma3qq[idx]*LF-C1qg[idx]*gamma3qq[idx]*LQ+C1qg[idx]*gamma2qq[idx]*gamma1gg[idx]*LF2+C1qg[idx]*gamma2qq[idx]*gamma1gg[idx]*LQ2-2.*C1qg[idx]*gamma2qq[idx]*gamma1gg[idx]*LQ*LF+C1qg[idx]*gamma1qq[idx]*gamma2gg[idx]*LF2+C1qg[idx]*gamma1qq[idx]*gamma2gg[idx]*LQ2-2.*C1qg[idx]*gamma1qq[idx]*gamma2gg[idx]*LQ*LF+1./2.*C1qg[idx]*gamma1qq[idx]*pow(gamma1gg[idx],2)*LF3-1./2.*C1qg[idx]*gamma1qq[idx]*pow(gamma1gg[idx],2)*LQ3+3./2.*C1qg[idx]*gamma1qq[idx]*pow(gamma1gg[idx],2)*LQ2*LF-3./2.*C1qg[idx]*gamma1qq[idx]*pow(gamma1gg[idx],2)*LQ*LF2+4./3.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-4./3.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+4.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-4.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+C1qg[idx]*gamma1qq[idx]*gamma2qq[idx]*LF2+C1qg[idx]*gamma1qq[idx]*gamma2qq[idx]*LQ2-2.*C1qg[idx]*gamma1qq[idx]*gamma2qq[idx]*LQ*LF+1./2.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gg[idx]*LF3-1./2.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gg[idx]*LQ3+3./2.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gg[idx]*LQ2*LF-3./2.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gg[idx]*LQ*LF2+1./6.*C1qg[idx]*pow(gamma1qq[idx],3)*LF3-1./6.*C1qg[idx]*pow(gamma1qq[idx],3)*LQ3+1./2.*C1qg[idx]*pow(gamma1qq[idx],3)*LQ2*LF-1./2.*C1qg[idx]*pow(gamma1qq[idx],3)*LQ*LF2+C2qq[idx]*C1qg[idx]*gamma1gg[idx]*LF-C2qq[idx]*C1qg[idx]*gamma1gg[idx]*LQ+C2qq[idx]*C1qg[idx]*gamma1qq[idx]*LF-C2qq[idx]*C1qg[idx]*gamma1qq[idx]*LQ+C1qq[idx]*C2qg[idx]*gamma1gg[idx]*LF-C1qq[idx]*C2qg[idx]*gamma1gg[idx]*LQ+C1qq[idx]*C2qg[idx]*gamma1qq[idx]*LF-C1qq[idx]*C2qg[idx]*gamma1qq[idx]*LQ+C1qq[idx]*C1qg[idx]*gamma2gg[idx]*LF-C1qq[idx]*C1qg[idx]*gamma2gg[idx]*LQ+1./2.*C1qq[idx]*C1qg[idx]*pow(gamma1gg[idx],2)*LF2+1./2.*C1qq[idx]*C1qg[idx]*pow(gamma1gg[idx],2)*LQ2-C1qq[idx]*C1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF+3./2.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+3./2.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-3.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+C1qq[idx]*C1qg[idx]*gamma2qq[idx]*LF-C1qq[idx]*C1qg[idx]*gamma2qq[idx]*LQ+C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LF2+C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ2-2.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ*LF+1./2.*C1qq[idx]*C1qg[idx]*pow(gamma1qq[idx],2)*LF2+1./2.*C1qq[idx]*C1qg[idx]*pow(gamma1qq[idx],2)*LQ2-C1qq[idx]*C1qg[idx]*pow(gamma1qq[idx],2)*LQ*LF+nf*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-nf*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+3.*nf*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-3.*nf*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+nf*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+nf*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-2.*nf*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+1./2.*beta1*C1qg[idx]*gamma1gg[idx]*LF2-3./2.*beta1*C1qg[idx]*gamma1gg[idx]*LQ2+beta1*C1qg[idx]*gamma1gg[idx]*LQ*LF+1./2.*beta1*C1qg[idx]*gamma1qq[idx]*LF2-3./2.*beta1*C1qg[idx]*gamma1qq[idx]*LQ2+beta1*C1qg[idx]*gamma1qq[idx]*LQ*LF+1./2.*beta0*C2qg[idx]*gamma1gg[idx]*LF2-5./2.*beta0*C2qg[idx]*gamma1gg[idx]*LQ2+2.*beta0*C2qg[idx]*gamma1gg[idx]*LQ*LF+1./2.*beta0*C2qg[idx]*gamma1qq[idx]*LF2-5./2.*beta0*C2qg[idx]*gamma1qq[idx]*LQ2+2.*beta0*C2qg[idx]*gamma1qq[idx]*LQ*LF+beta0*C1qg[idx]*gamma2gg[idx]*LF2-2.*beta0*C1qg[idx]*gamma2gg[idx]*LQ2+beta0*C1qg[idx]*gamma2gg[idx]*LQ*LF+1./2.*beta0*C1qg[idx]*pow(gamma1gg[idx],2)*LF3+beta0*C1qg[idx]*pow(gamma1gg[idx],2)*LQ3-3./2.*beta0*C1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF+3./2.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+3.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3-9./2.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+beta0*C1qg[idx]*gamma2qq[idx]*LF2-2.*beta0*C1qg[idx]*gamma2qq[idx]*LQ2+beta0*C1qg[idx]*gamma2qq[idx]*LQ*LF+beta0*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LF3+2.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ3-3.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ2*LF+1./2.*beta0*C1qg[idx]*pow(gamma1qq[idx],2)*LF3+beta0*C1qg[idx]*pow(gamma1qq[idx],2)*LQ3-3./2.*beta0*C1qg[idx]*pow(gamma1qq[idx],2)*LQ2*LF+1./2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gg[idx]*LF2-5./2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gg[idx]*LQ2+2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gg[idx]*LQ*LF+1./2.*beta0*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*LF2-5./2.*beta0*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*LQ2+2.*beta0*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*LQ*LF+beta0*nf*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+2.*beta0*nf*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3-3.*beta0*nf*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+1./3.*pow(beta0,2)*C1qg[idx]*gamma1gg[idx]*LF3-11./6.*pow(beta0,2)*C1qg[idx]*gamma1gg[idx]*LQ3+pow(beta0,2)*C1qg[idx]*gamma1gg[idx]*LQ2*LF+1./2.*pow(beta0,2)*C1qg[idx]*gamma1gg[idx]*LQ*LF2+1./3.*pow(beta0,2)*C1qg[idx]*gamma1qq[idx]*LF3-11./6.*pow(beta0,2)*C1qg[idx]*gamma1qq[idx]*LQ3+pow(beta0,2)*C1qg[idx]*gamma1qq[idx]*LQ2*LF+1./2.*pow(beta0,2)*C1qg[idx]*gamma1qq[idx]*LQ*LF2+B2*C1qg[idx]*gamma1gg[idx]*LQ2-B2*C1qg[idx]*gamma1gg[idx]*LQ*LF+B2*C1qg[idx]*gamma1qq[idx]*LQ2-B2*C1qg[idx]*gamma1qq[idx]*LQ*LF+B1*C2qg[idx]*gamma1gg[idx]*LQ2-B1*C2qg[idx]*gamma1gg[idx]*LQ*LF+B1*C2qg[idx]*gamma1qq[idx]*LQ2-B1*C2qg[idx]*gamma1qq[idx]*LQ*LF+B1*C1qg[idx]*gamma2gg[idx]*LQ2-B1*C1qg[idx]*gamma2gg[idx]*LQ*LF-1./2.*B1*C1qg[idx]*pow(gamma1gg[idx],2)*LQ3+B1*C1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF-1./2.*B1*C1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF2-3./2.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+3.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-3./2.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+B1*C1qg[idx]*gamma2qq[idx]*LQ2-B1*C1qg[idx]*gamma2qq[idx]*LQ*LF-B1*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ3+2.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ2*LF-B1*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ*LF2-1./2.*B1*C1qg[idx]*pow(gamma1qq[idx],2)*LQ3+B1*C1qg[idx]*pow(gamma1qq[idx],2)*LQ2*LF-1./2.*B1*C1qg[idx]*pow(gamma1qq[idx],2)*LQ*LF2+B1*C1qq[idx]*C1qg[idx]*gamma1gg[idx]*LQ2-B1*C1qq[idx]*C1qg[idx]*gamma1gg[idx]*LQ*LF+B1*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*LQ2-B1*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*LQ*LF-B1*nf*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+2.*B1*nf*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-B1*nf*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+2.*B1*beta0*C1qg[idx]*gamma1gg[idx]*LQ3-3./2.*B1*beta0*C1qg[idx]*gamma1gg[idx]*LQ2*LF-1./2.*B1*beta0*C1qg[idx]*gamma1gg[idx]*LQ*LF2+2.*B1*beta0*C1qg[idx]*gamma1qq[idx]*LQ3-3./2.*B1*beta0*C1qg[idx]*gamma1qq[idx]*LQ2*LF-1./2.*B1*beta0*C1qg[idx]*gamma1qq[idx]*LQ*LF2-1./2.*pow(B1,2)*C1qg[idx]*gamma1gg[idx]*LQ3+1./2.*pow(B1,2)*C1qg[idx]*gamma1gg[idx]*LQ2*LF-1./2.*pow(B1,2)*C1qg[idx]*gamma1qq[idx]*LQ3+1./2.*pow(B1,2)*C1qg[idx]*gamma1qq[idx]*LQ2*LF+1./2.*A2*C1qg[idx]*gamma1gg[idx]*LQ3-1./2.*A2*C1qg[idx]*gamma1gg[idx]*LQ2*LF+1./2.*A2*C1qg[idx]*gamma1qq[idx]*LQ3-1./2.*A2*C1qg[idx]*gamma1qq[idx]*LQ2*LF+1./2.*A1*C2qg[idx]*gamma1gg[idx]*LQ3-1./2.*A1*C2qg[idx]*gamma1gg[idx]*LQ2*LF+1./2.*A1*C2qg[idx]*gamma1qq[idx]*LQ3-1./2.*A1*C2qg[idx]*gamma1qq[idx]*LQ2*LF+1./2.*A1*C1qg[idx]*gamma2gg[idx]*LQ3-1./2.*A1*C1qg[idx]*gamma2gg[idx]*LQ2*LF-1./4.*A1*C1qg[idx]*pow(gamma1gg[idx],2)*LQ4+1./2.*A1*C1qg[idx]*pow(gamma1gg[idx],2)*LQ3*LF-1./4.*A1*C1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF2-3./4.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+3./2.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-3./4.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+1./2.*A1*C1qg[idx]*gamma2qq[idx]*LQ3-1./2.*A1*C1qg[idx]*gamma2qq[idx]*LQ2*LF-1./2.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ4+A1*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ3*LF-1./2.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ2*LF2-1./4.*A1*C1qg[idx]*pow(gamma1qq[idx],2)*LQ4+1./2.*A1*C1qg[idx]*pow(gamma1qq[idx],2)*LQ3*LF-1./4.*A1*C1qg[idx]*pow(gamma1qq[idx],2)*LQ2*LF2+1./2.*A1*C1qq[idx]*C1qg[idx]*gamma1gg[idx]*LQ3-1./2.*A1*C1qq[idx]*C1qg[idx]*gamma1gg[idx]*LQ2*LF+1./2.*A1*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*LQ3-1./2.*A1*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*LQ2*LF-1./2.*A1*nf*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+A1*nf*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-1./2.*A1*nf*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+13./12.*A1*beta0*C1qg[idx]*gamma1gg[idx]*LQ4-5./6.*A1*beta0*C1qg[idx]*gamma1gg[idx]*LQ3*LF-1./4.*A1*beta0*C1qg[idx]*gamma1gg[idx]*LQ2*LF2+13./12.*A1*beta0*C1qg[idx]*gamma1qq[idx]*LQ4-5./6.*A1*beta0*C1qg[idx]*gamma1qq[idx]*LQ3*LF-1./4.*A1*beta0*C1qg[idx]*gamma1qq[idx]*LQ2*LF2-1./2.*A1*B1*C1qg[idx]*gamma1gg[idx]*LQ4+1./2.*A1*B1*C1qg[idx]*gamma1gg[idx]*LQ3*LF-1./2.*A1*B1*C1qg[idx]*gamma1qq[idx]*LQ4+1./2.*A1*B1*C1qg[idx]*gamma1qq[idx]*LQ3*LF-1./8.*pow(A1,2)*C1qg[idx]*gamma1gg[idx]*LQ5+1./8.*pow(A1,2)*C1qg[idx]*gamma1gg[idx]*LQ4*LF-1./8.*pow(A1,2)*C1qg[idx]*gamma1qq[idx]*LQ5+1./8.*pow(A1,2)*C1qg[idx]*gamma1qq[idx]*LQ4*LF;
	    //complex <double> H4st_qq_qg    =+C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2-2.*C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF2+C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2-2.*C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF+C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3-C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF-3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LF2+C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ2-2.*C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ*LF+C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LF2+C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ2-2.*C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ*LF+1./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-1./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+3./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-3./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LF3-C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ3+3.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ2*LF-3.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ*LF2+C2qq[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ+C1qq[idx]*C2qg[idx]*gamma1gq[idx]*LF-C1qq[idx]*C2qg[idx]*gamma1gq[idx]*LQ+C1qq[idx]*C1qg[idx]*gamma2gq[idx]*LF-C1qq[idx]*C1qg[idx]*gamma2gq[idx]*LQ+1./2.*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF2+1./2.*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2-C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+3./2.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+3./2.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2-3.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+1./2.*beta1*C1qg[idx]*gamma1gq[idx]*LF2-3./2.*beta1*C1qg[idx]*gamma1gq[idx]*LQ2+beta1*C1qg[idx]*gamma1gq[idx]*LQ*LF+1./2.*beta0*C2qg[idx]*gamma1gq[idx]*LF2-5./2.*beta0*C2qg[idx]*gamma1gq[idx]*LQ2+2.*beta0*C2qg[idx]*gamma1gq[idx]*LQ*LF+beta0*C1qg[idx]*gamma2gq[idx]*LF2-2.*beta0*C1qg[idx]*gamma2gq[idx]*LQ2+beta0*C1qg[idx]*gamma2gq[idx]*LQ*LF+1./2.*beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3+beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3-3./2.*beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF+3./2.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF3+3.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3-9./2.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF+1./2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LF2-5./2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2+2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ*LF+1./3.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LF3-11./6.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ3+pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ*LF2+B2*C1qg[idx]*gamma1gq[idx]*LQ2-B2*C1qg[idx]*gamma1gq[idx]*LQ*LF+B1*C2qg[idx]*gamma1gq[idx]*LQ2-B1*C2qg[idx]*gamma1gq[idx]*LQ*LF+B1*C1qg[idx]*gamma2gq[idx]*LQ2-B1*C1qg[idx]*gamma2gq[idx]*LQ*LF-1./2.*B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-1./2.*B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2-3./2.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3+3.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF-3./2.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF2+B1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2-B1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ*LF+2.*B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ3-3./2.*B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ*LF2-1./2.*pow(B1,2)*C1qg[idx]*gamma1gq[idx]*LQ3+1./2.*pow(B1,2)*C1qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*A2*C1qg[idx]*gamma1gq[idx]*LQ3-1./2.*A2*C1qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*A1*C2qg[idx]*gamma1gq[idx]*LQ3-1./2.*A1*C2qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*A1*C1qg[idx]*gamma2gq[idx]*LQ3-1./2.*A1*C1qg[idx]*gamma2gq[idx]*LQ2*LF-1./4.*A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+1./2.*A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF-1./4.*A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-3./4.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ4+3./2.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3*LF-3./4.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF2+1./2.*A1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ3-1./2.*A1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2*LF+13./12.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ4-5./6.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ3*LF-1./4.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ2*LF2-1./2.*A1*B1*C1qg[idx]*gamma1gq[idx]*LQ4+1./2.*A1*B1*C1qg[idx]*gamma1gq[idx]*LQ3*LF-1./8.*pow(A1,2)*C1qg[idx]*gamma1gq[idx]*LQ5+1./8.*pow(A1,2)*C1qg[idx]*gamma1gq[idx]*LQ4*LF;
	    //complex <double> H4st_qqp_qg   =+C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2-2.*C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF2+C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2-2.*C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF+C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3-C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF-3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LF2+C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ2-2.*C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ*LF+C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LF2+C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ2-2.*C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ*LF+1./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-1./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+3./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-3./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LF3-C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ3+3.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ2*LF-3.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ*LF2+C2qq[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ+C1qq[idx]*C2qg[idx]*gamma1gq[idx]*LF-C1qq[idx]*C2qg[idx]*gamma1gq[idx]*LQ+C1qq[idx]*C1qg[idx]*gamma2gq[idx]*LF-C1qq[idx]*C1qg[idx]*gamma2gq[idx]*LQ+1./2.*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF2+1./2.*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2-C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+3./2.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+3./2.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2-3.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+1./2.*beta1*C1qg[idx]*gamma1gq[idx]*LF2-3./2.*beta1*C1qg[idx]*gamma1gq[idx]*LQ2+beta1*C1qg[idx]*gamma1gq[idx]*LQ*LF+1./2.*beta0*C2qg[idx]*gamma1gq[idx]*LF2-5./2.*beta0*C2qg[idx]*gamma1gq[idx]*LQ2+2.*beta0*C2qg[idx]*gamma1gq[idx]*LQ*LF+beta0*C1qg[idx]*gamma2gq[idx]*LF2-2.*beta0*C1qg[idx]*gamma2gq[idx]*LQ2+beta0*C1qg[idx]*gamma2gq[idx]*LQ*LF+1./2.*beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3+beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3-3./2.*beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF+3./2.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF3+3.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3-9./2.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF+1./2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LF2-5./2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2+2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ*LF+1./3.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LF3-11./6.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ3+pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ*LF2+B2*C1qg[idx]*gamma1gq[idx]*LQ2-B2*C1qg[idx]*gamma1gq[idx]*LQ*LF+B1*C2qg[idx]*gamma1gq[idx]*LQ2-B1*C2qg[idx]*gamma1gq[idx]*LQ*LF+B1*C1qg[idx]*gamma2gq[idx]*LQ2-B1*C1qg[idx]*gamma2gq[idx]*LQ*LF-1./2.*B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-1./2.*B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2-3./2.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3+3.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF-3./2.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF2+B1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2-B1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ*LF+2.*B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ3-3./2.*B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ*LF2-1./2.*pow(B1,2)*C1qg[idx]*gamma1gq[idx]*LQ3+1./2.*pow(B1,2)*C1qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*A2*C1qg[idx]*gamma1gq[idx]*LQ3-1./2.*A2*C1qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*A1*C2qg[idx]*gamma1gq[idx]*LQ3-1./2.*A1*C2qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*A1*C1qg[idx]*gamma2gq[idx]*LQ3-1./2.*A1*C1qg[idx]*gamma2gq[idx]*LQ2*LF-1./4.*A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+1./2.*A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF-1./4.*A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-3./4.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ4+3./2.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3*LF-3./4.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF2+1./2.*A1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ3-1./2.*A1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2*LF+13./12.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ4-5./6.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ3*LF-1./4.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ2*LF2-1./2.*A1*B1*C1qg[idx]*gamma1gq[idx]*LQ4+1./2.*A1*B1*C1qg[idx]*gamma1gq[idx]*LQ3*LF-1./8.*pow(A1,2)*C1qg[idx]*gamma1gq[idx]*LQ5+1./8.*pow(A1,2)*C1qg[idx]*gamma1gq[idx]*LQ4*LF;
	    //complex <double> H4st_qqbp_qg  =+C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2-2.*C2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF2+C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2-2.*C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF+C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3-C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF-3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LF2+C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ2-2.*C1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ*LF+C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LF2+C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ2-2.*C1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ*LF+1./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-1./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+3./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-3./2.*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LF3-C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ3+3.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ2*LF-3.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ*LF2+C2qq[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ+C1qq[idx]*C2qg[idx]*gamma1gq[idx]*LF-C1qq[idx]*C2qg[idx]*gamma1gq[idx]*LQ+C1qq[idx]*C1qg[idx]*gamma2gq[idx]*LF-C1qq[idx]*C1qg[idx]*gamma2gq[idx]*LQ+1./2.*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF2+1./2.*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2-C1qq[idx]*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+3./2.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+3./2.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2-3.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+1./2.*beta1*C1qg[idx]*gamma1gq[idx]*LF2-3./2.*beta1*C1qg[idx]*gamma1gq[idx]*LQ2+beta1*C1qg[idx]*gamma1gq[idx]*LQ*LF+1./2.*beta0*C2qg[idx]*gamma1gq[idx]*LF2-5./2.*beta0*C2qg[idx]*gamma1gq[idx]*LQ2+2.*beta0*C2qg[idx]*gamma1gq[idx]*LQ*LF+beta0*C1qg[idx]*gamma2gq[idx]*LF2-2.*beta0*C1qg[idx]*gamma2gq[idx]*LQ2+beta0*C1qg[idx]*gamma2gq[idx]*LQ*LF+1./2.*beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3+beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3-3./2.*beta0*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF+3./2.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF3+3.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3-9./2.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF+1./2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LF2-5./2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2+2.*beta0*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ*LF+1./3.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LF3-11./6.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ3+pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*pow(beta0,2)*C1qg[idx]*gamma1gq[idx]*LQ*LF2+B2*C1qg[idx]*gamma1gq[idx]*LQ2-B2*C1qg[idx]*gamma1gq[idx]*LQ*LF+B1*C2qg[idx]*gamma1gq[idx]*LQ2-B1*C2qg[idx]*gamma1gq[idx]*LQ*LF+B1*C1qg[idx]*gamma2gq[idx]*LQ2-B1*C1qg[idx]*gamma2gq[idx]*LQ*LF-1./2.*B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-1./2.*B1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2-3./2.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3+3.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF-3./2.*B1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF2+B1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2-B1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ*LF+2.*B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ3-3./2.*B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ2*LF-1./2.*B1*beta0*C1qg[idx]*gamma1gq[idx]*LQ*LF2-1./2.*pow(B1,2)*C1qg[idx]*gamma1gq[idx]*LQ3+1./2.*pow(B1,2)*C1qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*A2*C1qg[idx]*gamma1gq[idx]*LQ3-1./2.*A2*C1qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*A1*C2qg[idx]*gamma1gq[idx]*LQ3-1./2.*A1*C2qg[idx]*gamma1gq[idx]*LQ2*LF+1./2.*A1*C1qg[idx]*gamma2gq[idx]*LQ3-1./2.*A1*C1qg[idx]*gamma2gq[idx]*LQ2*LF-1./4.*A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+1./2.*A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF-1./4.*A1*C1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2-3./4.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ4+3./2.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3*LF-3./4.*A1*C1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF2+1./2.*A1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ3-1./2.*A1*C1qq[idx]*C1qg[idx]*gamma1gq[idx]*LQ2*LF+13./12.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ4-5./6.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ3*LF-1./4.*A1*beta0*C1qg[idx]*gamma1gq[idx]*LQ2*LF2-1./2.*A1*B1*C1qg[idx]*gamma1gq[idx]*LQ4+1./2.*A1*B1*C1qg[idx]*gamma1gq[idx]*LQ3*LF-1./8.*pow(A1,2)*C1qg[idx]*gamma1gq[idx]*LQ5+1./8.*pow(A1,2)*C1qg[idx]*gamma1gq[idx]*LQ4*LF;
	    //complex <double> H4st_gg_qg    =+2.*C3qg[idx]*gamma1qg[idx]*LF-2.*C3qg[idx]*gamma1qg[idx]*LQ+2.*C2qg[idx]*gamma2qg[idx]*LF-2.*C2qg[idx]*gamma2qg[idx]*LQ+3.*C2qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+3.*C2qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2-6.*C2qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+C2qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+C2qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2-2.*C2qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+2.*C1qg[idx]*gamma3qg[idx]*LF-2.*C1qg[idx]*gamma3qg[idx]*LQ+2.*C1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LF2+2.*C1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ2-6.*C1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ*LF-2.*C1qg[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF-2.*C1qg[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF-2.*C1qg[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF+4.*C1qg[idx]*gamma1qg[idx]*gamma2gg[idx]*LF2+4.*C1qg[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ2-6.*C1qg[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ*LF+7./3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LF3-7./3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3+7.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF-7.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF2-2.*C1qg[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ*LF+2.*C1qg[idx]*gamma1qq[idx]*gamma2qg[idx]*LF2+2.*C1qg[idx]*gamma1qq[idx]*gamma2qg[idx]*LQ2-2.*C1qg[idx]*gamma1qq[idx]*gamma2qg[idx]*LQ*LF+4./3.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF3-4./3.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3+4.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF-4.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2+1./3.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LF3-1./3.*C1qg[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ3+C1qg[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ2*LF-C1qg[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ*LF2+2.*C2qq[idx]*C1qg[idx]*gamma1qg[idx]*LF-2.*C2qq[idx]*C1qg[idx]*gamma1qg[idx]*LQ+2.*C1qq[idx]*C2qg[idx]*gamma1qg[idx]*LF-2.*C1qq[idx]*C2qg[idx]*gamma1qg[idx]*LQ+2.*C1qq[idx]*C1qg[idx]*gamma2qg[idx]*LF-2.*C1qq[idx]*C1qg[idx]*gamma2qg[idx]*LQ+3.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+3.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2-6.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2-2.*C1qq[idx]*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+8./3.*nf*C1qg[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3-8./3.*nf*C1qg[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+8.*nf*C1qg[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF-8.*nf*C1qg[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+beta1*C1qg[idx]*gamma1qg[idx]*LF2-3.*beta1*C1qg[idx]*gamma1qg[idx]*LQ2+2.*beta1*C1qg[idx]*gamma1qg[idx]*LQ*LF+beta0*C2qg[idx]*gamma1qg[idx]*LF2-5.*beta0*C2qg[idx]*gamma1qg[idx]*LQ2+4.*beta0*C2qg[idx]*gamma1qg[idx]*LQ*LF+2.*beta0*C1qg[idx]*gamma2qg[idx]*LF2-4.*beta0*C1qg[idx]*gamma2qg[idx]*LQ2+2.*beta0*C1qg[idx]*gamma2qg[idx]*LQ*LF+3.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LF3+6.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3-9.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF+beta0*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LF3+2.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3-3.*beta0*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF+beta0*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*LF2-5.*beta0*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*LQ2+4.*beta0*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*LQ*LF+2./3.*pow(beta0,2)*C1qg[idx]*gamma1qg[idx]*LF3-11./3.*pow(beta0,2)*C1qg[idx]*gamma1qg[idx]*LQ3+2.*pow(beta0,2)*C1qg[idx]*gamma1qg[idx]*LQ2*LF+pow(beta0,2)*C1qg[idx]*gamma1qg[idx]*LQ*LF2+2.*B2*C1qg[idx]*gamma1qg[idx]*LQ2-2.*B2*C1qg[idx]*gamma1qg[idx]*LQ*LF+2.*B1*C2qg[idx]*gamma1qg[idx]*LQ2-2.*B1*C2qg[idx]*gamma1qg[idx]*LQ*LF+2.*B1*C1qg[idx]*gamma2qg[idx]*LQ2-2.*B1*C1qg[idx]*gamma2qg[idx]*LQ*LF-3.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3+6.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF-3.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2-B1*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3+2.*B1*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF-B1*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF2+2.*B1*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*LQ2-2.*B1*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*LQ*LF+4.*B1*beta0*C1qg[idx]*gamma1qg[idx]*LQ3-3.*B1*beta0*C1qg[idx]*gamma1qg[idx]*LQ2*LF-B1*beta0*C1qg[idx]*gamma1qg[idx]*LQ*LF2-pow(B1,2)*C1qg[idx]*gamma1qg[idx]*LQ3+pow(B1,2)*C1qg[idx]*gamma1qg[idx]*LQ2*LF+A2*C1qg[idx]*gamma1qg[idx]*LQ3-A2*C1qg[idx]*gamma1qg[idx]*LQ2*LF+A1*C2qg[idx]*gamma1qg[idx]*LQ3-A1*C2qg[idx]*gamma1qg[idx]*LQ2*LF+A1*C1qg[idx]*gamma2qg[idx]*LQ3-A1*C1qg[idx]*gamma2qg[idx]*LQ2*LF-3./2.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ4+3.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF-3./2.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2-1./2.*A1*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ4+A1*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF-1./2.*A1*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2+A1*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*LQ3-A1*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*LQ2*LF+13./6.*A1*beta0*C1qg[idx]*gamma1qg[idx]*LQ4-5./3.*A1*beta0*C1qg[idx]*gamma1qg[idx]*LQ3*LF-1./2.*A1*beta0*C1qg[idx]*gamma1qg[idx]*LQ2*LF2-A1*B1*C1qg[idx]*gamma1qg[idx]*LQ4+A1*B1*C1qg[idx]*gamma1qg[idx]*LQ3*LF-1./4.*pow(A1,2)*C1qg[idx]*gamma1qg[idx]*LQ5+1./4.*pow(A1,2)*C1qg[idx]*gamma1qg[idx]*LQ4*LF;
	    //complex <double> H4st_qbg_qg   =+C2qg[idx]*gamma2qqb[idx]*LF-C2qg[idx]*gamma2qqb[idx]*LQ+3./2.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+3./2.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-3.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+C1qg[idx]*gamma3qqb[idx]*LF-C1qg[idx]*gamma3qqb[idx]*LQ+C1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LF2+C1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LQ2-2.*C1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LQ*LF+C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF2+C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2-3.*C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+2.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+2.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2-3.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+5./3.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-5./3.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+5.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-5.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2-C1qg[idx]*gamma1qq[idx]*gamma2qqb[idx]*LQ*LF+4./3.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-4./3.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+4.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-4.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+C1qq[idx]*C1qg[idx]*gamma2qqb[idx]*LF-C1qq[idx]*C1qg[idx]*gamma2qqb[idx]*LQ+3./2.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+3./2.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-3.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+beta0*C1qg[idx]*gamma2qqb[idx]*LF2-2.*beta0*C1qg[idx]*gamma2qqb[idx]*LQ2+beta0*C1qg[idx]*gamma2qqb[idx]*LQ*LF+3./2.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+3.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3-9./2.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+B1*C1qg[idx]*gamma2qqb[idx]*LQ2-B1*C1qg[idx]*gamma2qqb[idx]*LQ*LF-3./2.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+3.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-3./2.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+1./2.*A1*C1qg[idx]*gamma2qqb[idx]*LQ3-1./2.*A1*C1qg[idx]*gamma2qqb[idx]*LQ2*LF-3./4.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+3./2.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-3./4.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2;
	    //complex <double> H4st_qpg_qg   =+C2qg[idx]*gamma2qqp[idx]*LF-C2qg[idx]*gamma2qqp[idx]*LQ+3./2.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+3./2.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-3.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+C1qg[idx]*gamma3qqp[idx]*LF-C1qg[idx]*gamma3qqp[idx]*LQ+C1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LF2+C1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LQ2-2.*C1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LQ*LF+C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF2+C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2-3.*C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+2.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+2.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2-3.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+5./3.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-5./3.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+5.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-5.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2-C1qg[idx]*gamma1qq[idx]*gamma2qqp[idx]*LQ*LF+4./3.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-4./3.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+4.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-4.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+C1qq[idx]*C1qg[idx]*gamma2qqp[idx]*LF-C1qq[idx]*C1qg[idx]*gamma2qqp[idx]*LQ+3./2.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+3./2.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-3.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+beta0*C1qg[idx]*gamma2qqp[idx]*LF2-2.*beta0*C1qg[idx]*gamma2qqp[idx]*LQ2+beta0*C1qg[idx]*gamma2qqp[idx]*LQ*LF+3./2.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+3.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3-9./2.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+B1*C1qg[idx]*gamma2qqp[idx]*LQ2-B1*C1qg[idx]*gamma2qqp[idx]*LQ*LF-3./2.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+3.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-3./2.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+1./2.*A1*C1qg[idx]*gamma2qqp[idx]*LQ3-1./2.*A1*C1qg[idx]*gamma2qqp[idx]*LQ2*LF-3./4.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+3./2.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-3./4.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2;
	    //complex <double> H4st_qbpg_qg  =+C2qg[idx]*gamma2qqbp[idx]*LF-C2qg[idx]*gamma2qqbp[idx]*LQ+3./2.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+3./2.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-3.*C2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+C1qg[idx]*gamma3qqbp[idx]*LF-C1qg[idx]*gamma3qqbp[idx]*LQ+C1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LF2+C1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LQ2-2.*C1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LQ*LF+C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF2+C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2-3.*C1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+2.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+2.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2-3.*C1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+5./3.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3-5./3.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+5.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF-5.*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2-C1qg[idx]*gamma1qq[idx]*gamma2qqbp[idx]*LQ*LF+4./3.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3-4./3.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+4.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-4.*C1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+C1qq[idx]*C1qg[idx]*gamma2qqbp[idx]*LF-C1qq[idx]*C1qg[idx]*gamma2qqbp[idx]*LQ+3./2.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+3./2.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-3.*C1qq[idx]*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+beta0*C1qg[idx]*gamma2qqbp[idx]*LF2-2.*beta0*C1qg[idx]*gamma2qqbp[idx]*LQ2+beta0*C1qg[idx]*gamma2qqbp[idx]*LQ*LF+3./2.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+3.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3-9./2.*beta0*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+B1*C1qg[idx]*gamma2qqbp[idx]*LQ2-B1*C1qg[idx]*gamma2qqbp[idx]*LQ*LF-3./2.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+3.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF-3./2.*B1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+1./2.*A1*C1qg[idx]*gamma2qqbp[idx]*LQ3-1./2.*A1*C1qg[idx]*gamma2qqbp[idx]*LQ2*LF-3./4.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+3./2.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF-3./4.*A1*C1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2;
	    //complex <double> H4st_qbq_qg   =+2.*C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF2+2.*C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2-4.*C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF+C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3-C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF-3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2;
	    //complex <double> H4st_qbqp_qg  =+C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF2+C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2-2.*C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF+C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF2+C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2-2.*C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF+C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3-C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF-3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2;
	    //complex <double> H4st_qbqbp_qg =+C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF2+C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2-2.*C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF+C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF2+C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2-2.*C1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF+C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3-C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF-3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2;
	    //complex <double> H4st_qpqp_qg  =+C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF2+C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2-2.*C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF+C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF2+C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2-2.*C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF+C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3-C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF-3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2;
	    //complex <double> H4st_qpqbp_qg =+2.*C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF2+2.*C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2-4.*C1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF+C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3-C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF-3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2;
	    //complex <double> H4st_qbpqp_qg =+2.*C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF2+2.*C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2-4.*C1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF+C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3-C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF-3.*C1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2;

	    complex <double> H4st_qqb_qqb  = h4st_qqb_qqb  (idx);
	    complex <double> H4st_qg_qqb   = h4st_qg_qqb   (idx);
	    complex <double> H4st_qq_qqb   = h4st_qq_qqb   (idx);
	    complex <double> H4st_qqp_qqb  = h4st_qqp_qqb  (idx);
	    complex <double> H4st_qqbp_qqb = h4st_qqbp_qqb (idx);
	    complex <double> H4st_gg_qqb   = h4st_gg_qqb   (idx);
	    complex <double> H4st_qbg_qqb  = h4st_qbg_qqb  (idx);
	    complex <double> H4st_qpg_qqb  = h4st_qpg_qqb  (idx);
	    complex <double> H4st_qbpg_qqb = h4st_qbpg_qqb (idx);
	    complex <double> H4st_qbq_qqb  = h4st_qbq_qqb  (idx);
	    complex <double> H4st_qbqp_qqb = h4st_qbqp_qqb (idx);
	    complex <double> H4st_qbqbp_qqb= h4st_qbqbp_qqb(idx);
	    complex <double> H4st_qpqp_qqb = h4st_qpqp_qqb (idx);
	    complex <double> H4st_qpqbp_qqb= h4st_qpqbp_qqb(idx);
	    complex <double> H4st_qbpqp_qqb= h4st_qbpqp_qqb(idx);
	    complex <double> H4st_qqb_qg   = h4st_qqb_qg   (idx);
	    complex <double> H4st_qg_qg    = h4st_qg_qg    (idx);
	    complex <double> H4st_qq_qg    = h4st_qq_qg    (idx);
	    complex <double> H4st_qqp_qg   = h4st_qqp_qg   (idx);
	    complex <double> H4st_qqbp_qg  = h4st_qqbp_qg  (idx);
	    complex <double> H4st_gg_qg    = h4st_gg_qg    (idx);
	    complex <double> H4st_qbg_qg   = h4st_qbg_qg   (idx);
	    complex <double> H4st_qpg_qg   = h4st_qpg_qg   (idx);
	    complex <double> H4st_qbpg_qg  = h4st_qbpg_qg  (idx);
	    complex <double> H4st_qbq_qg   = h4st_qbq_qg   (idx);
	    complex <double> H4st_qbqp_qg  = h4st_qbqp_qg  (idx);
	    complex <double> H4st_qbqbp_qg = h4st_qbqbp_qg (idx);
	    complex <double> H4st_qpqp_qg  = h4st_qpqp_qg  (idx);
	    complex <double> H4st_qpqbp_qg = h4st_qpqbp_qg (idx);
	    complex <double> H4st_qbpqp_qg = h4st_qbpqp_qg (idx);
	    complex <double> H4st_qqb_gg=+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2-2.*pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF;
	    complex <double> H4st_qg_gg=+2.*C1qg[idx]*C2qg[idx]*gamma1gq[idx]*LF-2.*C1qg[idx]*C2qg[idx]*gamma1gq[idx]*LQ+pow(C1qg[idx],2)*gamma2gq[idx]*LF-pow(C1qg[idx],2)*gamma2gq[idx]*LQ+3./2.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF2+3./2.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2-3.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+1./2.*pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LF2+1./2.*pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ2-pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+1./2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LF2-5./2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2+2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LQ*LF+B1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2-B1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ*LF+1./2.*A1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ3-1./2.*A1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2*LF;
	    complex <double> H4st_qq_gg=+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2-2.*pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF;
	    complex <double> H4st_qqp_gg=+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2-2.*pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF;
	    complex <double> H4st_qqbp_gg=+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2-2.*pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF;
	    complex <double> H4st_gg_gg=+4.*C1qg[idx]*C2qg[idx]*gamma1gg[idx]*LF-4.*C1qg[idx]*C2qg[idx]*gamma1gg[idx]*LQ+2.*pow(C1qg[idx],2)*gamma2gg[idx]*LF-2.*pow(C1qg[idx],2)*gamma2gg[idx]*LQ+2.*pow(C1qg[idx],2)*pow(gamma1gg[idx],2)*LF2+2.*pow(C1qg[idx],2)*pow(gamma1gg[idx],2)*LQ2-4.*pow(C1qg[idx],2)*pow(gamma1gg[idx],2)*LQ*LF+2.*nf*pow(C1qg[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF2+2.*nf*pow(C1qg[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2-4.*nf*pow(C1qg[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+beta0*pow(C1qg[idx],2)*gamma1gg[idx]*LF2-5.*beta0*pow(C1qg[idx],2)*gamma1gg[idx]*LQ2+4.*beta0*pow(C1qg[idx],2)*gamma1gg[idx]*LQ*LF+2.*B1*pow(C1qg[idx],2)*gamma1gg[idx]*LQ2-2.*B1*pow(C1qg[idx],2)*gamma1gg[idx]*LQ*LF+A1*pow(C1qg[idx],2)*gamma1gg[idx]*LQ3-A1*pow(C1qg[idx],2)*gamma1gg[idx]*LQ2*LF;
	    complex <double> H4st_qbg_gg=+2.*C1qg[idx]*C2qg[idx]*gamma1gq[idx]*LF-2.*C1qg[idx]*C2qg[idx]*gamma1gq[idx]*LQ+pow(C1qg[idx],2)*gamma2gq[idx]*LF-pow(C1qg[idx],2)*gamma2gq[idx]*LQ+3./2.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF2+3./2.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2-3.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+1./2.*pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LF2+1./2.*pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ2-pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+1./2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LF2-5./2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2+2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LQ*LF+B1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2-B1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ*LF+1./2.*A1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ3-1./2.*A1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2*LF;
	    complex <double> H4st_qpg_gg=+2.*C1qg[idx]*C2qg[idx]*gamma1gq[idx]*LF-2.*C1qg[idx]*C2qg[idx]*gamma1gq[idx]*LQ+pow(C1qg[idx],2)*gamma2gq[idx]*LF-pow(C1qg[idx],2)*gamma2gq[idx]*LQ+3./2.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF2+3./2.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2-3.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+1./2.*pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LF2+1./2.*pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ2-pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+1./2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LF2-5./2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2+2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LQ*LF+B1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2-B1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ*LF+1./2.*A1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ3-1./2.*A1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2*LF;
	    complex <double> H4st_qbpg_gg=+2.*C1qg[idx]*C2qg[idx]*gamma1gq[idx]*LF-2.*C1qg[idx]*C2qg[idx]*gamma1gq[idx]*LQ+pow(C1qg[idx],2)*gamma2gq[idx]*LF-pow(C1qg[idx],2)*gamma2gq[idx]*LQ+3./2.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF2+3./2.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2-3.*pow(C1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+1./2.*pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LF2+1./2.*pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ2-pow(C1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+1./2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LF2-5./2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2+2.*beta0*pow(C1qg[idx],2)*gamma1gq[idx]*LQ*LF+B1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2-B1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ*LF+1./2.*A1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ3-1./2.*A1*pow(C1qg[idx],2)*gamma1gq[idx]*LQ2*LF;
	    complex <double> H4st_qbq_gg=+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2-2.*pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF;
	    complex <double> H4st_qbqp_gg=+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2-2.*pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF;
	    complex <double> H4st_qbqbp_gg=+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2-2.*pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF;
	    complex <double> H4st_qpqp_gg=+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2-2.*pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF;
	    complex <double> H4st_qpqbp_gg=+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2-2.*pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF;
	    complex <double> H4st_qbpqp_gg=+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2-2.*pow(C1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF;
	    complex <double> H4st_qg_qq=+C2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+C2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2-2.*C2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+C1qq[idx]*C2qqb[idx]*gamma1qg[idx]*LF-C1qq[idx]*C2qqb[idx]*gamma1qg[idx]*LQ+1./2.*beta0*C2qqb[idx]*gamma1qg[idx]*LF2-5./2.*beta0*C2qqb[idx]*gamma1qg[idx]*LQ2+2.*beta0*C2qqb[idx]*gamma1qg[idx]*LQ*LF+B1*C2qqb[idx]*gamma1qg[idx]*LQ2-B1*C2qqb[idx]*gamma1qg[idx]*LQ*LF+1./2.*A1*C2qqb[idx]*gamma1qg[idx]*LQ3-1./2.*A1*C2qqb[idx]*gamma1qg[idx]*LQ2*LF;
	    complex <double> H4st_qq_qq=+C3qqb[idx]*gamma1qq[idx]*LF-C3qqb[idx]*gamma1qq[idx]*LQ+1./2.*C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+C2qqb[idx]*gamma2qq[idx]*LF-C2qqb[idx]*gamma2qq[idx]*LQ+1./2.*C2qqb[idx]*pow(gamma1qq[idx],2)*LF2+1./2.*C2qqb[idx]*pow(gamma1qq[idx],2)*LQ2-C2qqb[idx]*pow(gamma1qq[idx],2)*LQ*LF+C1qq[idx]*C2qqb[idx]*gamma1qq[idx]*LF-C1qq[idx]*C2qqb[idx]*gamma1qq[idx]*LQ+1./2.*beta0*C2qqb[idx]*gamma1qq[idx]*LF2-5./2.*beta0*C2qqb[idx]*gamma1qq[idx]*LQ2+2.*beta0*C2qqb[idx]*gamma1qq[idx]*LQ*LF+B1*C2qqb[idx]*gamma1qq[idx]*LQ2-B1*C2qqb[idx]*gamma1qq[idx]*LQ*LF+1./2.*A1*C2qqb[idx]*gamma1qq[idx]*LQ3-1./2.*A1*C2qqb[idx]*gamma1qq[idx]*LQ2*LF;
	    complex <double> H4st_gg_qq=+2.*C2qqb[idx]*pow(gamma1qg[idx],2)*LF2+2.*C2qqb[idx]*pow(gamma1qg[idx],2)*LQ2-4.*C2qqb[idx]*pow(gamma1qg[idx],2)*LQ*LF;
	    complex <double> H4st_qbg_qq=+C3qqb[idx]*gamma1qg[idx]*LF-C3qqb[idx]*gamma1qg[idx]*LQ+C2qqb[idx]*gamma2qg[idx]*LF-C2qqb[idx]*gamma2qg[idx]*LQ+1./2.*C2qqb[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+1./2.*C2qqb[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2-C2qqb[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+1./2.*C2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+1./2.*C2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2-C2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+C1qq[idx]*C2qqb[idx]*gamma1qg[idx]*LF-C1qq[idx]*C2qqb[idx]*gamma1qg[idx]*LQ+1./2.*beta0*C2qqb[idx]*gamma1qg[idx]*LF2-5./2.*beta0*C2qqb[idx]*gamma1qg[idx]*LQ2+2.*beta0*C2qqb[idx]*gamma1qg[idx]*LQ*LF+B1*C2qqb[idx]*gamma1qg[idx]*LQ2-B1*C2qqb[idx]*gamma1qg[idx]*LQ*LF+1./2.*A1*C2qqb[idx]*gamma1qg[idx]*LQ3-1./2.*A1*C2qqb[idx]*gamma1qg[idx]*LQ2*LF;
	    complex <double> H4st_qbq_qq=+2.*C2qqb[idx]*gamma2qqb[idx]*LF-2.*C2qqb[idx]*gamma2qqb[idx]*LQ+C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-2.*C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF;
	    complex <double> H4st_qbqp_qq=+C2qqb[idx]*gamma2qqbp[idx]*LF-C2qqb[idx]*gamma2qqbp[idx]*LQ+1./2.*C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF;
	    complex <double> H4st_qbqbp_qq=+C2qqb[idx]*gamma2qqp[idx]*LF-C2qqb[idx]*gamma2qqp[idx]*LQ+1./2.*C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF;
	    complex <double> H4st_qg_qqp=+C2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+C2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2-2.*C2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+C1qq[idx]*C2qqbp[idx]*gamma1qg[idx]*LF-C1qq[idx]*C2qqbp[idx]*gamma1qg[idx]*LQ+1./2.*beta0*C2qqbp[idx]*gamma1qg[idx]*LF2-5./2.*beta0*C2qqbp[idx]*gamma1qg[idx]*LQ2+2.*beta0*C2qqbp[idx]*gamma1qg[idx]*LQ*LF+B1*C2qqbp[idx]*gamma1qg[idx]*LQ2-B1*C2qqbp[idx]*gamma1qg[idx]*LQ*LF+1./2.*A1*C2qqbp[idx]*gamma1qg[idx]*LQ3-1./2.*A1*C2qqbp[idx]*gamma1qg[idx]*LQ2*LF;
	    complex <double> H4st_qqp_qqp=+C3qqbp[idx]*gamma1qq[idx]*LF-C3qqbp[idx]*gamma1qq[idx]*LQ+1./2.*C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+C2qqbp[idx]*gamma2qq[idx]*LF-C2qqbp[idx]*gamma2qq[idx]*LQ+1./2.*C2qqbp[idx]*pow(gamma1qq[idx],2)*LF2+1./2.*C2qqbp[idx]*pow(gamma1qq[idx],2)*LQ2-C2qqbp[idx]*pow(gamma1qq[idx],2)*LQ*LF+C1qq[idx]*C2qqbp[idx]*gamma1qq[idx]*LF-C1qq[idx]*C2qqbp[idx]*gamma1qq[idx]*LQ+1./2.*beta0*C2qqbp[idx]*gamma1qq[idx]*LF2-5./2.*beta0*C2qqbp[idx]*gamma1qq[idx]*LQ2+2.*beta0*C2qqbp[idx]*gamma1qq[idx]*LQ*LF+B1*C2qqbp[idx]*gamma1qq[idx]*LQ2-B1*C2qqbp[idx]*gamma1qq[idx]*LQ*LF+1./2.*A1*C2qqbp[idx]*gamma1qq[idx]*LQ3-1./2.*A1*C2qqbp[idx]*gamma1qq[idx]*LQ2*LF;
	    complex <double> H4st_gg_qqp=+2.*C2qqbp[idx]*pow(gamma1qg[idx],2)*LF2+2.*C2qqbp[idx]*pow(gamma1qg[idx],2)*LQ2-4.*C2qqbp[idx]*pow(gamma1qg[idx],2)*LQ*LF;
	    complex <double> H4st_qbpg_qqp=+C3qqbp[idx]*gamma1qg[idx]*LF-C3qqbp[idx]*gamma1qg[idx]*LQ+C2qqbp[idx]*gamma2qg[idx]*LF-C2qqbp[idx]*gamma2qg[idx]*LQ+1./2.*C2qqbp[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+1./2.*C2qqbp[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2-C2qqbp[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+1./2.*C2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+1./2.*C2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2-C2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+C1qq[idx]*C2qqbp[idx]*gamma1qg[idx]*LF-C1qq[idx]*C2qqbp[idx]*gamma1qg[idx]*LQ+1./2.*beta0*C2qqbp[idx]*gamma1qg[idx]*LF2-5./2.*beta0*C2qqbp[idx]*gamma1qg[idx]*LQ2+2.*beta0*C2qqbp[idx]*gamma1qg[idx]*LQ*LF+B1*C2qqbp[idx]*gamma1qg[idx]*LQ2-B1*C2qqbp[idx]*gamma1qg[idx]*LQ*LF+1./2.*A1*C2qqbp[idx]*gamma1qg[idx]*LQ3-1./2.*A1*C2qqbp[idx]*gamma1qg[idx]*LQ2*LF;
	    complex <double> H4st_qbqp_qqp=+C2qqbp[idx]*gamma2qqb[idx]*LF-C2qqbp[idx]*gamma2qqb[idx]*LQ+1./2.*C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF;
	    complex <double> H4st_qpqp_qqp=+C2qqbp[idx]*gamma2qqp[idx]*LF-C2qqbp[idx]*gamma2qqp[idx]*LQ+1./2.*C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF;
	    complex <double> H4st_qbpqp_qqp=+2.*C2qqbp[idx]*gamma2qqbp[idx]*LF-2.*C2qqbp[idx]*gamma2qqbp[idx]*LQ+C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-2.*C2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF;
	    complex <double> H4st_qg_qqbp=+C2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+C2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2-2.*C2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+C1qq[idx]*C2qqp[idx]*gamma1qg[idx]*LF-C1qq[idx]*C2qqp[idx]*gamma1qg[idx]*LQ+1./2.*beta0*C2qqp[idx]*gamma1qg[idx]*LF2-5./2.*beta0*C2qqp[idx]*gamma1qg[idx]*LQ2+2.*beta0*C2qqp[idx]*gamma1qg[idx]*LQ*LF+B1*C2qqp[idx]*gamma1qg[idx]*LQ2-B1*C2qqp[idx]*gamma1qg[idx]*LQ*LF+1./2.*A1*C2qqp[idx]*gamma1qg[idx]*LQ3-1./2.*A1*C2qqp[idx]*gamma1qg[idx]*LQ2*LF;
	    complex <double> H4st_qqbp_qqbp=+C3qqp[idx]*gamma1qq[idx]*LF-C3qqp[idx]*gamma1qq[idx]*LQ+1./2.*C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+C2qqp[idx]*gamma2qq[idx]*LF-C2qqp[idx]*gamma2qq[idx]*LQ+1./2.*C2qqp[idx]*pow(gamma1qq[idx],2)*LF2+1./2.*C2qqp[idx]*pow(gamma1qq[idx],2)*LQ2-C2qqp[idx]*pow(gamma1qq[idx],2)*LQ*LF+C1qq[idx]*C2qqp[idx]*gamma1qq[idx]*LF-C1qq[idx]*C2qqp[idx]*gamma1qq[idx]*LQ+1./2.*beta0*C2qqp[idx]*gamma1qq[idx]*LF2-5./2.*beta0*C2qqp[idx]*gamma1qq[idx]*LQ2+2.*beta0*C2qqp[idx]*gamma1qq[idx]*LQ*LF+B1*C2qqp[idx]*gamma1qq[idx]*LQ2-B1*C2qqp[idx]*gamma1qq[idx]*LQ*LF+1./2.*A1*C2qqp[idx]*gamma1qq[idx]*LQ3-1./2.*A1*C2qqp[idx]*gamma1qq[idx]*LQ2*LF;
	    complex <double> H4st_gg_qqbp=+2.*C2qqp[idx]*pow(gamma1qg[idx],2)*LF2+2.*C2qqp[idx]*pow(gamma1qg[idx],2)*LQ2-4.*C2qqp[idx]*pow(gamma1qg[idx],2)*LQ*LF;
	    complex <double> H4st_qpg_qqbp=+C3qqp[idx]*gamma1qg[idx]*LF-C3qqp[idx]*gamma1qg[idx]*LQ+C2qqp[idx]*gamma2qg[idx]*LF-C2qqp[idx]*gamma2qg[idx]*LQ+1./2.*C2qqp[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+1./2.*C2qqp[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2-C2qqp[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+1./2.*C2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+1./2.*C2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2-C2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+C1qq[idx]*C2qqp[idx]*gamma1qg[idx]*LF-C1qq[idx]*C2qqp[idx]*gamma1qg[idx]*LQ+1./2.*beta0*C2qqp[idx]*gamma1qg[idx]*LF2-5./2.*beta0*C2qqp[idx]*gamma1qg[idx]*LQ2+2.*beta0*C2qqp[idx]*gamma1qg[idx]*LQ*LF+B1*C2qqp[idx]*gamma1qg[idx]*LQ2-B1*C2qqp[idx]*gamma1qg[idx]*LQ*LF+1./2.*A1*C2qqp[idx]*gamma1qg[idx]*LQ3-1./2.*A1*C2qqp[idx]*gamma1qg[idx]*LQ2*LF;
	    complex <double> H4st_qbqbp_qqbp=+C2qqp[idx]*gamma2qqb[idx]*LF-C2qqp[idx]*gamma2qqb[idx]*LQ+1./2.*C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF;
	    complex <double> H4st_qpqp_qqbp=+C2qqp[idx]*gamma2qqbp[idx]*LF-C2qqp[idx]*gamma2qqbp[idx]*LQ+1./2.*C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+1./2.*C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF;
	    complex <double> H4st_qpqbp_qqbp=+2.*C2qqp[idx]*gamma2qqp[idx]*LF-2.*C2qqp[idx]*gamma2qqp[idx]*LQ+C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2-2.*C2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF;
	    complex <double> H4st_qq_qbg=+C2qqb[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qqb[idx]*C1qg[idx]*gamma1gq[idx]*LQ;
	    complex <double> H4st_gg_qbg=+2.*C2qqb[idx]*C1qg[idx]*gamma1qg[idx]*LF-2.*C2qqb[idx]*C1qg[idx]*gamma1qg[idx]*LQ;
	    complex <double> H4st_qbg_qbg=+C2qqb[idx]*C1qg[idx]*gamma1gg[idx]*LF-C2qqb[idx]*C1qg[idx]*gamma1gg[idx]*LQ;
	    complex <double> H4st_qbq_qbg=+2.*C2qqb[idx]*C1qg[idx]*gamma1gq[idx]*LF-2.*C2qqb[idx]*C1qg[idx]*gamma1gq[idx]*LQ;
	    complex <double> H4st_qbqp_qbg=+C2qqb[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qqb[idx]*C1qg[idx]*gamma1gq[idx]*LQ;
	    complex <double> H4st_qbqbp_qbg=+C2qqb[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qqb[idx]*C1qg[idx]*gamma1gq[idx]*LQ;
	    complex <double> H4st_qqbp_qpg=+C2qqp[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qqp[idx]*C1qg[idx]*gamma1gq[idx]*LQ;
	    complex <double> H4st_gg_qpg=+2.*C2qqp[idx]*C1qg[idx]*gamma1qg[idx]*LF-2.*C2qqp[idx]*C1qg[idx]*gamma1qg[idx]*LQ;
	    complex <double> H4st_qpg_qpg=+C2qqp[idx]*C1qg[idx]*gamma1gg[idx]*LF-C2qqp[idx]*C1qg[idx]*gamma1gg[idx]*LQ;
	    complex <double> H4st_qbqbp_qpg=+C2qqp[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qqp[idx]*C1qg[idx]*gamma1gq[idx]*LQ;
	    complex <double> H4st_qpqp_qpg=+C2qqp[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qqp[idx]*C1qg[idx]*gamma1gq[idx]*LQ;
	    complex <double> H4st_qpqbp_qpg=+2.*C2qqp[idx]*C1qg[idx]*gamma1gq[idx]*LF-2.*C2qqp[idx]*C1qg[idx]*gamma1gq[idx]*LQ;
	    complex <double> H4st_qqp_qbpg=+C2qqbp[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qqbp[idx]*C1qg[idx]*gamma1gq[idx]*LQ;
	    complex <double> H4st_gg_qbpg=+2.*C2qqbp[idx]*C1qg[idx]*gamma1qg[idx]*LF-2.*C2qqbp[idx]*C1qg[idx]*gamma1qg[idx]*LQ;
	    complex <double> H4st_qbpg_qbpg=+C2qqbp[idx]*C1qg[idx]*gamma1gg[idx]*LF-C2qqbp[idx]*C1qg[idx]*gamma1gg[idx]*LQ;
	    complex <double> H4st_qbqp_qbpg=+C2qqbp[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qqbp[idx]*C1qg[idx]*gamma1gq[idx]*LQ;
	    complex <double> H4st_qpqp_qbpg=+C2qqbp[idx]*C1qg[idx]*gamma1gq[idx]*LF-C2qqbp[idx]*C1qg[idx]*gamma1gq[idx]*LQ;
	    complex <double> H4st_qbpqp_qbpg=+2.*C2qqbp[idx]*C1qg[idx]*gamma1gq[idx]*LF-2.*C2qqbp[idx]*C1qg[idx]*gamma1gq[idx]*LQ;

	    //Mixed LR*LQF variations
	    H4st_qqb_qqb  += -3.*beta0*H3st_qqb_qqb[idx]*LR  -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qqb_qqb[idx] - (beta2*LR + 5./2.*beta1*beta0*LR2 + pow(beta0,3)*LR3)*H1st_qqb_qqb[idx];
	    H4st_qg_qqb   += -3.*beta0*H3st_qg_qqb[idx]*LR   -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qg_qqb[idx]  - (beta2*LR + 5./2.*beta1*beta0*LR2 + pow(beta0,3)*LR3)*H1st_qg_qqb[idx];
	    H4st_qq_qqb   += -3.*beta0*H3st_qq_qqb[idx]*LR   -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qq_qqb[idx];
	    H4st_qqp_qqb  += -3.*beta0*H3st_qqp_qqb[idx]*LR  -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qqp_qqb[idx];
	    H4st_qqbp_qqb += -3.*beta0*H3st_qqbp_qqb[idx]*LR -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qqbp_qqb[idx];
	    H4st_gg_qqb   += -3.*beta0*H3st_gg_qqb[idx]*LR   -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_gg_qqb[idx];
	    H4st_qbg_qqb  += -3.*beta0*H3st_qbg_qqb[idx]*LR;
	    H4st_qpg_qqb  += -3.*beta0*H3st_qpg_qqb[idx]*LR;
	    H4st_qbpg_qqb += -3.*beta0*H3st_qbpg_qqb[idx]*LR;

	    H4st_qqb_qg  += -3.*beta0*H3st_qqb_qg[idx]*LR  -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qqb_qg[idx];
	    H4st_qg_qg   += -3.*beta0*H3st_qg_qg[idx]*LR   -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qg_qg[idx];
	    H4st_qq_qg   += -3.*beta0*H3st_qq_qg[idx]*LR   -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qq_qg[idx];
	    H4st_qqp_qg  += -3.*beta0*H3st_qqp_qg[idx]*LR  -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qqp_qg[idx];
	    H4st_qqbp_qg += -3.*beta0*H3st_qqbp_qg[idx]*LR -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qqbp_qg[idx];
	    H4st_gg_qg   += -3.*beta0*H3st_gg_qg[idx]*LR   -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_gg_qg[idx];
	    H4st_qbg_qg  += -3.*beta0*H3st_qbg_qg[idx]*LR;
	    H4st_qpg_qg  += -3.*beta0*H3st_qpg_qg[idx]*LR;
	    H4st_qbpg_qg += -3.*beta0*H3st_qbpg_qg[idx]*LR;

	    H4st_qg_qq   += -3.*beta0*H3st_qg_qq[idx]*LR;
	    H4st_qq_qq   += -3.*beta0*H3st_qq_qq[idx]*LR;
	    H4st_qbg_qq  += -3.*beta0*H3st_qbg_qq[idx]*LR;

	    H4st_qg_qqp   += -3.*beta0*H3st_qg_qqp[idx]*LR;
	    H4st_qqp_qqp  += -3.*beta0*H3st_qqp_qqp[idx]*LR;
	    H4st_qbpg_qqp += -3.*beta0*H3st_qbpg_qqp[idx]*LR;

	    H4st_qg_qqbp   += -3.*beta0*H3st_qg_qqbp[idx]*LR;
	    H4st_qqbp_qqbp += -3.*beta0*H3st_qqbp_qqbp[idx]*LR;
	    H4st_qpg_qqbp  += -3.*beta0*H3st_qpg_qqbp[idx]*LR;

	    H4st_qg_gg   += -3.*beta0*H3st_qg_gg[idx]*LR;
	    H4st_gg_gg   += -3.*beta0*H3st_gg_gg[idx]*LR;
	    H4st_qbg_gg  += -3.*beta0*H3st_qbg_gg[idx]*LR;
	    H4st_qpg_gg  += -3.*beta0*H3st_qpg_gg[idx]*LR;
	    H4st_qbpg_gg += -3.*beta0*H3st_qbpg_gg[idx]*LR;
	    
	    qqb_eqqb[idx]  += as4*H4st_qqb_qqb;
	    qg_eqqb[idx]   += as4*H4st_qg_qqb;
	    qq_eqqb[idx]   += as4*H4st_qq_qqb;
	    qqp_eqqb[idx]  += as4*H4st_qqp_qqb;
	    qqbp_eqqb[idx] += as4*H4st_qqbp_qqb;
	    gg_eqqb[idx]   += as4*H4st_gg_qqb;
	    qbg_eqqb[idx]  += as4*H4st_qbg_qqb;
	    qpg_eqqb[idx]  += as4*H4st_qpg_qqb;
	    qbpg_eqqb[idx] += as4*H4st_qbpg_qqb;
	    qbq_eqqb[idx]  += as4*H4st_qbq_qqb;
	    qpqbp_eqqb[idx]+= as4*H4st_qpqbp_qqb;
	    qbpqp_eqqb[idx]+= as4*H4st_qbpqp_qqb;
	    qbqp_eqqb[idx] += as4*H4st_qbqp_qqb;
	    qbqbp_eqqb[idx]+= as4*H4st_qbqbp_qqb;
	    qpqp_eqqb[idx] += as4*H4st_qpqp_qqb;

	    qqb_eqg[idx]   += as4*H4st_qqb_qg;
	    qg_eqg[idx]    += as4*H4st_qg_qg;
	    qq_eqg[idx]    += as4*H4st_qq_qg;
	    qqp_eqg[idx]   += as4*H4st_qqp_qg;
	    qqbp_eqg[idx]  += as4*H4st_qqbp_qg;
	    gg_eqg[idx]    += as4*H4st_gg_qg;
	    qbg_eqg[idx]   += as4*H4st_qbg_qg;
	    qpg_eqg[idx]   += as4*H4st_qpg_qg;
	    qbpg_eqg[idx]  += as4*H4st_qbpg_qg;
	    qbq_eqg[idx]   += as4*H4st_qbq_qg;
	    qbqp_eqg[idx]  += as4*H4st_qbqp_qg;
	    qbqbp_eqg[idx] += as4*H4st_qbqbp_qg;
	    qpqp_eqg[idx]  += as4*H4st_qpqp_qg;
	    qpqbp_eqg[idx] += as4*H4st_qpqbp_qg;
	    qbpqp_eqg[idx] += as4*H4st_qbpqp_qg;
		
	    qg_eqq[idx]	  += as4*H4st_qg_qq;
	    qq_eqq[idx]	  += as4*H4st_qq_qq;
	    gg_eqq[idx]	  += as4*H4st_gg_qq;
	    qbg_eqq[idx]  += as4*H4st_qbg_qq;
	    qbq_eqq[idx]  += as4*H4st_qbq_qq;
	    qbqp_eqq[idx] += as4*H4st_qbqp_qq;
	    qbqbp_eqq[idx]+= as4*H4st_qbqbp_qq;

	    qg_eqqp[idx]   += as4*H4st_qg_qqp;
	    qqp_eqqp[idx]  += as4*H4st_qqp_qqp;
	    gg_eqqp[idx]   += as4*H4st_gg_qqp;
	    qbpg_eqqp[idx] += as4*H4st_qbpg_qqp;
	    qbqp_eqqp[idx] += as4*H4st_qbqp_qqp;
	    qpqp_eqqp[idx] += as4*H4st_qpqp_qqp;
	    qbpqp_eqqp[idx]+= as4*H4st_qbpqp_qqp;
	    
	    qg_eqqbp[idx]   += as4*H4st_qg_qqbp;
	    qqbp_eqqbp[idx] += as4*H4st_qqbp_qqbp;
	    gg_eqqbp[idx]   += as4*H4st_gg_qqbp;
	    qpg_eqqbp[idx]  += as4*H4st_qpg_qqbp;
	    qbqbp_eqqbp[idx]+= as4*H4st_qbqbp_qqbp;
	    qpqp_eqqbp[idx] += as4*H4st_qpqp_qqbp;
	    qpqbp_eqqbp[idx]+= as4*H4st_qpqbp_qqbp;

	    qqb_egg[idx]  += as4*H4st_qqb_gg;
	    qg_egg[idx]	  += as4*H4st_qg_gg;
	    qq_egg[idx]	  += as4*H4st_qq_gg;
	    qqp_egg[idx]  += as4*H4st_qqp_gg;
	    qqbp_egg[idx] += as4*H4st_qqbp_gg;
	    gg_egg[idx]	  += as4*H4st_gg_gg;
	    qbg_egg[idx]  += as4*H4st_qbg_gg;
	    qpg_egg[idx]  += as4*H4st_qpg_gg;
	    qbpg_egg[idx] += as4*H4st_qbpg_gg;
	    qbq_egg[idx]  += as4*H4st_qbq_gg;
	    qbqp_egg[idx] += as4*H4st_qbqp_gg;
	    qbqbp_egg[idx]+= as4*H4st_qbqbp_gg;
	    qpqp_egg[idx] += as4*H4st_qpqp_gg;
	    qpqbp_egg[idx]+= as4*H4st_qpqbp_gg;
	    qbpqp_egg[idx]+= as4*H4st_qbpqp_gg;

	    qq_eqbg[idx]    += as4*H4st_qq_qbg;
	    gg_eqbg[idx]    += as4*H4st_gg_qbg;
	    qbg_eqbg[idx]   += as4*H4st_qbg_qbg;
	    qbq_eqbg[idx]   += as4*H4st_qbq_qbg;
	    qbqp_eqbg[idx]  += as4*H4st_qbqp_qbg;
	    qbqbp_eqbg[idx] += as4*H4st_qbqbp_qbg;
	               
	    qqbp_eqpg[idx]  += as4*H4st_qqbp_qpg;
	    gg_eqpg[idx]    += as4*H4st_gg_qpg;
	    qpg_eqpg[idx]   += as4*H4st_qpg_qpg;
	    qbqbp_eqpg[idx] += as4*H4st_qbqbp_qpg;
	    qpqp_eqpg[idx]  += as4*H4st_qpqp_qpg;
	    qpqbp_eqpg[idx] += as4*H4st_qpqbp_qpg;
	               
	    qqp_eqbpg[idx]  += as4*H4st_qqp_qbpg;
	    gg_eqbpg[idx]   += as4*H4st_gg_qbpg;
	    qbpg_eqbpg[idx] += as4*H4st_qbpg_qbpg;
	    qbqp_eqbpg[idx] += as4*H4st_qbqp_qbpg;
	    qpqp_eqbpg[idx] += as4*H4st_qpqp_qbpg;
	    qbpqp_eqbpg[idx]+= as4*H4st_qbpqp_qbpg;
	  }
    }
}
